/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <string.h>
#include <iostream>
#include <fstream>

#include "DebugOutput.hpp"

#include <JobProperties.hpp>

static ostream *vostreamErr_d = &cerr;

ostream& DebugOutput::
getErrorStream ()
{
   return *vostreamErr_d;
}

void DebugOutput::
setErrorStream (FILE *pFile)
{
   // @TBD store and clean up new filebuf
#ifdef GCC_VER_3
   vostreamErr_d = new ostream (new filebuf (pFile, ios::binary | ios::out));
#else
   vostreamErr_d = new ostream (new filebuf (fileno (pFile)));
#endif
}

void DebugOutput::
applyAllDebugOutput (char *pszJobProperties)
{
#ifndef RETAIL
   if (  pszJobProperties
      && *pszJobProperties
      )
   {
      JobProperties jobProp (pszJobProperties);

      while (jobProp.hasKeyNameLeft ("debugoutput"))
      {
         setDebugOutput (jobProp.getCurrentValue ());

         jobProp.getNextKey ("debugoutput");
      }
   }
#endif
}

static bool vfOutputDeviceBlitter_d         = false;
static bool vfOutputDeviceDither_d          = false;
static bool vfOutputDeviceInstance_d        = false;
static bool vfOutputDevicePrintMode_d       = false;
static bool vfOutputDeviceTester_d          = false;
static bool vfOutputGplCompression_d        = false;
static bool vfOutputGplDitherInstance_d     = false;
static bool vfOutputOmni_d                  = false;
static bool vfOutputOmniInterface_d         = false;
static bool vfOutputOmniProxy_d             = false;
static bool vfOutputPrintDevice_d           = false;
static bool vfOutputBlitter_d               = false;
static bool vfOutputInstance_d              = false;
static bool vfOutputOmniPDCProxy_d          = false;
static bool vfOutputOmniServer_d            = false;
static bool vfOutputPrinterCommand_d        = false;
static bool vfOutputPluggableInstance_d     = false;
static bool vfOutputPluggableBlitter_d      = false;
static bool vfOutputDriverInfo_d            = false;
static bool vfOutputPDCBlitter_d            = false;
static bool vfOutputPDCBlitterClient_d      = false;
static bool vfOutputDeviceString_d          = false;
static bool vfOutputUPDFDevice_d            = false;
static bool vfOutputUPDFDeviceOrientation_d = false;
static bool vfOutputUPDFDeviceForm_d        = false;
static bool vfOutputUPDFDeviceTray_d        = false;
static bool vfOutputUPDFDeviceMedia_d       = false;
static bool vfOutputUPDFDevicePrintMode_d   = false;
static bool vfOutputUPDFDeviceResolution_d  = false;
static bool vfOutputUPDFDeviceInstance_d    = false;
static bool vfOutputUPDFDeviceBlitter_d     = false;

void DebugOutput::
setDebugOutput (char *pszValue)
{
   if (0 == strcasecmp ("DeviceBlitter", pszValue))
   {
      setOutputDeviceBlitter (true);
   }
   else if (0 == strcasecmp ("DeviceDither", pszValue))
   {
      setOutputDeviceDither (true);
   }
   else if (0 == strcasecmp ("DeviceInstance", pszValue))
   {
      setOutputDeviceInstance (true);
   }
   else if (0 == strcasecmp ("DevicePrintMode", pszValue))
   {
      setOutputDevicePrintMode (true);
   }
   else if (0 == strcasecmp ("DeviceTester", pszValue))
   {
      setOutputDeviceTester (true);
   }
   else if (0 == strcasecmp ("GplCompression", pszValue))
   {
      setOutputGplCompression (true);
   }
   else if (0 == strcasecmp ("GplDitherInstance", pszValue))
   {
      setOutputGplDitherInstance (true);
   }
   else if (0 == strcasecmp ("Omni", pszValue))
   {
      setOutputOmni (true);
   }
   else if (0 == strcasecmp ("OmniInterface", pszValue))
   {
      setOutputOmniInterface (true);
   }
   else if (0 == strcasecmp ("OmniProxy", pszValue))
   {
      setOutputOmniProxy (true);
   }
   else if (0 == strcasecmp ("PrintDevice", pszValue))
   {
      setOutputPrintDevice (true);
   }
   else if (0 == strcasecmp ("Blitter", pszValue))
   {
      setOutputBlitter (true);
   }
   else if (0 == strcasecmp ("Instance", pszValue))
   {
      setOutputInstance (true);
   }
   else if (0 == strcasecmp ("OmniPDCProxy", pszValue))
   {
      setOutputOmniPDCProxy (true);
   }
   else if (0 == strcasecmp ("OmniServer", pszValue))
   {
      setOutputOmniServer (true);
   }
   else if (0 == strcasecmp ("PrinterCommand", pszValue))
   {
      setOutputPrinterCommand (true);
   }
   else if (0 == strcasecmp ("PluggableInstance", pszValue))
   {
      setOutputPluggableInstance (true);
   }
   else if (0 == strcasecmp ("PluggableBlitter", pszValue))
   {
      setOutputPluggableBlitter (true);
   }
   else if (0 == strcasecmp ("DriverInfo", pszValue))
   {
      setOutputDriverInfo (true);
   }
   else if (0 == strcasecmp ("PDCBlitter", pszValue))
   {
      setOutputPDCBlitter (true);
   }
   else if (0 == strcasecmp ("PDCBlitterClient", pszValue))
   {
      setOutputPDCBlitterClient (true);
   }
   else if (0 == strcasecmp ("DeviceString", pszValue))
   {
      setOutputDeviceString (true);
   }
   else if (0 == strcasecmp ("UPDFDevice", pszValue))
   {
      setOutputUPDFDevice (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceOrientation", pszValue))
   {
      setOutputUPDFDeviceOrientation (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceForm", pszValue))
   {
      setOutputUPDFDeviceForm (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceTray", pszValue))
   {
      setOutputUPDFDeviceTray (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceMedia", pszValue))
   {
      setOutputUPDFDeviceMedia (true);
   }
   else if (0 == strcasecmp ("UPDFDevicePrintMode", pszValue))
   {
      setOutputUPDFDevicePrintMode (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceResolution", pszValue))
   {
      setOutputUPDFDeviceResolution (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceInstance", pszValue))
   {
      setOutputUPDFDeviceInstance (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceBlitter", pszValue))
   {
      setOutputUPDFDeviceBlitter (true);
   }
   else if (0 == strcasecmp ("all", pszValue))
   {
      setOutputDeviceBlitter         (true);
      setOutputDeviceDither          (true);
      setOutputDeviceInstance        (true);
      setOutputDevicePrintMode       (true);
      setOutputDeviceTester          (true);
      setOutputGplCompression        (true);
      setOutputGplDitherInstance     (true);
      setOutputOmni                  (true);
      setOutputOmniInterface         (true);
      setOutputOmniProxy             (true);
      setOutputPrintDevice           (true);
      setOutputBlitter               (true);
      setOutputInstance              (true);
      setOutputOmniPDCProxy          (true);
      setOutputOmniServer            (true);
      setOutputPrinterCommand        (true);
      setOutputPluggableInstance     (true);
      setOutputPluggableBlitter      (true);
      setOutputDriverInfo            (true);
      setOutputPDCBlitter            (true);
      setOutputPDCBlitterClient      (true);
      setOutputDeviceString          (true);
      setOutputUPDFDevice            (true);
      setOutputUPDFDeviceOrientation (true);
      setOutputUPDFDeviceForm        (true);
      setOutputUPDFDeviceTray        (true);
      setOutputUPDFDeviceMedia       (true);
      setOutputUPDFDevicePrintMode   (true);
      setOutputUPDFDeviceResolution  (true);
      setOutputUPDFDeviceInstance    (true);
      setOutputUPDFDeviceBlitter     (true);
   }
   else if (0 == strcasecmp ("PDC", pszValue))
   {
      setOutputOmniPDCProxy     (true);
      setOutputOmniServer       (true);
      setOutputPrinterCommand   (true);
      setOutputPDCBlitter       (true);
      setOutputPDCBlitterClient (true);
   }
   else if (0 == strcasecmp ("Pluggable", pszValue))
   {
      setOutputPluggableInstance (true);
      setOutputPluggableBlitter  (true);
   }
   else if (0 == strcasecmp ("UPDF", pszValue))
   {
      setOutputUPDFDevice            (true);
      setOutputUPDFDeviceOrientation (true);
      setOutputUPDFDeviceForm        (true);
      setOutputUPDFDeviceTray        (true);
      setOutputUPDFDeviceMedia       (true);
      setOutputUPDFDevicePrintMode   (true);
      setOutputUPDFDeviceResolution  (true);
      setOutputUPDFDeviceInstance    (true);
      setOutputUPDFDeviceBlitter     (true);
   }
}

bool DebugOutput::
shouldOutputDeviceBlitter ()
{
   return vfOutputDeviceBlitter_d;
}

bool DebugOutput::
shouldOutputDeviceDither ()
{
   return vfOutputDeviceDither_d;
}

bool DebugOutput::
shouldOutputDeviceInstance ()
{
   return vfOutputDeviceInstance_d;
}

bool DebugOutput::
shouldOutputDevicePrintMode ()
{
   return vfOutputDevicePrintMode_d;
}

bool DebugOutput::
shouldOutputDeviceTester ()
{
   return vfOutputDeviceTester_d;
}

bool DebugOutput::
shouldOutputGplCompression ()
{
   return vfOutputGplCompression_d;
}

bool DebugOutput::
shouldOutputGplDitherInstance ()
{
   return vfOutputGplDitherInstance_d;
}

bool DebugOutput::
shouldOutputOmni ()
{
   return vfOutputOmni_d;
}

bool DebugOutput::
shouldOutputOmniInterface ()
{
   return vfOutputOmniInterface_d;
}

bool DebugOutput::
shouldOutputOmniProxy ()
{
   return vfOutputOmniProxy_d;
}

bool DebugOutput::
shouldOutputPrintDevice ()
{
   return vfOutputPrintDevice_d;
}

bool DebugOutput::
shouldOutputBlitter ()
{
   return vfOutputBlitter_d;
}

bool DebugOutput::
shouldOutputInstance ()
{
   return vfOutputInstance_d;
}

bool DebugOutput::
shouldOutputOmniPDCProxy ()
{
   return vfOutputOmniPDCProxy_d;
}

bool DebugOutput::
shouldOutputOmniServer ()
{
   return vfOutputOmniServer_d;
}

bool DebugOutput::
shouldOutputPrinterCommand ()
{
   return vfOutputPrinterCommand_d;
}

bool DebugOutput::
shouldOutputPluggableInstance ()
{
   return vfOutputPluggableInstance_d;
}

bool DebugOutput::
shouldOutputPluggableBlitter ()
{
   return vfOutputPluggableBlitter_d;
}

bool DebugOutput::
shouldOutputDriverInfo ()
{
   return vfOutputDriverInfo_d;
}

bool DebugOutput::
shouldOutputPDCBlitter ()
{
   return vfOutputPDCBlitter_d;
}

bool DebugOutput::
shouldOutputPDCBlitterClient ()
{
   return vfOutputPDCBlitterClient_d;
}

bool DebugOutput::
shouldOutputDeviceString ()
{
   return vfOutputDeviceString_d;
}

bool DebugOutput::
shouldOutputUPDFDevice ()
{
   return vfOutputUPDFDevice_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceOrientation ()
{
   return vfOutputUPDFDeviceOrientation_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceForm ()
{
   return vfOutputUPDFDeviceForm_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceTray ()
{
   return vfOutputUPDFDeviceTray_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceMedia ()
{
   return vfOutputUPDFDeviceMedia_d;
}

bool DebugOutput::
shouldOutputUPDFDevicePrintMode ()
{
   return vfOutputUPDFDevicePrintMode_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceResolution ()
{
   return vfOutputUPDFDeviceResolution_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceInstance ()
{
   return vfOutputUPDFDeviceInstance_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceBlitter ()
{
   return vfOutputUPDFDeviceBlitter_d;
}

void DebugOutput::
setOutputDeviceBlitter (bool fOutputDeviceBlitter)
{
   vfOutputDeviceBlitter_d = fOutputDeviceBlitter;
}

void DebugOutput::
setOutputDeviceDither (bool fOutputDeviceDither)
{
   vfOutputDeviceDither_d = fOutputDeviceDither;
}

void DebugOutput::
setOutputDeviceInstance (bool fOutputDeviceInstance)
{
   vfOutputDeviceInstance_d = fOutputDeviceInstance;
}

void DebugOutput::
setOutputDevicePrintMode (bool fOutputDevicePrintMode)
{
   vfOutputDevicePrintMode_d = fOutputDevicePrintMode;
}

void DebugOutput::
setOutputDeviceTester (bool fOutputDeviceTester)
{
   vfOutputDeviceTester_d = fOutputDeviceTester;
}

void DebugOutput::
setOutputGplCompression (bool fOutputGplCompression)
{
   vfOutputGplCompression_d = fOutputGplCompression;
}

void DebugOutput::
setOutputGplDitherInstance (bool fOutputGplDitherInstance)
{
   vfOutputGplDitherInstance_d = fOutputGplDitherInstance;
}

void DebugOutput::
setOutputOmni (bool fOutputOmni)
{
   vfOutputOmni_d = fOutputOmni;
}

void DebugOutput::
setOutputOmniInterface (bool fOutputOmniInterface)
{
   vfOutputOmniInterface_d = fOutputOmniInterface;
}

void DebugOutput::
setOutputOmniProxy (bool fOutputOmniProxy)
{
   vfOutputOmniProxy_d = fOutputOmniProxy;
}

void DebugOutput::
setOutputPrintDevice (bool fOutputPrintDevice)
{
   vfOutputPrintDevice_d = fOutputPrintDevice;
}

void DebugOutput::
setOutputBlitter (bool fOutputBlitter)
{
   vfOutputBlitter_d = fOutputBlitter;
}

void DebugOutput::
setOutputInstance (bool fOutputInstance)
{
   vfOutputInstance_d = fOutputInstance;
}

void DebugOutput::
setOutputOmniPDCProxy (bool fOutputOmniPDCProxy)
{
   vfOutputOmniPDCProxy_d = fOutputOmniPDCProxy;
}

void DebugOutput::
setOutputOmniServer (bool fOutputOmniServer)
{
   vfOutputOmniServer_d = fOutputOmniServer;
}

void DebugOutput::
setOutputPrinterCommand (bool fOutputPrinterCommand)
{
   vfOutputPrinterCommand_d = fOutputPrinterCommand;
}

void DebugOutput::
setOutputPluggableInstance (bool fOutputPluggableInstance)
{
   vfOutputPluggableInstance_d = fOutputPluggableInstance;
}

void DebugOutput::
setOutputPluggableBlitter (bool fOutputPluggableBlitter)
{
   vfOutputPluggableBlitter_d = fOutputPluggableBlitter;
}

void DebugOutput::
setOutputDriverInfo (bool fOutputDriverInfo)
{
   vfOutputDriverInfo_d = fOutputDriverInfo;
}

void DebugOutput::
setOutputPDCBlitter (bool fOutputPDCBlitter)
{
   vfOutputPDCBlitter_d = fOutputPDCBlitter;
}

void DebugOutput::
setOutputPDCBlitterClient (bool fOutputPDCBlitterClient)
{
   vfOutputPDCBlitterClient_d = fOutputPDCBlitterClient;
}

void DebugOutput::
setOutputDeviceString (bool fOutputDeviceString)
{
   vfOutputDeviceString_d = fOutputDeviceString;
}

void DebugOutput::
setOutputUPDFDevice (bool fOutputUPDFDevice)
{
   vfOutputUPDFDevice_d = fOutputUPDFDevice;
}

void DebugOutput::
setOutputUPDFDeviceOrientation (bool fOutputUPDFDeviceOrientation)
{
   vfOutputUPDFDeviceOrientation_d = fOutputUPDFDeviceOrientation;
}

void DebugOutput::
setOutputUPDFDeviceForm (bool fOutputUPDFDeviceForm)
{
   vfOutputUPDFDeviceForm_d = fOutputUPDFDeviceForm;
}

void DebugOutput::
setOutputUPDFDeviceTray (bool fOutputUPDFDeviceTray)
{
   vfOutputUPDFDeviceTray_d = fOutputUPDFDeviceTray;
}

void DebugOutput::
setOutputUPDFDeviceMedia (bool fOutputUPDFDeviceMedia)
{
   vfOutputUPDFDeviceMedia_d = fOutputUPDFDeviceMedia;
}

void DebugOutput::
setOutputUPDFDevicePrintMode (bool fOutputUPDFDevicePrintMode)
{
   vfOutputUPDFDevicePrintMode_d = fOutputUPDFDevicePrintMode;
}

void DebugOutput::
setOutputUPDFDeviceResolution (bool fOutputUPDFDeviceResolution)
{
   vfOutputUPDFDeviceResolution_d = fOutputUPDFDeviceResolution;
}

void DebugOutput::
setOutputUPDFDeviceInstance (bool fOutputUPDFDeviceInstance)
{
   vfOutputUPDFDeviceInstance_d = fOutputUPDFDeviceInstance;
}

void DebugOutput::
setOutputUPDFDeviceBlitter (bool fOutputUPDFDeviceBlitter)
{
   vfOutputUPDFDeviceBlitter_d = fOutputUPDFDeviceBlitter;
}
