/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "DeviceForm.hpp"
#include "BinaryData.hpp"
#include "StringResource.hpp"
#include <stdio.h>
#include <string.h>

/* Function prototypes...
*/
static int    omniNameToID          (char             *pszId);
static int    standardNameToID      (char             *pszId);
static int    shortStandardNameToID (char             *pszId);

#define STRING_OF(x) (apszOmniFormNames[x - DeviceForm::FORM_UNLISTED])

static char *apszOmniFormNames[] = {
   "FORM_UNLISTED",
   "FORM_NONE",
   "FORM_USER_DEFINED",
   "FORM_TEST",
   "FORM_TEST2",
   "FORM_LEDGER",
   "FORM_LEGAL",
   "FORM_LEGAL_EXTRA",
   "FORM_LETTER",
   "FORM_LETTER_PLUS",
   "FORM_LETTER_EXTRA",
   "FORM_HALF_LETTER",
   "FORM_EXECUTIVE",
   "FORM_STATEMENT",
   "FORM_TABLOID",
   "FORM_UNIVERSAL",
   "FORM_FOOLSCAP",
   "FORM_QUARTO",
   "FORM_FOLIO",
   "FORM_FOLIO_SP",
   "FORM_ROC_8K",
   "FORM_ROC_16K",
   "FORM_GOVERNMENT_LETTER",
   "FORM_GOVERNMENT_LEGAL",
   "FORM_EDP",
   "FORM_EUROPEAN_EDP",
   "FORM_PRC_16K",
   "FORM_PRC_32K",
   "FORM_DAI_PA_KAI",
   "FORM_JUURO_KU_KAI",
   "FORM_PA_KAI",
   "FORM_WIDE",
   "FORM_LETTER_WIDE",
   "FORM_FOOLSCAP_WIDE",
   "FORM_A3_WIDE",
   "FORM_A4_WIDE",
   "FORM_WIDE_FORMAT",
   "FORM_10_X_11",
   "FORM_11_X_12",
   "FORM_11_X_15",
   "FORM_12_X_19",
   "FORM_15_X_11",
   "FORM_5_X_7",
   "FORM_3_X_5_CARD",
   "FORM_4_X_6_CARD",
   "FORM_5_X_8_CARD",
   "FORM_8_X_10_CARD",
   "FORM_A6_CARD",
   "FORM_CARD_148",
   "FORM_HAGAKI_CARD",
   "FORM_OUFUKU_CARD",
   "FORM_POSTCARD",
   "FORM_INDEX_4_X_6_EXT",
   "FORM_A2_ENVELOPE",
   "FORM_C5_ENVELOPE",
   "FORM_C6_ENVELOPE",
   "FORM_C7_ENVELOPE",
   "FORM_C9_ENVELOPE",
   "FORM_C10_ENVELOPE",
   "FORM_D5_ENVELOPE",
   "FORM_DL_ENVELOPE",
   "FORM_ENVELOPE_132_X_220",
   "FORM_ENVELOPE_6_X_9",
   "FORM_ENVELOPE_7_X_9",
   "FORM_ENVELOPE_6_1_2",
   "FORM_ENVELOPE_9_X_11",
   "FORM_ENVELOPE_10_X_13",
   "FORM_ENVELOPE_10_X_14",
   "FORM_ENVELOPE_10_X_15",
   "FORM_MONARCH_ENVELOPE",
   "FORM_PERSONAL_ENVELOPE",
   "FORM_NUMBER_9_ENVELOPE",
   "FORM_NUMBER_10_ENVELOPE",
   "FORM_NUMBER_11_ENVELOPE",
   "FORM_NUMBER_12_ENVELOPE",
   "FORM_NUMBER_14_ENVELOPE",
   "FORM_ITALIAN_ENVELOPE",
   "FORM_POSTFIX_ENVELOPE",
   "FORM_INVITE_ENVELOPE",
   "FORM_CHOU2_ENVELOPE",
   "FORM_CHOU3_ENVELOPE",
   "FORM_CHOU4_ENVELOPE",
   "FORM_KAHU_ENVELOPE",
   "FORM_KAKU2_ENVELOPE",
   "FORM_YOU4_ENVELOPE",
   "FORM_PRC1_ENVELOPE",
   "FORM_PRC2_ENVELOPE",
   "FORM_PRC3_ENVELOPE",
   "FORM_PRC4_ENVELOPE",
   "FORM_PRC5_ENVELOPE",
   "FORM_PRC6_ENVELOPE",
   "FORM_PRC7_ENVELOPE",
   "FORM_PRC8_ENVELOPE",
   "FORM_PRC9_ENVELOPE",
   "FORM_PRC10_ENVELOPE",
   "FORM_DISK_LABELS",
   "FORM_EURO_LABELS",
   "FORM_SHIPPING_LABELS",
   "FORM_STANDARD_LABELS_CLEAR",
   "FORM_STANDARD_LABELS_WHITE",
   "FORM_2A0",
   "FORM_4A0",
   "FORM_A0",
   "FORM_A1",
   "FORM_A2",
   "FORM_A3",
   "FORM_A3_EXTRA",
   "FORM_A4",
   "FORM_A4_EXTRA",
   "FORM_A4_TAB",
   "FORM_A5",
   "FORM_A5_EXTRA",
   "FORM_A6",
   "FORM_A7",
   "FORM_A8",
   "FORM_A9",
   "FORM_A10",
   "FORM_B_PLUS",
   "FORM_B0",
   "FORM_B1",
   "FORM_B2",
   "FORM_B3",
   "FORM_B4",
   "FORM_B5",
   "FORM_B5_EXTRA",
   "FORM_B6",
   "FORM_B6_C4",
   "FORM_B7",
   "FORM_B8",
   "FORM_B9",
   "FORM_B10",
   "FORM_JIS_B0",
   "FORM_JIS_B1",
   "FORM_JIS_B2",
   "FORM_JIS_B3",
   "FORM_JIS_B4",
   "FORM_JIS_B5",
   "FORM_JIS_B6",
   "FORM_JIS_B7",
   "FORM_JIS_B8",
   "FORM_JIS_B9",
   "FORM_JIS_B10",
   "FORM_JIS_EXEC",
   "FORM_C0",
   "FORM_C1",
   "FORM_C2",
   "FORM_C3",
   "FORM_C4",
   "FORM_C5",
   "FORM_C6",
   "FORM_C6_C5",
   "FORM_C7",
   "FORM_C7_C6",
   "FORM_C8",
   "FORM_C9",
   "FORM_C10",
   "FORM_E1",
   "FORM_RA0",
   "FORM_RA1",
   "FORM_RA2",
   "FORM_SRA0",
   "FORM_SRA1",
   "FORM_SRA2",
   "FORM_US_FANFOLD",
   "FORM_FANFOLD_1",
   "FORM_FANFOLD_2",
   "FORM_FANFOLD_3",
   "FORM_FANFOLD_4",
   "FORM_FANFOLD_5",
   "FORM_GERMAN_LEGAL_FANFOLD",
   "FORM_EUROPEAN_FANFOLD",
   "FORM_PANORAMIC",
   "FORM_PHOTO_100_150",
   "FORM_PHOTO_200_300",
   "FORM_PHOTO_4_6",
   "FORM_PLOTTER_SIZE_A",
   "FORM_PLOTTER_SIZE_B",
   "FORM_PLOTTER_SIZE_C",
   "FORM_PLOTTER_SIZE_D",
   "FORM_PLOTTER_SIZE_E",
   "FORM_PLOTTER_SIZE_F",
   "FORM_US_A_ARCHITECTURAL",
   "FORM_US_B_ARCHITECTURAL",
   "FORM_US_C_ARCHITECTURAL",
   "FORM_US_D_ARCHITECTURAL",
   "FORM_US_E_ARCHITECTURAL",
   "FORM_SUPER_A3_B",
   "FORM_SUPER_A",
   "FORM_SUPER_B",
   "FORM_ROLL_76_2MM",
   "FORM_ROLL_69_5MM"
};

#ifdef USE_STANDARD_NAMES

static char *apszStandardFormNames[] = {
/* @MISSING FORM_UNLISTED              */ 0,
/* @MISSING FORM_NONE                  */ 0,
/* @MISSING FORM_USER_DEFINED          */ 0,
/* @MISSING FORM_TEST                  */ 0,
/* @MISSING FORM_TEST2                 */ 0,
/*          FORM_LEDGER                */ "na_ledger",
/*          FORM_LEGAL                 */ "na_legal",
/*          FORM_LEGAL_EXTRA           */ "na_legal-extra",
/*          FORM_LETTER                */ "na_letter",
/*          FORM_LETTER_PLUS           */ "na_letter-plus",
/*          FORM_LETTER_EXTRA          */ "na_letter-extra",
/* @MISSING FORM_HALF_LETTER           */ 0,
/*          FORM_EXECUTIVE             */ "na_executive",
/* @MISSING FORM_STATEMENT             */ 0,
/* @MISSING FORM_TABLOID               */ 0,
/* @MISSING FORM_UNIVERSAL             */ 0,
/*          FORM_FOOLSCAP              */ "na_foolscap",
/*          FORM_QUARTO                */ "na_quarto",
/*          FORM_FOLIO                 */ "om_folio",
/*          FORM_FOLIO_SP              */ "om_folio-sp",
/*          FORM_ROC_8K                */ "roc_8k",
/*          FORM_ROC_16K               */ "roc_16k",
/*          FORM_GOVERNMENT_LETTER     */ "na_govt-legal",
/*          FORM_GOVERNMENT_LEGAL      */ "na_govt-letter",
/*          FORM_EDP                   */ "na_edp",
/*          FORM_EUROPEAN_EDP          */ "na_eur-edp",
/*          FORM_PRC_16K               */ "prc_16k",
/*          FORM_PRC_32K               */ "prc_32k",
/*          FORM_DAI_PA_KAI            */ "om_dai-pa-kai",
/*          FORM_JUURO_KU_KAI          */ "om_juuro-ku-kai",
/*          FORM_PA_KAI                */ "om_pa-kai",
/*          FORM_WIDE                  */ "na_wide-format",
/* @MISSING FORM_LETTER_WIDE           */ 0,
/* @MISSING FORM_FOOLSCAP_WIDE         */ 0,
/* @MISSING FORM_A3_WIDE               */ 0,
/* @MISSING FORM_A4_WIDE               */ 0,
/* @MISSING FORM_WIDE_FORMAT           */ 0,
/*          FORM_10_X_11               */ "na_10x11",
/*          FORM_11_X_12               */ "na_11x12",
/*          FORM_11_X_15               */ "na_11x15",
/*          FORM_12_X_19               */ "na_12x19",
/* @MISSING FORM_15_X_11               */ 0,
/*          FORM_5_X_7                 */ "na_5x7",
/* @MISSING FORM_3_X_5_CARD            */ 0,
/* @MISSING FORM_4_X_6_CARD            */ 0,
/* @MISSING FORM_5_X_8_CARD            */ 0,
/* @MISSING FORM_8_X_10_CARD           */ 0,
/* @MISSING FORM_A6_CARD               */ 0,
/* @MISSING FORM_CARD_148              */ 0,
/*          FORM_HAGAKI_CARD           */ "jpn_hagaki",
/*          FORM_OUFUKU_CARD           */ "jpn_oufuku",
/* @MISSING FORM_POSTCARD              */ 0,
/*          FORM_INDEX_4_X_6_EXT       */ "na_index-4x6-ext",
/*          FORM_A2_ENVELOPE           */ "na_a2",
/*          FORM_C5_ENVELOPE           */ "na_c5",
/* @MISSING FORM_C6_ENVELOPE           */ 0,
/* @MISSING FORM_C7_ENVELOPE           */ 0,
/* @MISSING FORM_C9_ENVELOPE           */ 0,
/* @MISSING FORM_C10_ENVELOPE          */ 0,
/* @MISSING FORM_D5_ENVELOPE           */ 0,
/* @MISSING FORM_DL_ENVELOPE           */ 0,
/* @MISSING FORM_ENVELOPE_132_X_220    */ 0,
/* @MISSING FORM_ENVELOPE_6_X_9        */ 0,
/* @MISSING FORM_ENVELOPE_7_X_9        */ 0,
/* @MISSING FORM_ENVELOPE_6_1_2        */ 0,
/* @MISSING FORM_ENVELOPE_9_X_11       */ 0,
/* @MISSING FORM_ENVELOPE_10_X_13      */ 0,
/* @MISSING FORM_ENVELOPE_10_X_14      */ 0,
/* @MISSING FORM_ENVELOPE_10_X_15      */ 0,
/*          FORM_MONARCH_ENVELOPE      */ "na_monarch",
/*          FORM_PERSONAL_ENVELOPE     */ "na_personal",
/*          FORM_NUMBER_9_ENVELOPE     */ "na_number-9",
/*          FORM_NUMBER_10_ENVELOPE    */ "na_number-10",
/*          FORM_NUMBER_11_ENVELOPE    */ "na_number-11",
/*          FORM_NUMBER_12_ENVELOPE    */ "na_number-12",
/*          FORM_NUMBER_14_ENVELOPE    */ "na_number-14",
/*          FORM_ITALIAN_ENVELOPE      */ "om_italian",
/*          FORM_POSTFIX_ENVELOPE      */ "om_postfix",
/*          FORM_INVITE_ENVELOPE       */ "om_invite",
/*          FORM_CHOU2_ENVELOPE        */ "jpn_chou2",
/*          FORM_CHOU3_ENVELOPE        */ "jpn_chou3",
/*          FORM_CHOU4_ENVELOPE        */ "jpn_chou4",
/*          FORM_KAHU_ENVELOPE         */ "jpn_kahu",
/*          FORM_KAKU2_ENVELOPE        */ "jpn_kaku2",
/*          FORM_YOU4_ENVELOPE         */ "jpn_you4",
/*          FORM_PRC1_ENVELOPE         */ "prc_1",
/*          FORM_PRC2_ENVELOPE         */ "prc_2",
/*          FORM_PRC3_ENVELOPE         */ "prc_3",
/*          FORM_PRC4_ENVELOPE         */ "prc_4",
/*          FORM_PRC5_ENVELOPE         */ "prc_5",
/*          FORM_PRC6_ENVELOPE         */ "prc_6",
/*          FORM_PRC7_ENVELOPE         */ "prc_7",
/*          FORM_PRC8_ENVELOPE         */ "prc_8",
/*          FORM_PRC9_ENVELOPE         */ "prc_9",
/*          FORM_PRC10_ENVELOPE        */ "prc_10",
/* @MISSING FORM_DISK_LABELS           */ 0,
/* @MISSING FORM_EURO_LABELS           */ 0,
/* @MISSING FORM_SHIPPING_LABELS       */ 0,
/* @MISSING FORM_STANDARD_LABELS_CLEAR */ 0,
/* @MISSING FORM_STANDARD_LABELS_WHITE */ 0,
/*          FORM_2A0                   */ "iso_2a0",
/*          FORM_4A0                   */ "iso_4a0",
/*          FORM_A0                    */ "iso_a0",
/*          FORM_A1                    */ "iso_a1",
/*          FORM_A2                    */ "iso_a2",
/*          FORM_A3                    */ "iso_a3",
/*          FORM_A3_EXTRA              */ "iso_a3-extra",
/*          FORM_A4                    */ "iso_a4",
/*          FORM_A4_EXTRA              */ "iso_a4-extra",
/*          FORM_A4_TAB                */ "iso_a4-tab",
/*          FORM_A5                    */ "iso_a5",
/*          FORM_A5_EXTRA              */ "iso_a5-extra",
/*          FORM_A6                    */ "iso_a6",
/*          FORM_A7                    */ "iso_a7",
/*          FORM_A8                    */ "iso_a8",
/*          FORM_A9                    */ "iso_a9",
/*          FORM_A10                   */ "iso_a10",
/*          FORM_B_PLUS                */ "na_b-plus",
/*          FORM_B0                    */ "iso_b0",
/*          FORM_B1                    */ "iso_b1",
/*          FORM_B2                    */ "iso_b2",
/*          FORM_B3                    */ "iso_b3",
/*          FORM_B4                    */ "iso_b4",
/*          FORM_B5                    */ "iso_b5",
/*          FORM_B5_EXTRA              */ "iso_b5-extra",
/*          FORM_B6                    */ "iso_b6",
/*          FORM_B6_C4                 */ "iso_b6c4",
/*          FORM_B7                    */ "iso_b7",
/*          FORM_B8                    */ "iso_b8",
/*          FORM_B9                    */ "iso_b9",
/*          FORM_B10                   */ "iso_b10",
/*          FORM_JIS_B0                */ "jis_b0",
/*          FORM_JIS_B1                */ "jis_b1",
/*          FORM_JIS_B2                */ "jis_b2",
/*          FORM_JIS_B3                */ "jis_b3",
/*          FORM_JIS_B4                */ "jis_b4",
/*          FORM_JIS_B5                */ "jis_b5",
/*          FORM_JIS_B6                */ "jis_b6",
/*          FORM_JIS_B7                */ "jis_b7",
/*          FORM_JIS_B8                */ "jis_b8",
/*          FORM_JIS_B9                */ "jis_b9",
/*          FORM_JIS_B10               */ "jis_b10",
/*          FORM_JIS_EXEC              */ "jis_exec",
/*          FORM_C0                    */ "iso_c0",
/*          FORM_C1                    */ "iso_c1",
/*          FORM_C2                    */ "iso_c2",
/*          FORM_C3                    */ "iso_c3",
/*          FORM_C4                    */ "iso_c4",
/*          FORM_C5                    */ "iso_c5",
/*          FORM_C6                    */ "iso_c6",
/*          FORM_C6_C5                 */ "iso_c6c5",
/*          FORM_C7                    */ "iso_c7",
/*          FORM_C7_C6                 */ "iso_c7c6",
/*          FORM_C8                    */ "iso_c8",
/*          FORM_C9                    */ "iso_c9",
/*          FORM_C10                   */ "iso_c10",
/* @MISSING FORM_E1                    */ 0,
/*          FORM_RA0                   */ "iso_ra0",
/*          FORM_RA1                   */ "iso_ra1",
/*          FORM_RA2                   */ "iso_ra2",
/*          FORM_SRA0                  */ "iso_sra0",
/*          FORM_SRA1                  */ "iso_sra1",
/*          FORM_SRA2                  */ "iso_sra2",
/*          FORM_US_FANFOLD            */ "na_fanfold-us",
/* @MISSING FORM_FANFOLD_1             */ 0,
/* @MISSING FORM_FANFOLD_2             */ 0,
/* @MISSING FORM_FANFOLD_3             */ 0,
/* @MISSING FORM_FANFOLD_4             */ 0,
/* @MISSING FORM_FANFOLD_5             */ 0,
/* @MISSING FORM_GERMAN_LEGAL_FANFOLD  */ 0,
/*          FORM_EUROPEAN_FANFOLD      */ "na_fanfold-eur",
/* @MISSING FORM_PANORAMIC             */ 0,
/* @MISSING FORM_PHOTO_100_150         */ 0,
/* @MISSING FORM_PHOTO_200_300         */ 0,
/* @MISSING FORM_PHOTO_4_6             */ 0,
/* @MISSING FORM_PLOTTER_SIZE_A        */ 0,
/* @MISSING FORM_PLOTTER_SIZE_B        */ 0,
/* @MISSING FORM_PLOTTER_SIZE_C        */ 0,
/* @MISSING FORM_PLOTTER_SIZE_D        */ 0,
/* @MISSING FORM_PLOTTER_SIZE_E        */ 0,
/* @MISSING FORM_PLOTTER_SIZE_F        */ 0,
/*          FORM_US_A_ARCHITECTURAL    */ "na_arch-a",
/*          FORM_US_B_ARCHITECTURAL    */ "na_arch-b",
/*          FORM_US_C_ARCHITECTURAL    */ "na_arch-c",
/*          FORM_US_D_ARCHITECTURAL    */ "na_arch-d",
/*          FORM_US_E_ARCHITECTURAL    */ "na_arch-e",
/* @MISSING FORM_SUPER_A3_B            */ 0,
/*          FORM_SUPER_A               */ "na_super-a",
/*          FORM_SUPER_B               */ "na_super-b",
/* @MISSING FORM_ROLL_76_2MM           */ 0,
/* @MISSING FORM_ROLL_69_5MM           */ 0
};

#endif

typedef struct _FormMapping {
   char   *pszUPDFName;
   int     iOmniFormID;
} FORMMAP, *PFORMMAP;

static FORMMAP vaStandardFormMapping[] = {
   { "iso_2a0",          DeviceForm::FORM_2A0                },
   { "iso_4a0",          DeviceForm::FORM_4A0                },
   { "iso_a0",           DeviceForm::FORM_A0                 },
   { "iso_a0x3",         DeviceForm::FORM_UNLISTED           },
   { "iso_a1",           DeviceForm::FORM_A1                 },
   { "iso_a10",          DeviceForm::FORM_A10                },
   { "iso_a1x3",         DeviceForm::FORM_UNLISTED           },
   { "iso_a1x4",         DeviceForm::FORM_UNLISTED           },
   { "iso_a2",           DeviceForm::FORM_A2                 },
   { "iso_a2x3",         DeviceForm::FORM_UNLISTED           },
   { "iso_a2x4",         DeviceForm::FORM_UNLISTED           },
   { "iso_a2x5",         DeviceForm::FORM_UNLISTED           },
   { "iso_a3",           DeviceForm::FORM_A3                 },
   { "iso_a3-extra",     DeviceForm::FORM_A3_EXTRA           },
   { "iso_a3x3",         DeviceForm::FORM_UNLISTED           },
   { "iso_a3x4",         DeviceForm::FORM_UNLISTED           },
   { "iso_a3x5",         DeviceForm::FORM_UNLISTED           },
   { "iso_a3x6",         DeviceForm::FORM_UNLISTED           },
   { "iso_a3x7",         DeviceForm::FORM_UNLISTED           },
   { "iso_a4",           DeviceForm::FORM_A4                 },
   { "iso_a4-extra",     DeviceForm::FORM_A4_EXTRA           },
   { "iso_a4-tab",       DeviceForm::FORM_A4_TAB             },
   { "iso_a4x3",         DeviceForm::FORM_UNLISTED           },
   { "iso_a4x4",         DeviceForm::FORM_UNLISTED           },
   { "iso_a4x5",         DeviceForm::FORM_UNLISTED           },
   { "iso_a4x6",         DeviceForm::FORM_UNLISTED           },
   { "iso_a4x7",         DeviceForm::FORM_UNLISTED           },
   { "iso_a4x8",         DeviceForm::FORM_UNLISTED           },
   { "iso_a4x9",         DeviceForm::FORM_UNLISTED           },
   { "iso_a5",           DeviceForm::FORM_A5                 },
   { "iso_a5-extra",     DeviceForm::FORM_A5_EXTRA           },
   { "iso_a6",           DeviceForm::FORM_A6                 },
   { "iso_a7",           DeviceForm::FORM_A7                 },
   { "iso_a8",           DeviceForm::FORM_A8                 },
   { "iso_a9",           DeviceForm::FORM_A9                 },
   { "iso_b0",           DeviceForm::FORM_B0                 },
   { "iso_b1",           DeviceForm::FORM_B1                 },
   { "iso_b10",          DeviceForm::FORM_B10                },
   { "iso_b2",           DeviceForm::FORM_B2                 },
   { "iso_b3",           DeviceForm::FORM_B3                 },
   { "iso_b4",           DeviceForm::FORM_B4                 },
   { "iso_b5",           DeviceForm::FORM_B5                 },
   { "iso_b5-extra",     DeviceForm::FORM_B5_EXTRA           },
   { "iso_b6",           DeviceForm::FORM_B6                 },
   { "iso_b6c4",         DeviceForm::FORM_B6_C4              },
   { "iso_b7",           DeviceForm::FORM_B7                 },
   { "iso_b8",           DeviceForm::FORM_B8                 },
   { "iso_b9",           DeviceForm::FORM_B9                 },
   { "iso_c0",           DeviceForm::FORM_C0                 },
   { "iso_c1",           DeviceForm::FORM_C1                 },
   { "iso_c10",          DeviceForm::FORM_C10                },
   { "iso_c2",           DeviceForm::FORM_C2                 },
   { "iso_c3",           DeviceForm::FORM_C3                 },
   { "iso_c4",           DeviceForm::FORM_C4                 },
   { "iso_c5",           DeviceForm::FORM_C5                 },
   { "iso_c6",           DeviceForm::FORM_C6                 },
   { "iso_c6c5",         DeviceForm::FORM_C6_C5              },
   { "iso_c7",           DeviceForm::FORM_C7                 },
   { "iso_c7c6",         DeviceForm::FORM_C7_C6              },
   { "iso_c8",           DeviceForm::FORM_C8                 },
   { "iso_c9",           DeviceForm::FORM_C9                 },
   { "iso_dl",           DeviceForm::FORM_UNLISTED           },
   { "iso_ra0",          DeviceForm::FORM_RA0                },
   { "iso_ra1",          DeviceForm::FORM_RA1                },
   { "iso_ra2",          DeviceForm::FORM_RA2                },
   { "iso_sra0",         DeviceForm::FORM_SRA0               },
   { "iso_sra1",         DeviceForm::FORM_SRA1               },
   { "iso_sra2",         DeviceForm::FORM_SRA2               },
   { "jis_b0",           DeviceForm::FORM_JIS_B0             },
   { "jis_b1",           DeviceForm::FORM_JIS_B1             },
   { "jis_b10",          DeviceForm::FORM_JIS_B10            },
   { "jis_b2",           DeviceForm::FORM_JIS_B2             },
   { "jis_b3",           DeviceForm::FORM_JIS_B3             },
   { "jis_b4",           DeviceForm::FORM_JIS_B4             },
   { "jis_b5",           DeviceForm::FORM_JIS_B5             },
   { "jis_b6",           DeviceForm::FORM_JIS_B6             },
   { "jis_b7",           DeviceForm::FORM_JIS_B7             },
   { "jis_b8",           DeviceForm::FORM_JIS_B8             },
   { "jis_b9",           DeviceForm::FORM_JIS_B9             },
   { "jis_exec",         DeviceForm::FORM_JIS_EXEC           },
   { "jpn_chou2",        DeviceForm::FORM_CHOU2_ENVELOPE     },
   { "jpn_chou3",        DeviceForm::FORM_CHOU3_ENVELOPE     },
   { "jpn_chou4",        DeviceForm::FORM_CHOU4_ENVELOPE     },
   { "jpn_hagaki",       DeviceForm::FORM_HAGAKI_CARD        },
   { "jpn_kahu",         DeviceForm::FORM_KAHU_ENVELOPE      },
   { "jpn_kaku2",        DeviceForm::FORM_KAKU2_ENVELOPE     },
   { "jpn_oufuku",       DeviceForm::FORM_OUFUKU_CARD        },
   { "jpn_you4",         DeviceForm::FORM_YOU4_ENVELOPE      },
   { "na_10x11",         DeviceForm::FORM_10_X_11            },
   { "na_10x13",         DeviceForm::FORM_UNLISTED           },
   { "na_10x14",         DeviceForm::FORM_UNLISTED           },
   { "na_10x15",         DeviceForm::FORM_UNLISTED           },
   { "na_11x12",         DeviceForm::FORM_11_X_12            },
   { "na_11x15",         DeviceForm::FORM_11_X_15            },
   { "na_12x19",         DeviceForm::FORM_12_X_19            },
   { "na_5x7",           DeviceForm::FORM_5_X_7              },
   { "na_6x9",           DeviceForm::FORM_UNLISTED           },
   { "na_7x9",           DeviceForm::FORM_UNLISTED           },
   { "na_9x11",          DeviceForm::FORM_UNLISTED           },
   { "na_a2",            DeviceForm::FORM_A2_ENVELOPE        },
   { "na_arch-a",        DeviceForm::FORM_US_A_ARCHITECTURAL },
   { "na_arch-b",        DeviceForm::FORM_US_B_ARCHITECTURAL },
   { "na_arch-c",        DeviceForm::FORM_US_C_ARCHITECTURAL },
   { "na_arch-d",        DeviceForm::FORM_US_D_ARCHITECTURAL },
   { "na_arch-e",        DeviceForm::FORM_US_E_ARCHITECTURAL },
   { "na_b-plus",        DeviceForm::FORM_B_PLUS             },
   { "na_c",             DeviceForm::FORM_UNLISTED           },
   { "na_c5",            DeviceForm::FORM_C5_ENVELOPE        },
   { "na_d",             DeviceForm::FORM_UNLISTED           },
   { "na_e",             DeviceForm::FORM_UNLISTED           },
   { "na_edp",           DeviceForm::FORM_EDP                },
   { "na_eur-edp",       DeviceForm::FORM_EUROPEAN_EDP       },
   { "na_executive",     DeviceForm::FORM_EXECUTIVE          },
   { "na_f",             DeviceForm::FORM_UNLISTED           },
   { "na_fanfold-eur",   DeviceForm::FORM_EUROPEAN_FANFOLD   },
   { "na_fanfold-us",    DeviceForm::FORM_US_FANFOLD         },
   { "na_foolscap",      DeviceForm::FORM_FOOLSCAP           },
   { "na_govt-legal",    DeviceForm::FORM_GOVERNMENT_LETTER  },
   { "na_govt-letter",   DeviceForm::FORM_GOVERNMENT_LEGAL   },
   { "na_index-3x5",     DeviceForm::FORM_UNLISTED           },
   { "na_index-4x6",     DeviceForm::FORM_UNLISTED           },
   { "na_index-4x6-ext", DeviceForm::FORM_INDEX_4_X_6_EXT    },
   { "na_index-5x8",     DeviceForm::FORM_UNLISTED           },
   { "na_invoice",       DeviceForm::FORM_UNLISTED           },
   { "na_ledger",        DeviceForm::FORM_LEDGER             },
   { "na_legal",         DeviceForm::FORM_LEGAL              },
   { "na_legal-extra",   DeviceForm::FORM_LEGAL_EXTRA        },
   { "na_letter",        DeviceForm::FORM_LETTER             },
   { "na_letter-extra",  DeviceForm::FORM_LETTER_EXTRA       },
   { "na_letter-plus",   DeviceForm::FORM_LETTER_PLUS        },
   { "na_monarch",       DeviceForm::FORM_MONARCH_ENVELOPE   },
   { "na_number-10",     DeviceForm::FORM_NUMBER_10_ENVELOPE },
   { "na_number-11",     DeviceForm::FORM_NUMBER_11_ENVELOPE },
   { "na_number-12",     DeviceForm::FORM_NUMBER_12_ENVELOPE },
   { "na_number-14",     DeviceForm::FORM_NUMBER_14_ENVELOPE },
   { "na_number-9",      DeviceForm::FORM_NUMBER_9_ENVELOPE  },
   { "na_personal",      DeviceForm::FORM_PERSONAL_ENVELOPE  },
   { "na_quarto",        DeviceForm::FORM_QUARTO             },
   { "na_super-a",       DeviceForm::FORM_SUPER_A            },
   { "na_super-b",       DeviceForm::FORM_SUPER_B            },
   { "na_wide-format",   DeviceForm::FORM_WIDE               },
   { "om_dai-pa-kai",    DeviceForm::FORM_DAI_PA_KAI         },
   { "om_folio",         DeviceForm::FORM_FOLIO              },
   { "om_folio-sp",      DeviceForm::FORM_FOLIO_SP           },
   { "om_invite",        DeviceForm::FORM_INVITE_ENVELOPE    },
   { "om_italian",       DeviceForm::FORM_ITALIAN_ENVELOPE   },
   { "om_juuro-ku-kai",  DeviceForm::FORM_JUURO_KU_KAI       },
   { "om_large-photo",   DeviceForm::FORM_UNLISTED           },
   { "om_pa-kai",        DeviceForm::FORM_PA_KAI             },
   { "om_postfix",       DeviceForm::FORM_POSTFIX_ENVELOPE   },
   { "om_small-photo",   DeviceForm::FORM_UNLISTED           },
   { "prc_1",            DeviceForm::FORM_PRC1_ENVELOPE      },
   { "prc_10",           DeviceForm::FORM_PRC10_ENVELOPE     },
   { "prc_16k",          DeviceForm::FORM_PRC_16K            },
   { "prc_2",            DeviceForm::FORM_PRC2_ENVELOPE      },
   { "prc_3",            DeviceForm::FORM_PRC3_ENVELOPE      },
   { "prc_32k",          DeviceForm::FORM_PRC_32K            },
   { "prc_4",            DeviceForm::FORM_PRC4_ENVELOPE      },
   { "prc_5",            DeviceForm::FORM_PRC5_ENVELOPE      },
   { "prc_6",            DeviceForm::FORM_PRC6_ENVELOPE      },
   { "prc_7",            DeviceForm::FORM_PRC7_ENVELOPE      },
   { "prc_8",            DeviceForm::FORM_PRC8_ENVELOPE      },
   { "prc_9",            DeviceForm::FORM_PRC9_ENVELOPE      },
   { "roc_16k",          DeviceForm::FORM_ROC_16K            },
   { "roc_8k",           DeviceForm::FORM_ROC_8K             }
};

static FORMMAP vaShortFormMapping[] = {
   { "1",             DeviceForm::FORM_PRC1_ENVELOPE      },
   { "10",            DeviceForm::FORM_PRC10_ENVELOPE     },
   { "10x11",         DeviceForm::FORM_10_X_11            },
   { "10x13",         DeviceForm::FORM_UNLISTED           },
   { "10x14",         DeviceForm::FORM_UNLISTED           },
   { "10x15",         DeviceForm::FORM_UNLISTED           },
   { "11x12",         DeviceForm::FORM_11_X_12            },
   { "11x15",         DeviceForm::FORM_11_X_15            },
   { "12x19",         DeviceForm::FORM_12_X_19            },
   { "16k",           DeviceForm::FORM_PRC_16K            },
   { "2",             DeviceForm::FORM_PRC2_ENVELOPE      },
   { "2a0",           DeviceForm::FORM_2A0                },
   { "3",             DeviceForm::FORM_PRC3_ENVELOPE      },
   { "32k",           DeviceForm::FORM_PRC_32K            },
   { "4",             DeviceForm::FORM_PRC4_ENVELOPE      },
   { "4a0",           DeviceForm::FORM_4A0                },
   { "5",             DeviceForm::FORM_PRC5_ENVELOPE      },
   { "5x7",           DeviceForm::FORM_5_X_7              },
   { "6",             DeviceForm::FORM_PRC6_ENVELOPE      },
   { "6x9",           DeviceForm::FORM_UNLISTED           },
   { "7",             DeviceForm::FORM_PRC7_ENVELOPE      },
   { "7x9",           DeviceForm::FORM_UNLISTED           },
   { "8",             DeviceForm::FORM_PRC8_ENVELOPE      },
   { "8k",            DeviceForm::FORM_ROC_8K             },
   { "9",             DeviceForm::FORM_PRC9_ENVELOPE      },
   { "9x11",          DeviceForm::FORM_UNLISTED           },
   { "a0",            DeviceForm::FORM_A0                 },
   { "a0x3",          DeviceForm::FORM_UNLISTED           },
   { "a1",            DeviceForm::FORM_A1                 },
   { "a10",           DeviceForm::FORM_A10                },
   { "a1x3",          DeviceForm::FORM_UNLISTED           },
   { "a1x4",          DeviceForm::FORM_UNLISTED           },
   { "a2",            DeviceForm::FORM_A2                 },
   { "a2x3",          DeviceForm::FORM_UNLISTED           },
   { "a2x4",          DeviceForm::FORM_UNLISTED           },
   { "a2x5",          DeviceForm::FORM_UNLISTED           },
   { "a3",            DeviceForm::FORM_A3                 },
   { "a3-extra",      DeviceForm::FORM_A3_EXTRA           },
   { "a3x3",          DeviceForm::FORM_UNLISTED           },
   { "a3x4",          DeviceForm::FORM_UNLISTED           },
   { "a3x5",          DeviceForm::FORM_UNLISTED           },
   { "a3x6",          DeviceForm::FORM_UNLISTED           },
   { "a3x7",          DeviceForm::FORM_UNLISTED           },
   { "a4",            DeviceForm::FORM_A4                 },
   { "a4-extra",      DeviceForm::FORM_A4_EXTRA           },
   { "a4-tab",        DeviceForm::FORM_A4_TAB             },
   { "a4x3",          DeviceForm::FORM_UNLISTED           },
   { "a4x4",          DeviceForm::FORM_UNLISTED           },
   { "a4x5",          DeviceForm::FORM_UNLISTED           },
   { "a4x6",          DeviceForm::FORM_UNLISTED           },
   { "a4x7",          DeviceForm::FORM_UNLISTED           },
   { "a4x8",          DeviceForm::FORM_UNLISTED           },
   { "a4x9",          DeviceForm::FORM_UNLISTED           },
   { "a5",            DeviceForm::FORM_A5                 },
   { "a5-extra",      DeviceForm::FORM_A5_EXTRA           },
   { "a6",            DeviceForm::FORM_A6                 },
   { "a7",            DeviceForm::FORM_A7                 },
   { "a8",            DeviceForm::FORM_A8                 },
   { "a9",            DeviceForm::FORM_A9                 },
   { "arch-a",        DeviceForm::FORM_US_A_ARCHITECTURAL },
   { "arch-b",        DeviceForm::FORM_US_B_ARCHITECTURAL },
   { "arch-c",        DeviceForm::FORM_US_C_ARCHITECTURAL },
   { "arch-d",        DeviceForm::FORM_US_D_ARCHITECTURAL },
   { "arch-e",        DeviceForm::FORM_US_E_ARCHITECTURAL },
   { "b-plus",        DeviceForm::FORM_B_PLUS             },
   { "b0",            DeviceForm::FORM_B0                 },
   { "b1",            DeviceForm::FORM_B1                 },
   { "b10",           DeviceForm::FORM_B10                },
   { "b2",            DeviceForm::FORM_B2                 },
   { "b3",            DeviceForm::FORM_B3                 },
   { "b4",            DeviceForm::FORM_B4                 },
   { "b5",            DeviceForm::FORM_B5                 },
   { "b5-extra",      DeviceForm::FORM_B5_EXTRA           },
   { "b6",            DeviceForm::FORM_B6                 },
   { "b6c4",          DeviceForm::FORM_B6_C4              },
   { "b7",            DeviceForm::FORM_B7                 },
   { "b8",            DeviceForm::FORM_B8                 },
   { "b9",            DeviceForm::FORM_B9                 },
   { "c",             DeviceForm::FORM_UNLISTED           },
   { "c0",            DeviceForm::FORM_C0                 },
   { "c1",            DeviceForm::FORM_C1                 },
   { "c10",           DeviceForm::FORM_C10                },
   { "c2",            DeviceForm::FORM_C2                 },
   { "c3",            DeviceForm::FORM_C3                 },
   { "c4",            DeviceForm::FORM_C4                 },
   { "c5",            DeviceForm::FORM_C5                 },
   { "c6",            DeviceForm::FORM_C6                 },
   { "c6c5",          DeviceForm::FORM_C6_C5              },
   { "c7",            DeviceForm::FORM_C7                 },
   { "c7c6",          DeviceForm::FORM_C7_C6              },
   { "c8",            DeviceForm::FORM_C8                 },
   { "c9",            DeviceForm::FORM_C9                 },
   { "chou2",         DeviceForm::FORM_CHOU2_ENVELOPE     },
   { "chou3",         DeviceForm::FORM_CHOU3_ENVELOPE     },
   { "chou4",         DeviceForm::FORM_CHOU4_ENVELOPE     },
   { "d",             DeviceForm::FORM_UNLISTED           },
   { "dai-pa-kai",    DeviceForm::FORM_DAI_PA_KAI         },
   { "dl",            DeviceForm::FORM_UNLISTED           },
   { "e",             DeviceForm::FORM_UNLISTED           },
   { "edp",           DeviceForm::FORM_EDP                },
   { "eur-edp",       DeviceForm::FORM_EUROPEAN_EDP       },
   { "exec",          DeviceForm::FORM_JIS_EXEC           },
   { "executive",     DeviceForm::FORM_EXECUTIVE          },
   { "f",             DeviceForm::FORM_UNLISTED           },
   { "fanfold-eur",   DeviceForm::FORM_EUROPEAN_FANFOLD   },
   { "fanfold-us",    DeviceForm::FORM_US_FANFOLD         },
   { "folio",         DeviceForm::FORM_FOLIO              },
   { "folio-sp",      DeviceForm::FORM_FOLIO_SP           },
   { "foolscap",      DeviceForm::FORM_FOOLSCAP           },
   { "govt-legal",    DeviceForm::FORM_GOVERNMENT_LETTER  },
   { "govt-letter",   DeviceForm::FORM_GOVERNMENT_LEGAL   },
   { "hagaki",        DeviceForm::FORM_HAGAKI_CARD        },
   { "index-3x5",     DeviceForm::FORM_UNLISTED           },
   { "index-4x6",     DeviceForm::FORM_UNLISTED           },
   { "index-4x6-ext", DeviceForm::FORM_INDEX_4_X_6_EXT    },
   { "index-5x8",     DeviceForm::FORM_UNLISTED           },
   { "invite",        DeviceForm::FORM_INVITE_ENVELOPE    },
   { "invoice",       DeviceForm::FORM_UNLISTED           },
   { "italian",       DeviceForm::FORM_ITALIAN_ENVELOPE   },
   { "juuro-ku-kai",  DeviceForm::FORM_JUURO_KU_KAI       },
   { "kahu",          DeviceForm::FORM_KAHU_ENVELOPE      },
   { "kaku2",         DeviceForm::FORM_KAKU2_ENVELOPE     },
   { "large-photo",   DeviceForm::FORM_UNLISTED           },
   { "ledger",        DeviceForm::FORM_LEDGER             },
   { "legal",         DeviceForm::FORM_LEGAL              },
   { "legal-extra",   DeviceForm::FORM_LEGAL_EXTRA        },
   { "letter",        DeviceForm::FORM_LETTER             },
   { "letter-extra",  DeviceForm::FORM_LETTER_EXTRA       },
   { "letter-plus",   DeviceForm::FORM_LETTER_PLUS        },
   { "monarch",       DeviceForm::FORM_MONARCH_ENVELOPE   },
   { "number-10",     DeviceForm::FORM_NUMBER_10_ENVELOPE },
   { "number-11",     DeviceForm::FORM_NUMBER_11_ENVELOPE },
   { "number-12",     DeviceForm::FORM_NUMBER_12_ENVELOPE },
   { "number-14",     DeviceForm::FORM_NUMBER_14_ENVELOPE },
   { "number-9",      DeviceForm::FORM_NUMBER_9_ENVELOPE  },
   { "oufuku",        DeviceForm::FORM_OUFUKU_CARD        },
   { "pa-kai",        DeviceForm::FORM_PA_KAI             },
   { "personal",      DeviceForm::FORM_PERSONAL_ENVELOPE  },
   { "postfix",       DeviceForm::FORM_POSTFIX_ENVELOPE   },
   { "quarto",        DeviceForm::FORM_QUARTO             },
   { "ra0",           DeviceForm::FORM_RA0                },
   { "ra1",           DeviceForm::FORM_RA1                },
   { "ra2",           DeviceForm::FORM_RA2                },
   { "small-photo",   DeviceForm::FORM_UNLISTED           },
   { "sra0",          DeviceForm::FORM_SRA0               },
   { "sra1",          DeviceForm::FORM_SRA1               },
   { "sra2",          DeviceForm::FORM_SRA2               },
   { "super-a",       DeviceForm::FORM_SUPER_A            },
   { "super-b",       DeviceForm::FORM_SUPER_B            },
   { "wide-format",   DeviceForm::FORM_WIDE               },
   { "you4",          DeviceForm::FORM_YOU4_ENVELOPE      }
};

typedef struct _PrefixEntry {
   char *pszName;
   int   cbName;
} PREFIXENTRY, *PPREFIXENTRY;

static PREFIXENTRY aPrefixes[] = {
   { "iso_", 4 },
   { "jis_", 4 },
   { "jpn_", 4 },
   { "na_",  3 },
   { "om_",  3 },
   { "prc_", 4 },
   { "roc_", 3 }
};

typedef enum _eUnits {
   INCHES,
   MM
} EUNITS;

typedef struct _FormSize {
   int    cx;
   int    cy;
   EUNITS eExternalDisplayUnits;
} FORMSIZE, *PFORMSIZE;

static FORMSIZE vaFormSizes[] = {
   /* FORM_USER_DEFINED          */ {       0,        0, INCHES },
   /* FORM_TEST                  */ {   69500,    76200, INCHES },
   /* FORM_TEST2                 */ {       0,        0, INCHES },
   /* FORM_LEDGER                */ {  431800,   279400, INCHES },
   /* FORM_LEGAL                 */ {  215900,   355600, INCHES },
   /* FORM_LEGAL_EXTRA           */ {  241300,   381000, INCHES },
   /* FORM_LETTER                */ {  215900,   279400, INCHES },
   /* FORM_LETTER_PLUS           */ {  215900,   322326, INCHES },
   /* FORM_LETTER_EXTRA          */ {  241300,   304800, INCHES },
   /* FORM_HALF_LETTER           */ {  139690,   215970, INCHES },
   /* FORM_EXECUTIVE             */ {  184150,   266700, INCHES },
   /* FORM_STATEMENT             */ {  139700,   215900, INCHES },
   /* FORM_TABLOID               */ {  279400,   431800, INCHES },
   /* FORM_UNIVERSAL             */ {  297040,   431800, INCHES },
   /* FORM_FOOLSCAP              */ {  203200,   330200, INCHES },
   /* FORM_QUARTO                */ {  215900,   275082, INCHES },
   /* FORM_FOLIO                 */ {  215900,   330000, MM     },
   /* FORM_FOLIO_SP              */ {  215000,   315000, MM     },
   /* FORM_ROC_8K                */ {  273050,   393700, MM     },
   /* FORM_ROC_16K               */ {  196850,   273050, MM     },
   /* FORM_GOVERNMENT_LETTER     */ {  203200,   254000, INCHES },
   /* FORM_GOVERNMENT_LEGAL      */ {  203200,   330200, INCHES },
   /* FORM_EDP                   */ {  279400,   355600, INCHES },
   /* FORM_EUROPEAN_EDP          */ {  294000,   355600, INCHES },
   /* FORM_PRC_16K               */ {  146000,   215000, MM     },
   /* FORM_PRC_32K               */ {   97000,   151000, MM     },
   /* FORM_DAI_PA_KAI            */ {  275000,   395000, MM     },
   /* FORM_JUURO_KU_KAI          */ {  198000,   275000, MM     },
   /* FORM_PA_KAI                */ {  267000,   389000, MM     },
   /* FORM_WIDE                  */ {  345000,   279400, INCHES },
   /* FORM_LETTER_WIDE           */ {  228400,   337800, INCHES },
   /* FORM_FOOLSCAP_WIDE         */ {  215900,   330200, INCHES },
   /* FORM_A3_WIDE               */ {  330200,   482600, MM     },
   /* FORM_A4_WIDE               */ {  223500,   355600, MM     },
   /* FORM_WIDE_FORMAT           */ {  762000,  1066800, INCHES },
   /* FORM_10_X_11               */ {  254000,   279400, INCHES },
   /* FORM_11_X_12               */ {  279400,   304800, INCHES },
   /* FORM_11_X_15               */ {  279400,   381000, INCHES },
   /* FORM_12_X_19               */ {  304800,   482600, INCHES },
   /* FORM_15_X_11               */ {  381000,   279400, INCHES },
   /* FORM_5_X_7                 */ {  127000,   177800, INCHES },
   /* FORM_3_X_5_CARD            */ {   76200,   127000, INCHES },
   /* FORM_4_X_6_CARD            */ {  101600,   152400, INCHES },
   /* FORM_5_X_8_CARD            */ {  127000,   203200, INCHES },
   /* FORM_8_X_10_CARD           */ {  203200,   254000, INCHES },
   /* FORM_A6_CARD               */ {  104980,   148090, INCHES },
   /* FORM_CARD_148              */ {  148000,   105000, MM     },
   /* FORM_HAGAKI_CARD           */ {  100000,   148000, MM     },
   /* FORM_OUFUKU_CARD           */ {  148000,   200000, MM     },
   /* FORM_POSTCARD              */ {   99836,   147030, INCHES }, // was: { 96970, 147030, MM }
   /* FORM_INDEX_4_X_6_EXT       */ {  152400,   203200, INCHES },
   /* FORM_A2_ENVELOPE           */ {  111125,   146050, INCHES },
   /* FORM_C5_ENVELOPE           */ {  162000,   229000, INCHES },
   /* FORM_C6_ENVELOPE           */ {  114000,   162000, INCHES },
   /* FORM_C7_ENVELOPE           */ {   98400,   190500, INCHES },
   /* FORM_C9_ENVELOPE           */ {   98400,   225400, INCHES },
   /* FORM_C10_ENVELOPE          */ {  104700,   241300, INCHES },
   /* FORM_D5_ENVELOPE           */ {  176000,   250000, INCHES },
   /* FORM_DL_ENVELOPE           */ {  110000,   220000, INCHES },
   /* FORM_ENVELOPE_132_X_220    */ {  132000,   220000, INCHES },
   /* FORM_ENVELOPE_6_X_9        */ {  152400,   228600, INCHES },
   /* FORM_ENVELOPE_7_X_9        */ {  177800,   228600, INCHES },
   /* FORM_ENVELOPE_6_1_2        */ {  165100,    92070, INCHES },
   /* FORM_ENVELOPE_9_X_11       */ {  228600,   279400, INCHES },
   /* FORM_ENVELOPE_10_X_13      */ {  254000,   330200, INCHES },
   /* FORM_ENVELOPE_10_X_14      */ {  254000,   355600, INCHES },
   /* FORM_ENVELOPE_10_X_15      */ {  254000,   381000, INCHES },
   /* FORM_MONARCH_ENVELOPE      */ {   98425,   190500, INCHES },
   /* FORM_PERSONAL_ENVELOPE     */ {   92075,   165100, INCHES },
   /* FORM_NUMBER_9_ENVELOPE     */ {   98425,   225425, INCHES },
   /* FORM_NUMBER_10_ENVELOPE    */ {  104775,   241300, INCHES },
   /* FORM_NUMBER_11_ENVELOPE    */ {  114300,   263525, INCHES },
   /* FORM_NUMBER_12_ENVELOPE    */ {  120650,   279400, INCHES },
   /* FORM_NUMBER_14_ENVELOPE    */ {  127000,   292100, INCHES },
   /* FORM_ITALIAN_ENVELOPE      */ {  100000,   230000, MM     },
   /* FORM_POSTFIX_ENVELOPE      */ {  114000,   229000, MM     },
   /* FORM_INVITE_ENVELOPE       */ {  220000,   220000, MM     },
   /* FORM_CHOU2_ENVELOPE        */ {  111100,   146000, MM     },
   /* FORM_CHOU3_ENVELOPE        */ {  120000,   235000, MM     },
   /* FORM_CHOU4_ENVELOPE        */ {   90000,   205000, MM     },
   /* FORM_KAHU_ENVELOPE         */ {  240000,   322100, MM     },
   /* FORM_KAKU2_ENVELOPE        */ {  240000,   332000, MM     },
   /* FORM_YOU4_ENVELOPE         */ {  105000,   235000, MM     },
   /* FORM_PRC1_ENVELOPE         */ {  102000,   165000, MM     },
   /* FORM_PRC2_ENVELOPE         */ {  102000,   176000, MM     },
   /* FORM_PRC3_ENVELOPE         */ {  125000,   176000, MM     },
   /* FORM_PRC4_ENVELOPE         */ {  110000,   208000, MM     },
   /* FORM_PRC5_ENVELOPE         */ {  110000,   220000, MM     },
   /* FORM_PRC6_ENVELOPE         */ {  120000,   320000, MM     },
   /* FORM_PRC7_ENVELOPE         */ {  160000,   230000, MM     },
   /* FORM_PRC8_ENVELOPE         */ {  120000,   309000, MM     },
   /* FORM_PRC9_ENVELOPE         */ {  229000,   324000, MM     },
   /* FORM_PRC10_ENVELOPE        */ {  324000,   458000, MM     },
   /* FORM_DISK_LABELS           */ {   54000,    70000, INCHES },
   /* FORM_EURO_LABELS           */ {   36000,    89000, INCHES },
   /* FORM_SHIPPING_LABELS       */ {   54000,   101000, INCHES },
   /* FORM_STANDARD_LABELS_CLEAR */ {   28000,    89000, INCHES },
   /* FORM_STANDARD_LABELS_WHITE */ {   28000,    89000, INCHES },
   /* FORM_2A0                   */ { 1189000,  1682000, MM     },
   /* FORM_4A0                   */ { 1682000,  2378000, MM     },
   /* FORM_A0                    */ {  841000,  1189000, MM     },
   /* FORM_A1                    */ {  594000,   841000, MM     },
   /* FORM_A2                    */ {  420000,   594000, MM     },
   /* FORM_A3                    */ {  297000,   420000, MM     },
   /* FORM_A3_EXTRA              */ {  322000,   445000, MM     },
   /* FORM_A4                    */ {  210000,   297000, MM     },
   /* FORM_A4_EXTRA              */ {  235500,   322300, MM     },
   /* FORM_A4_TAB                */ {  225000,   297000, MM     },
   /* FORM_A5                    */ {  148000,   210000, MM     },
   /* FORM_A5_EXTRA              */ {  174000,   235000, MM     },
   /* FORM_A6                    */ {  105000,   148000, MM     },
   /* FORM_A7                    */ {   74000,   105000, MM     },
   /* FORM_A8                    */ {   52000,    74000, MM     },
   /* FORM_A9                    */ {   37000,    52000, MM     },
   /* FORM_A10                   */ {   26000,    37000, MM     },
   /* FORM_B_PLUS                */ {  304800,   486918, INCHES },
   /* FORM_B0                    */ { 1000000,  1414000, MM     },
   /* FORM_B1                    */ {  707000,  1000000, MM     },
   /* FORM_B2                    */ {  500000,   707000, MM     },
   /* FORM_B3                    */ {  353000,   500000, MM     },
   /* FORM_B4                    */ {  250000,   353000, MM     },
   /* FORM_B5                    */ {  176000,   250000, MM     },
   /* FORM_B5_EXTRA              */ {  201000,   276000, MM     },
   /* FORM_B6                    */ {  125000,   176000, MM     },
   /* FORM_B6_C4                 */ {  125000,   324000, MM     },
   /* FORM_B7                    */ {   88000,   125000, MM     },
   /* FORM_B8                    */ {   62000,    88000, MM     },
   /* FORM_B9                    */ {   44000,    62000, MM     },
   /* FORM_B10                   */ {   31000,    44000, MM     },
   /* FORM_JIS_B0                */ { 1030000,  1456000, MM     },
   /* FORM_JIS_B1                */ {  728000,  1030000, MM     },
   /* FORM_JIS_B2                */ {  515000,   728000, MM     },
   /* FORM_JIS_B3                */ {  364000,   515000, MM     },
   /* FORM_JIS_B4                */ {  257000,   364000, MM     },
   /* FORM_JIS_B5                */ {  182000,   257000, MM     },
   /* FORM_JIS_B6                */ {  128000,   182000, MM     },
   /* FORM_JIS_B7                */ {   91000,   128000, MM     },
   /* FORM_JIS_B8                */ {   64000,    91000, MM     },
   /* FORM_JIS_B9                */ {   45000,    64000, MM     },
   /* FORM_JIS_B10               */ {   32000,    45000, MM     },
   /* FORM_JIS_EXEC              */ {  216000,   330000, MM     },
   /* FORM_C0                    */ {  917000,  1297000, MM     },
   /* FORM_C1                    */ {  648000,   917000, MM     },
   /* FORM_C2                    */ {  458000,   648000, MM     },
   /* FORM_C3                    */ {  324000,   458000, MM     },
   /* FORM_C4                    */ {  229000,   324000, MM     },
   /* FORM_C5                    */ {  162000,   229000, MM     },
   /* FORM_C6                    */ {  114000,   162000, MM     },
   /* FORM_C6_C5                 */ {  114000,   229000, MM     },
   /* FORM_C7                    */ {   81000,   114000, MM     },
   /* FORM_C7_C6                 */ {   81000,   162000, MM     },
   /* FORM_C8                    */ {   57000,    81000, MM     },
   /* FORM_C9                    */ {   40000,    57000, MM     },
   /* FORM_C10                   */ {   28000,    40000, MM     },
   /* FORM_E1                    */ {  711200,  1016000, MM     },
   /* FORM_RA0                   */ {  860000,  1220000, MM     },
   /* FORM_RA1                   */ {  610000,   860000, MM     },
   /* FORM_RA2                   */ {  430000,   610000, MM     },
   /* FORM_SRA0                  */ {  900000,  1280000, MM     },
   /* FORM_SRA1                  */ {  640000,   900000, MM     },
   /* FORM_SRA2                  */ {  450000,   640000, MM     },
   /* FORM_US_FANFOLD            */ {  377825,   279400, INCHES },
   /* FORM_FANFOLD_1             */ {  368300,   279400, INCHES },
   /* FORM_FANFOLD_2             */ {  304800,   215900, INCHES },
   /* FORM_FANFOLD_3             */ {  241300,   279400, INCHES },
   /* FORM_FANFOLD_4             */ {  215900,   304800, INCHES },
   /* FORM_FANFOLD_5             */ {  215900,   279400, INCHES },
   /* FORM_GERMAN_LEGAL_FANFOLD  */ {  215900,   330200, INCHES },
   /* FORM_EUROPEAN_FANFOLD      */ {  215900,   304800, INCHES },
   /* FORM_PANORAMIC             */ {  210000,   594000, MM     },
   /* FORM_PHOTO_4_6             */ {  101600,   152400, INCHES },
   /* FORM_PHOTO_100_150         */ {  100000,   150000, MM     },
   /* FORM_PHOTO_200_300         */ {  200000,   300000, MM     },
   /* FORM_PLOTTER_SIZE_A        */ {  215900,   279400, INCHES },
   /* FORM_PLOTTER_SIZE_B        */ {  279400,   431800, INCHES },
   /* FORM_PLOTTER_SIZE_C        */ {  431800,   558800, INCHES },
   /* FORM_PLOTTER_SIZE_D        */ {  558800,   863600, INCHES },
   /* FORM_PLOTTER_SIZE_E        */ {  863690,  1117600, INCHES },
   /* FORM_PLOTTER_SIZE_F        */ { 1117600,  1727200, INCHES },
   /* FORM_US_A_ARCHITECTURAL    */ {  228600,   304800, INCHES },
   /* FORM_US_B_ARCHITECTURAL    */ {  304800,   457200, INCHES },
   /* FORM_US_C_ARCHITECTURAL    */ {  457200,   609600, INCHES },
   /* FORM_US_D_ARCHITECTURAL    */ {  609600,   914400, INCHES },
   /* FORM_US_E_ARCHITECTURAL    */ {  914400,  1219200, INCHES },
   /* FORM_SUPER_A3_B            */ {  328920,   483020, MM     },
   /* FORM_SUPER_A               */ {  227076,   355600, MM     },
   /* FORM_SUPER_B               */ {  330200,   482600, MM     },
   /* FORM_ROLL_76_2MM           */ {   76200,        0, MM     },
   /* FORM_ROLL_69_5MM           */ {   69500,        0, MM     }
};

DeviceForm::
DeviceForm (Device      *pDevice,
            int          id,
            int          iCapabilities,
            BinaryData  *data,
            HardCopyCap *hcInfo)
{
   pDevice_d       = pDevice;
   id_d            = id;
   iCapabilities_d = iCapabilities;
   data_d          = data;
   hcInfo_d        = hcInfo;

   iXOverride_d    = 0;
   iYOverride_d    = 0;

   if (hcInfo)
   {
      hcInfo->setOwner (this);
   }
}

DeviceForm::
~DeviceForm ()
{
   delete data_d;
   delete hcInfo_d;

   pDevice_d       = 0;
   id_d            = 0;
   iCapabilities_d = 0;
   data_d          = 0;
   hcInfo_d        = 0;
   iXOverride_d    = 0;
   iYOverride_d    = 0;
}

char * DeviceForm::
getName (Device *pDevice, int id)
{
   return StringResource::getString (pDevice->getLanguageResource (),
                                     StringResource::STRINGGROUP_FORMS,
                                     id);
}

char * DeviceForm::
getName ()
{
   return StringResource::getString (pDevice_d->getLanguageResource (),
                                     StringResource::STRINGGROUP_FORMS,
                                     id_d);
}

bool DeviceForm::
isID (int id)
{
   return id == id_d;
}

int DeviceForm::
getID ()
{
   return id_d;
}

char * DeviceForm::
getDeviceID ()
{
   return 0;
}

bool DeviceForm::
hasCapability (int iFlag)
{
   return 0 != (iFlag & iCapabilities_d);
}

int DeviceForm::
getCapabilities ()
{
   return iCapabilities_d;
}

BinaryData * DeviceForm::
getData ()
{
   return data_d;
}

HardCopyCap * DeviceForm::
getHardCopyCap ()
{
   return hcInfo_d;
}

void DeviceForm::
associateWith (DeviceResolution *pResolution)
{
   hcInfo_d->associateWith (pResolution);
}

DeviceForm * DeviceForm::
create (char *pszId)
{
   return this->createV (nameToID (pszId));
}

int DeviceForm::
nameToID (char *pszId)
{
   if (  !pszId
      || !*pszId
      )
   {
      return FORM_UNLISTED;
   }

   int iFormID = omniNameToID (pszId);

   if (FORM_UNLISTED != iFormID)
      return iFormID;

   iFormID = standardNameToID (pszId);

   if (FORM_UNLISTED != iFormID)
      return iFormID;

   iFormID = shortStandardNameToID (pszId);

   return iFormID;
}

int
omniNameToID (char *pszId)
{
   static short int asiEntries[] = {
      DeviceForm::FORM_10_X_11,
      DeviceForm::FORM_11_X_12,
      DeviceForm::FORM_11_X_15,
      DeviceForm::FORM_12_X_19,
      DeviceForm::FORM_15_X_11,
      DeviceForm::FORM_2A0,
      DeviceForm::FORM_3_X_5_CARD,
      DeviceForm::FORM_4A0,
      DeviceForm::FORM_4_X_6_CARD,
      DeviceForm::FORM_5_X_7,
      DeviceForm::FORM_5_X_8_CARD,
      DeviceForm::FORM_8_X_10_CARD,
      DeviceForm::FORM_A0,
      DeviceForm::FORM_A1,
      DeviceForm::FORM_A10,
      DeviceForm::FORM_A2,
      DeviceForm::FORM_A2_ENVELOPE,
      DeviceForm::FORM_A3,
      DeviceForm::FORM_A3_EXTRA,
      DeviceForm::FORM_A3_WIDE,
      DeviceForm::FORM_A4,
      DeviceForm::FORM_A4_EXTRA,
      DeviceForm::FORM_A4_TAB,
      DeviceForm::FORM_A4_WIDE,
      DeviceForm::FORM_A5,
      DeviceForm::FORM_A5_EXTRA,
      DeviceForm::FORM_A6,
      DeviceForm::FORM_A6_CARD,
      DeviceForm::FORM_A7,
      DeviceForm::FORM_A8,
      DeviceForm::FORM_A9,
      DeviceForm::FORM_B0,
      DeviceForm::FORM_B1,
      DeviceForm::FORM_B10,
      DeviceForm::FORM_B2,
      DeviceForm::FORM_B3,
      DeviceForm::FORM_B4,
      DeviceForm::FORM_B5,
      DeviceForm::FORM_B5_EXTRA,
      DeviceForm::FORM_B6,
      DeviceForm::FORM_B6_C4,
      DeviceForm::FORM_B7,
      DeviceForm::FORM_B8,
      DeviceForm::FORM_B9,
      DeviceForm::FORM_B_PLUS,
      DeviceForm::FORM_C0,
      DeviceForm::FORM_C1,
      DeviceForm::FORM_C10,
      DeviceForm::FORM_C10_ENVELOPE,
      DeviceForm::FORM_C2,
      DeviceForm::FORM_C3,
      DeviceForm::FORM_C4,
      DeviceForm::FORM_C5,
      DeviceForm::FORM_C5_ENVELOPE,
      DeviceForm::FORM_C6,
      DeviceForm::FORM_C6_C5,
      DeviceForm::FORM_C6_ENVELOPE,
      DeviceForm::FORM_C7,
      DeviceForm::FORM_C7_C6,
      DeviceForm::FORM_C7_ENVELOPE,
      DeviceForm::FORM_C8,
      DeviceForm::FORM_C9,
      DeviceForm::FORM_C9_ENVELOPE,
      DeviceForm::FORM_CARD_148,
      DeviceForm::FORM_CHOU2_ENVELOPE,
      DeviceForm::FORM_CHOU3_ENVELOPE,
      DeviceForm::FORM_CHOU4_ENVELOPE,
      DeviceForm::FORM_D5_ENVELOPE,
      DeviceForm::FORM_DAI_PA_KAI,
      DeviceForm::FORM_DISK_LABELS,
      DeviceForm::FORM_DL_ENVELOPE,
      DeviceForm::FORM_E1,
      DeviceForm::FORM_EDP,
      DeviceForm::FORM_ENVELOPE_10_X_13,
      DeviceForm::FORM_ENVELOPE_10_X_14,
      DeviceForm::FORM_ENVELOPE_10_X_15,
      DeviceForm::FORM_ENVELOPE_132_X_220,
      DeviceForm::FORM_ENVELOPE_6_1_2,
      DeviceForm::FORM_ENVELOPE_6_X_9,
      DeviceForm::FORM_ENVELOPE_7_X_9,
      DeviceForm::FORM_ENVELOPE_9_X_11,
      DeviceForm::FORM_EUROPEAN_EDP,
      DeviceForm::FORM_EUROPEAN_FANFOLD,
      DeviceForm::FORM_EURO_LABELS,
      DeviceForm::FORM_EXECUTIVE,
      DeviceForm::FORM_FANFOLD_1,
      DeviceForm::FORM_FANFOLD_2,
      DeviceForm::FORM_FANFOLD_3,
      DeviceForm::FORM_FANFOLD_4,
      DeviceForm::FORM_FANFOLD_5,
      DeviceForm::FORM_FOLIO,
      DeviceForm::FORM_FOLIO_SP,
      DeviceForm::FORM_FOOLSCAP,
      DeviceForm::FORM_FOOLSCAP_WIDE,
      DeviceForm::FORM_GERMAN_LEGAL_FANFOLD,
      DeviceForm::FORM_GOVERNMENT_LEGAL,
      DeviceForm::FORM_GOVERNMENT_LETTER,
      DeviceForm::FORM_HAGAKI_CARD,
      DeviceForm::FORM_HALF_LETTER,
      DeviceForm::FORM_INDEX_4_X_6_EXT,
      DeviceForm::FORM_INVITE_ENVELOPE,
      DeviceForm::FORM_ITALIAN_ENVELOPE,
      DeviceForm::FORM_JIS_B0,
      DeviceForm::FORM_JIS_B1,
      DeviceForm::FORM_JIS_B10,
      DeviceForm::FORM_JIS_B2,
      DeviceForm::FORM_JIS_B3,
      DeviceForm::FORM_JIS_B4,
      DeviceForm::FORM_JIS_B5,
      DeviceForm::FORM_JIS_B6,
      DeviceForm::FORM_JIS_B7,
      DeviceForm::FORM_JIS_B8,
      DeviceForm::FORM_JIS_B9,
      DeviceForm::FORM_JIS_EXEC,
      DeviceForm::FORM_JUURO_KU_KAI,
      DeviceForm::FORM_KAHU_ENVELOPE,
      DeviceForm::FORM_KAKU2_ENVELOPE,
      DeviceForm::FORM_LEDGER,
      DeviceForm::FORM_LEGAL,
      DeviceForm::FORM_LEGAL_EXTRA,
      DeviceForm::FORM_LETTER,
      DeviceForm::FORM_LETTER_EXTRA,
      DeviceForm::FORM_LETTER_PLUS,
      DeviceForm::FORM_LETTER_WIDE,
      DeviceForm::FORM_MONARCH_ENVELOPE,
      DeviceForm::FORM_NONE,
      DeviceForm::FORM_NUMBER_10_ENVELOPE,
      DeviceForm::FORM_NUMBER_11_ENVELOPE,
      DeviceForm::FORM_NUMBER_12_ENVELOPE,
      DeviceForm::FORM_NUMBER_14_ENVELOPE,
      DeviceForm::FORM_NUMBER_9_ENVELOPE,
      DeviceForm::FORM_OUFUKU_CARD,
      DeviceForm::FORM_PANORAMIC,
      DeviceForm::FORM_PA_KAI,
      DeviceForm::FORM_PERSONAL_ENVELOPE,
      DeviceForm::FORM_PHOTO_100_150,
      DeviceForm::FORM_PHOTO_200_300,
      DeviceForm::FORM_PHOTO_4_6,
      DeviceForm::FORM_PLOTTER_SIZE_A,
      DeviceForm::FORM_PLOTTER_SIZE_B,
      DeviceForm::FORM_PLOTTER_SIZE_C,
      DeviceForm::FORM_PLOTTER_SIZE_D,
      DeviceForm::FORM_PLOTTER_SIZE_E,
      DeviceForm::FORM_PLOTTER_SIZE_F,
      DeviceForm::FORM_POSTCARD,
      DeviceForm::FORM_POSTFIX_ENVELOPE,
      DeviceForm::FORM_PRC10_ENVELOPE,
      DeviceForm::FORM_PRC1_ENVELOPE,
      DeviceForm::FORM_PRC2_ENVELOPE,
      DeviceForm::FORM_PRC3_ENVELOPE,
      DeviceForm::FORM_PRC4_ENVELOPE,
      DeviceForm::FORM_PRC5_ENVELOPE,
      DeviceForm::FORM_PRC6_ENVELOPE,
      DeviceForm::FORM_PRC7_ENVELOPE,
      DeviceForm::FORM_PRC8_ENVELOPE,
      DeviceForm::FORM_PRC9_ENVELOPE,
      DeviceForm::FORM_PRC_16K,
      DeviceForm::FORM_PRC_32K,
      DeviceForm::FORM_QUARTO,
      DeviceForm::FORM_RA0,
      DeviceForm::FORM_RA1,
      DeviceForm::FORM_RA2,
      DeviceForm::FORM_ROC_16K,
      DeviceForm::FORM_ROC_8K,
      DeviceForm::FORM_ROLL_69_5MM,
      DeviceForm::FORM_ROLL_76_2MM,
      DeviceForm::FORM_SHIPPING_LABELS,
      DeviceForm::FORM_SRA0,
      DeviceForm::FORM_SRA1,
      DeviceForm::FORM_SRA2,
      DeviceForm::FORM_STANDARD_LABELS_CLEAR,
      DeviceForm::FORM_STANDARD_LABELS_WHITE,
      DeviceForm::FORM_STATEMENT,
      DeviceForm::FORM_SUPER_A,
      DeviceForm::FORM_SUPER_A3_B,
      DeviceForm::FORM_SUPER_B,
      DeviceForm::FORM_TABLOID,
      DeviceForm::FORM_TEST,
      DeviceForm::FORM_TEST2,
      DeviceForm::FORM_UNIVERSAL,
      DeviceForm::FORM_UNLISTED,
      DeviceForm::FORM_USER_DEFINED,
      DeviceForm::FORM_US_A_ARCHITECTURAL,
      DeviceForm::FORM_US_B_ARCHITECTURAL,
      DeviceForm::FORM_US_C_ARCHITECTURAL,
      DeviceForm::FORM_US_D_ARCHITECTURAL,
      DeviceForm::FORM_US_E_ARCHITECTURAL,
      DeviceForm::FORM_US_FANFOLD,
      DeviceForm::FORM_WIDE,
      DeviceForm::FORM_WIDE_FORMAT,
      DeviceForm::FORM_YOU4_ENVELOPE
   };

   int iLow  = 0;
   int iMid  = (int)dimof (asiEntries) / 2;
   int iHigh = (int)dimof (asiEntries) - 1;
   int iResult;

   while (iLow <= iHigh)
   {
      iResult = strcmp (pszId, STRING_OF (asiEntries[iMid]));

      if (0 == iResult)
      {
         return asiEntries[iMid];
      }
      else if (0 > iResult)
      {
         // str1 < str2
         iHigh = iMid - 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
      else // (0 < iResult)
      {
         // str1 > str2
         iLow  = iMid + 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
   }

   return DeviceForm::FORM_UNLISTED;
}

int
standardNameToID (char *pszId)
{
   int   iLow          = 0;
   int   iMid          = (int)dimof (vaStandardFormMapping) / 2;
   int   iHigh         = (int)dimof (vaStandardFormMapping) - 1;
   int   iResult;
   char *pszUnderscore = 0;

   pszUnderscore = strchr (pszId, '_');
   if (pszUnderscore)
      pszUnderscore = strchr (pszUnderscore + 1, '_');
   if (pszUnderscore)
      *pszUnderscore = '\0';

   while (iLow <= iHigh)
   {
      iResult = strcmp (pszId,
                        vaStandardFormMapping[iMid].pszUPDFName);

      if (0 == iResult)
      {
         if (pszUnderscore)
            *pszUnderscore = '_';

         return vaStandardFormMapping[iMid].iOmniFormID;
      }
      else if (0 > iResult)
      {
         // str1 < str2
         iHigh = iMid - 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
      else // (0 < iResult)
      {
         // str1 > str2
         iLow  = iMid + 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
   }

   if (pszUnderscore)
      *pszUnderscore = '_';

   return DeviceForm::FORM_UNLISTED;
}

int
shortStandardNameToID (char *pszId)
{
   int   iLow          = 0;
   int   iMid          = (int)dimof (vaShortFormMapping) / 2;
   int   iHigh         = (int)dimof (vaShortFormMapping) - 1;
   int   iResult;
   char *pszUnderscore = 0;

   // If the name starts with a known prefix that has an underscore in
   // it, then fail this function
   for (int i = 0; i < (int)dimof (aPrefixes); i++)
   {
      if (0 == strncmp (pszId,
                        aPrefixes[i].pszName,
                        aPrefixes[i].cbName))
      {
         return DeviceForm::FORM_UNLISTED;
      }
   }

   pszUnderscore = strchr (pszId, '_');
   if (pszUnderscore)
      *pszUnderscore = '\0';

   while (iLow <= iHigh)
   {
      iResult = strcmp (pszId,
                        vaShortFormMapping[iMid].pszUPDFName);

      if (0 == iResult)
      {
         if (pszUnderscore)
            *pszUnderscore = '_';

         return vaShortFormMapping[iMid].iOmniFormID;
      }
      else if (0 > iResult)
      {
         // str1 < str2
         iHigh = iMid - 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
      else // (0 < iResult)
      {
         // str1 > str2
         iLow  = iMid + 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
   }

   if (pszUnderscore)
      *pszUnderscore = '_';

   return DeviceForm::FORM_UNLISTED;
}

char * DeviceForm::
IDToName (int id)
{
   id -= FORM_UNLISTED;

   if (  0 <= id
      && id < (int)dimof (apszOmniFormNames)
      )
   {
#ifdef USE_STANDARD_NAMES
      if (  !apszStandardFormNames[id]
         || 1 >= id                    // FORM_UNLISTED or FORM_NONE
         )
         return apszOmniFormNames[id];

      static char achTemp[128];  // @TBD
      float       flX;
      float       flY;
      char       *pszUnits;

      if (INCHES == vaFormSizes[id - 2].eExternalDisplayUnits)
      {
         flX      = (float)vaFormSizes[id - 2].cx / 25400.0;
         flY      = (float)vaFormSizes[id - 2].cy / 25400.0;
         pszUnits = "in";
      }
      else
      {
         flX      = (float)vaFormSizes[id - 2].cx / 100.0;
         flY      = (float)vaFormSizes[id - 2].cy / 100.0;
         pszUnits = "mm";
      }

      sprintf (achTemp,
               "%s_%2.2fx%2.2f%s",
               apszStandardFormNames[id],
               flX,
               flY,
               pszUnits);

      return achTemp;
#else

      return apszOmniFormNames[id];

#endif
   }
   else
   {
      static char achUnknown[21];

      sprintf (achUnknown, "Unknown (%d)", id + FORM_UNLISTED);

      return achUnknown;
   }
}

int DeviceForm::
sizeToID (int cx,
          int cy)
{
   for (int i = 0; i < (int)dimof (vaFormSizes); i++)
   {
      if (  cx == vaFormSizes[i].cx
         && cy == vaFormSizes[i].cy
         )
      {
         return i + 1;
      }
   }

   return DeviceForm::FORM_UNLISTED;
}

int DeviceForm::
getCx ()
{
   if (iXOverride_d)
   {
      return iXOverride_d;
   }

   if (  DeviceForm::FORM_USER_DEFINED > id_d
      || DeviceForm::FORM_ROLL_69_5MM  < id_d
      )
   {
      return 0;
   }

   return vaFormSizes[id_d - DeviceForm::FORM_USER_DEFINED].cx;
}

int DeviceForm::
getCy ()
{
   if (iYOverride_d)
   {
      return iYOverride_d;
   }

   if (  DeviceForm::FORM_USER_DEFINED > id_d
      || DeviceForm::FORM_ROLL_69_5MM  < id_d
      )
   {
      return 0;
   }

   return vaFormSizes[id_d - DeviceForm::FORM_USER_DEFINED].cy;
}

void DeviceForm::
setCx (int iXOverride)
{
   iXOverride_d = iXOverride;
}

void DeviceForm::
setCy (int iYOverride)
{
   iYOverride_d = iYOverride;
}

int DeviceForm::
getOverrideX ()
{
   return iXOverride_d;
}

int DeviceForm::
getOverrideY ()
{
   return iYOverride_d;
}

typedef struct _ReservedMap {
   const char *pszName;
   int         iValue;
} RESERVEDMAP, *PRESERVEDMAP;

const RESERVEDMAP aReservedKeywords[] = {
   { "NO_CAPABILITIES",             0x00000000 },
   { "FORM_CAPABILITY_ROLL",        0x80000000 },
   { "FORM_CAPABILITY_USERDEFINED", 0x40000000 }
};

bool DeviceForm::
isReservedKeyword (const char *pszId)
{
   for (int i = 0; i < (int)dimof (aReservedKeywords); i++)
   {
      if (0 == strcmp (pszId, aReservedKeywords[i].pszName))
         return true;
   }

   return false;
}

int DeviceForm::
getReservedValue (const char *pszId)
{
   for (int i = 0; i < (int)dimof (aReservedKeywords); i++)
   {
      if (0 == strcmp (pszId, aReservedKeywords[i].pszName))
         return aReservedKeywords[i].iValue;
   }

   return 0;
}

string DeviceForm::
toString (ostringstream& oss)
{
   oss << "{"
       << STRING_OF (id_d)
       << ", iCapabilities = 0x" << hex << iCapabilities_d << dec
       << ", hcInfo_d = " << *hcInfo_d
       << "}";

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const DeviceForm& const_self)
{
   DeviceForm&   self = const_cast<DeviceForm&>(const_self);
   ostringstream oss;

   os << self.toString (oss);

   return os;
}
