/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _DevicePrintMode
#define _DevicePrintMode

#include "Device.hpp"

#include <iostream>
#include <sstream>
#include <string>

class DevicePrintMode
{
public:
   enum {
      PRINT_MODE_UNLISTED       = -1,
      PRINT_MODE_NONE,
      PRINT_MODE_1_ANY,
      PRINT_MODE_8_K,
      PRINT_MODE_8_CMY,
      PRINT_MODE_8_CMYK,
      PRINT_MODE_8_CcMmYK,
      PRINT_MODE_8_CcMmYyK,
      PRINT_MODE_8_RGB,
      PRINT_MODE_24_K,
      PRINT_MODE_24_CMY,
      PRINT_MODE_24_CMYK,
      PRINT_MODE_24_CcMmYK,
      PRINT_MODE_24_CcMmYyK,
      PRINT_MODE_24_RGB
   };
   enum {
      COUNT = PRINT_MODE_24_RGB - PRINT_MODE_UNLISTED + 1
   };

   enum {
      COLOR_TECH_K,
      COLOR_TECH_CMY,
      COLOR_TECH_CMYK,
      COLOR_TECH_CcMmYK,
      COLOR_TECH_CcMmYyK,
      COLOR_TECH_RGB
   };

   enum {
      COLOR_PLANE_CYAN,
      COLOR_PLANE_MAGENTA,
      COLOR_PLANE_YELLOW,
      COLOR_PLANE_BLACK,
      COLOR_PLANE_LIGHT_CYAN,
      COLOR_PLANE_LIGHT_MAGENTA,
      COLOR_PLANE_LIGHT_YELLOW
   };

                            DevicePrintMode      (Device *pDevice,
                                                  int     id,
                                                  int     iPhysicalCount,
                                                  int     iLogicalCount,
                                                  int     iPlanes);
   virtual                 ~DevicePrintMode      ();

   // By bringing named creates into base class, it creates a lot of complexity
   static  DevicePrintMode *create               (int                    id)    { return (DevicePrintMode *)0; }
   DevicePrintMode         *create               (char                  *pszId);
   static  int              nameToID             (char                  *pszId);
   static  char            *IDToName             (int                    id);

   static char             *getName              (Device                *pDevice,
                                                  int                    id);
   char                    *getName              ();

   bool                     isID                 (int                    id);
   int                      getID                ();
   virtual char            *getDeviceID          ();
   int                      getColorTech         ();
   int                      getPhysicalCount     ();  // How many colors the printer outputs
   int                      getLogicalCount      ();  // What the printer wants for bits per pel
   int                      getNumPlanes         ();  // What the printer wants for planes per bitmap

   virtual bool             isPrintModeSupported (int                    id) = 0;

   virtual Enumeration     *getEnumeration       () = 0;

   virtual string           toString             (ostringstream&         oss);
   friend ostream&          operator<<           (ostream&               os,
                                                  const DevicePrintMode& self);

protected:
   virtual DevicePrintMode *createV              (int                    id) = 0;

   Device  *pDevice_d;
   int      id_d;
   int      iPhysicalCount_d;
   int      iLogicalCount_d;
   int      iPlanes_d;
};

#endif
