/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "DeviceResolution.hpp"
#include "StringResource.hpp"
#include <stdio.h>
#include <string.h>

/* Function prototypes...
*/
static int    omniNameToID          (char             *pszId);
static int    resolutionNumberToID  (char             *pszId);
static int    standardNameToID      (char             *pszId);

#define STRING_OF(x) (apszResolutionNames[x - DeviceResolution::RESOLUTION_UNLISTED])

static char *apszResolutionNames[] = {
   "RESOLUTION_UNLISTED",
   "RESOLUTION_NONE",
   "RESOLUTION_60_X_60",
   "RESOLUTION_60_X_72",
   "RESOLUTION_60_X_180",
   "RESOLUTION_60_X_360",
   "RESOLUTION_72_X_72",
   "RESOLUTION_75_X_75",
   "RESOLUTION_80_X_60",
   "RESOLUTION_80_X_72",
   "RESOLUTION_90_X_180",
   "RESOLUTION_90_X_360",
   "RESOLUTION_90_X_60",
   "RESOLUTION_90_X_72",
   "RESOLUTION_90_X_90",
   "RESOLUTION_100_X_100",
   "RESOLUTION_120_X_60",
   "RESOLUTION_120_X_72",
   "RESOLUTION_120_X_120",
   "RESOLUTION_120_X_144",
   "RESOLUTION_120_X_180",
   "RESOLUTION_120_X_360",
   "RESOLUTION_144_X_72",
   "RESOLUTION_150_X_150",
   "RESOLUTION_180_X_180",
   "RESOLUTION_180_X_360",
   "RESOLUTION_200_X_200",
   "RESOLUTION_240_X_60",
   "RESOLUTION_240_X_72",
   "RESOLUTION_240_X_144",
   "RESOLUTION_240_X_240",
   "RESOLUTION_300_X_300",
   "RESOLUTION_360_X_180",
   "RESOLUTION_360_X_360",
   "RESOLUTION_360_X_720",
   "RESOLUTION_400_X_400",
   "RESOLUTION_600_X_300",
   "RESOLUTION_600_X_600",
   "RESOLUTION_720_X_360",
   "RESOLUTION_720_X_720",
   "RESOLUTION_1200_X_600",
   "RESOLUTION_1200_X_1200",
   "RESOLUTION_1440_X_720",
   "RESOLUTION_2880_X_720",
   "RESOLUTION_DRAFT",
   "RESOLUTION_FINE",
   "RESOLUTION_GROUP_3",
   "RESOLUTION_GROUP_4",
   "RESOLUTION_HIGH",
   "RESOLUTION_LOW",
   "RESOLUTION_MEDIUM",
   "RESOLUTION_NORMAL",
   "RESOLUTION_PHOTO_QUALITY",
   "RESOLUTION_PRESENTATION"
};

#ifdef USE_STANDARD_NAMES

static char *apszStandardFormNames[] = {
   "unlisted",
   "none",
   "60x60",
   "60x72",
   "60x180",
   "60x360",
   "72x72",
   "75x75",
   "80x60",
   "80x72",
   "90x180",
   "90x360",
   "90x60",
   "90x72",
   "90x90",
   "100x100",
   "120x60",
   "120x72",
   "120x120",
   "120x144",
   "120x180",
   "120x360",
   "144x72",
   "150x150",
   "180x180",
   "180x360",
   "200x200",
   "240x60",
   "240x72",
   "240x144",
   "240x240",
   "300x300",
   "360x180",
   "360x360",
   "360x720",
   "400x400",
   "600x300",
   "600x600",
   "720x360",
   "720x720",
   "1200x600",
   "1200x1200",
   "1440x720",
   "2880x720",
   "draft",
   "fine",
   "group-3",
   "group-4",
   "high",
   "low",
   "medium",
   "normal",
   "photo-quality",
   "presentation"
};

#endif

DeviceResolution::
DeviceResolution (Device     *pDevice,
                  int         id,
                  int         iXRes,
                  int         iYRes,
                  int         iXInternalRes,
                  int         iYInternalRes,
                  BinaryData *pbdData,
                  int         iCapabilities,
                  int         iDestinationBitsPerPel,
                  int         iScanlineMultiple)
{
   pDevice_d                = pDevice;
   id_d                     = id;
   iXRes_d                  = iXRes;
   iYRes_d                  = iYRes;
   iXInternalRes_d          = iXInternalRes;
   iYInternalRes_d          = iYInternalRes;
   pbdData_d                = pbdData;
   iCapabilities_d          = iCapabilities;
   iDestinationBitsPerPel_d = iDestinationBitsPerPel;
   iScanlineMultiple_d      = iScanlineMultiple;
}

DeviceResolution::
~DeviceResolution ()
{
   delete pbdData_d;

   pDevice_d                = 0;
   id_d                     = 0;
   iXRes_d                  = 0;
   iYRes_d                  = 0;
   iXInternalRes_d          = 0;
   iYInternalRes_d          = 0;
   pbdData_d                = 0;
   iCapabilities_d          = 0;
   iDestinationBitsPerPel_d = 0;
   iScanlineMultiple_d      = 0;
}

char * DeviceResolution::
getName (Device *pDevice, int id)
{
   return StringResource::getString (pDevice->getLanguageResource (),
                                     StringResource::STRINGGROUP_RESOLUTIONS,
                                     id);
}

char * DeviceResolution::
getName ()
{
   return StringResource::getString (pDevice_d->getLanguageResource (),
                                     StringResource::STRINGGROUP_RESOLUTIONS,
                                     id_d);
}

bool DeviceResolution::
isID (int id)
{
   return id == id_d;
}

int DeviceResolution::
getID ()
{
   return id_d;
}

char * DeviceResolution::
getDeviceID ()
{
   return 0;
}

int DeviceResolution::
getXRes ()
{
   if (iXInternalRes_d > 0)
   {
      return iXInternalRes_d;
   }
   else
   {
      return iXRes_d;
   }
}

int DeviceResolution::
getYRes ()
{
   if (iYInternalRes_d > 0)
   {
      return iYInternalRes_d;
   }
   else
   {
      return iYRes_d;
   }
}

int DeviceResolution::
getExternalXRes ()
{
   return iXRes_d;
}

int DeviceResolution::
getExternalYRes ()
{
   return iYRes_d;
}

int DeviceResolution::
getInternalXRes ()
{
   return iXInternalRes_d;
}

int DeviceResolution::
getInternalYRes ()
{
   return iYInternalRes_d;
}

BinaryData * DeviceResolution::
getData ()
{
   return pbdData_d;
}

bool DeviceResolution::
hasCapability (int iFlag)
{
   return 0 != (iFlag & iCapabilities_d);
}

int DeviceResolution::
getCapabilities ()
{
   return iCapabilities_d;
}

int DeviceResolution::
getDstBitsPerPel ()
{
   return iDestinationBitsPerPel_d;
}

int DeviceResolution::
getScanlineMultiple ()
{
   return iScanlineMultiple_d;
}

void DeviceResolution::
setInternalXRes (int iXInternalRes)
{
   iXInternalRes_d = iXInternalRes;
}

void DeviceResolution::
setInternalYRes (int iYInternalRes)
{
   iYInternalRes_d = iYInternalRes;
}

DeviceResolution * DeviceResolution::
create (char *pszId)
{
   return this->createV (nameToID (pszId));
}

int DeviceResolution::
nameToID (char *pszId)
{
   if (  !pszId
      || !*pszId
      )
   {
      return RESOLUTION_UNLISTED;
   }

   int iResolutionID = omniNameToID (pszId);

   if (RESOLUTION_UNLISTED != iResolutionID)
      return iResolutionID;

   iResolutionID = resolutionNumberToID (pszId);

   if (RESOLUTION_UNLISTED != iResolutionID)
      return iResolutionID;

   iResolutionID = standardNameToID (pszId);

   return iResolutionID;
}

int
omniNameToID (char *pszId)
{
   static short int asiEntries[] = {
      DeviceResolution::RESOLUTION_100_X_100,
      DeviceResolution::RESOLUTION_1200_X_1200,
      DeviceResolution::RESOLUTION_1200_X_600,
      DeviceResolution::RESOLUTION_120_X_120,
      DeviceResolution::RESOLUTION_120_X_144,
      DeviceResolution::RESOLUTION_120_X_180,
      DeviceResolution::RESOLUTION_120_X_360,
      DeviceResolution::RESOLUTION_120_X_60,
      DeviceResolution::RESOLUTION_120_X_72,
      DeviceResolution::RESOLUTION_1440_X_720,
      DeviceResolution::RESOLUTION_144_X_72,
      DeviceResolution::RESOLUTION_150_X_150,
      DeviceResolution::RESOLUTION_180_X_180,
      DeviceResolution::RESOLUTION_180_X_360,
      DeviceResolution::RESOLUTION_200_X_200,
      DeviceResolution::RESOLUTION_240_X_144,
      DeviceResolution::RESOLUTION_240_X_240,
      DeviceResolution::RESOLUTION_240_X_60,
      DeviceResolution::RESOLUTION_240_X_72,
      DeviceResolution::RESOLUTION_2880_X_720,
      DeviceResolution::RESOLUTION_300_X_300,
      DeviceResolution::RESOLUTION_360_X_180,
      DeviceResolution::RESOLUTION_360_X_360,
      DeviceResolution::RESOLUTION_360_X_720,
      DeviceResolution::RESOLUTION_400_X_400,
      DeviceResolution::RESOLUTION_600_X_300,
      DeviceResolution::RESOLUTION_600_X_600,
      DeviceResolution::RESOLUTION_60_X_180,
      DeviceResolution::RESOLUTION_60_X_360,
      DeviceResolution::RESOLUTION_60_X_60,
      DeviceResolution::RESOLUTION_60_X_72,
      DeviceResolution::RESOLUTION_720_X_360,
      DeviceResolution::RESOLUTION_720_X_720,
      DeviceResolution::RESOLUTION_72_X_72,
      DeviceResolution::RESOLUTION_75_X_75,
      DeviceResolution::RESOLUTION_80_X_60,
      DeviceResolution::RESOLUTION_80_X_72,
      DeviceResolution::RESOLUTION_90_X_180,
      DeviceResolution::RESOLUTION_90_X_360,
      DeviceResolution::RESOLUTION_90_X_60,
      DeviceResolution::RESOLUTION_90_X_72,
      DeviceResolution::RESOLUTION_90_X_90,
      DeviceResolution::RESOLUTION_DRAFT,
      DeviceResolution::RESOLUTION_FINE,
      DeviceResolution::RESOLUTION_GROUP_3,
      DeviceResolution::RESOLUTION_GROUP_4,
      DeviceResolution::RESOLUTION_HIGH,
      DeviceResolution::RESOLUTION_LOW,
      DeviceResolution::RESOLUTION_MEDIUM,
      DeviceResolution::RESOLUTION_NONE,
      DeviceResolution::RESOLUTION_NORMAL,
      DeviceResolution::RESOLUTION_PHOTO_QUALITY,
      DeviceResolution::RESOLUTION_PRESENTATION
   };

   int iLow  = 0;
   int iMid  = (int)dimof (asiEntries) / 2;
   int iHigh = (int)dimof (asiEntries) - 1;
   int iResult;

   while (iLow <= iHigh)
   {
      iResult = strcmp (pszId, STRING_OF (asiEntries[iMid]));

      if (0 == iResult)
      {
         return asiEntries[iMid];
      }
      else if (0 > iResult)
      {
         // str1 < str2
         iHigh = iMid - 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
      else // (0 < iResult)
      {
         // str1 > str2
         iLow  = iMid + 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
   }

   return DeviceResolution::RESOLUTION_UNLISTED;
}

typedef struct _ResolutionMap1 {
   int iOmniId;
   int iX;
   int iY;
} RESMAP1, *PRESMAP1;

static RESMAP1 aResMap1[] = {
   { DeviceResolution::RESOLUTION_60_X_60,       60,   60 },
   { DeviceResolution::RESOLUTION_60_X_72,       60,   72 },
   { DeviceResolution::RESOLUTION_60_X_180,      60,  180 },
   { DeviceResolution::RESOLUTION_60_X_360,      60,  360 },
   { DeviceResolution::RESOLUTION_72_X_72,       72,   72 },
   { DeviceResolution::RESOLUTION_75_X_75,       75,   75 },
   { DeviceResolution::RESOLUTION_80_X_60,       80,   60 },
   { DeviceResolution::RESOLUTION_80_X_72,       80,   72 },
   { DeviceResolution::RESOLUTION_90_X_60,       90,   60 },
   { DeviceResolution::RESOLUTION_90_X_72,       90,   72 },
   { DeviceResolution::RESOLUTION_90_X_90,       90,   90 },
   { DeviceResolution::RESOLUTION_90_X_180,      90,  180 },
   { DeviceResolution::RESOLUTION_90_X_360,      90,  360 },
   { DeviceResolution::RESOLUTION_100_X_100,    100,  100 },
   { DeviceResolution::RESOLUTION_120_X_60,     120,   60 },
   { DeviceResolution::RESOLUTION_120_X_72,     120,   72 },
   { DeviceResolution::RESOLUTION_120_X_120,    120,  120 },
   { DeviceResolution::RESOLUTION_120_X_144,    120,  144 },
   { DeviceResolution::RESOLUTION_120_X_180,    120,  180 },
   { DeviceResolution::RESOLUTION_120_X_360,    120,  360 },
   { DeviceResolution::RESOLUTION_144_X_72,     144,   72 },
   { DeviceResolution::RESOLUTION_150_X_150,    150,  150 },
   { DeviceResolution::RESOLUTION_180_X_180,    180,  180 },
   { DeviceResolution::RESOLUTION_180_X_360,    180,  360 },
   { DeviceResolution::RESOLUTION_200_X_200,    200,  200 },
   { DeviceResolution::RESOLUTION_240_X_60,     240,   60 },
   { DeviceResolution::RESOLUTION_240_X_72,     240,   72 },
   { DeviceResolution::RESOLUTION_240_X_144,    240,  144 },
   { DeviceResolution::RESOLUTION_240_X_240,    240,  240 },
   { DeviceResolution::RESOLUTION_300_X_300,    300,  300 },
   { DeviceResolution::RESOLUTION_360_X_180,    360,  180 },
   { DeviceResolution::RESOLUTION_360_X_360,    360,  360 },
   { DeviceResolution::RESOLUTION_360_X_720,    360,  720 },
   { DeviceResolution::RESOLUTION_400_X_400,    400,  400 },
   { DeviceResolution::RESOLUTION_600_X_300,    600,  300 },
   { DeviceResolution::RESOLUTION_600_X_600,    600,  600 },
   { DeviceResolution::RESOLUTION_720_X_360,    720,  360 },
   { DeviceResolution::RESOLUTION_720_X_720,    720,  720 },
   { DeviceResolution::RESOLUTION_1200_X_600,  1200,  600 },
   { DeviceResolution::RESOLUTION_1200_X_1200, 1200, 1200 },
   { DeviceResolution::RESOLUTION_1440_X_720,  1440,  720 },
   { DeviceResolution::RESOLUTION_2880_X_720,  2880,  720 }
};

typedef struct _ResolutionMap2 {
   int   iOmniId;
   char *pszId;
} RESMAP2, *PRESMAP2;

static RESMAP2 aResMap2[] = {
   { DeviceResolution::RESOLUTION_DRAFT,         "draft"         },
   { DeviceResolution::RESOLUTION_FINE,          "fine"          },
   { DeviceResolution::RESOLUTION_GROUP_3,       "group-3"       },
   { DeviceResolution::RESOLUTION_GROUP_4,       "group-4"       },
   { DeviceResolution::RESOLUTION_HIGH,          "high"          },
   { DeviceResolution::RESOLUTION_LOW,           "low"           },
   { DeviceResolution::RESOLUTION_MEDIUM,        "medium"        },
   { DeviceResolution::RESOLUTION_NONE,          "none"          },
   { DeviceResolution::RESOLUTION_NORMAL,        "normal"        },
   { DeviceResolution::RESOLUTION_PHOTO_QUALITY, "photo-quality" },
   { DeviceResolution::RESOLUTION_PRESENTATION,  "presentation"  }
};

int
resolutionNumberToID (char *pszId)
{
   char *pszScan         = pszId;
   char *pszSecondNumber = 0;
   int   iX              = 0;
   int   iY              = 0;

   if (  '0' > *pszScan
      || '9' < *pszScan
      )
   {
      return DeviceResolution::RESOLUTION_UNLISTED;
   }
   while (  '0' <= *pszScan
         && '9' >= *pszScan
         )
   {
      pszScan++;
   }
   if (  '\0' != *pszScan
      && 'x'  != *pszScan
      && 'X'  != *pszScan
      )
   {
      return DeviceResolution::RESOLUTION_UNLISTED;
   }
   if (*pszScan)
   {
      pszScan++;
      pszSecondNumber = pszScan;
      while (  '0' <= *pszScan
            && '9' >= *pszScan
            )
      {
         pszScan++;
      }
      if (*pszScan)
      {
         return DeviceResolution::RESOLUTION_UNLISTED;
      }
   }

   if (0 == sscanf (pszId, "%d", &iX))
   {
      return DeviceResolution::RESOLUTION_UNLISTED;
   }
   if (pszSecondNumber)
   {
      if (0 == sscanf (pszSecondNumber, "%d", &iY))
      {
         return DeviceResolution::RESOLUTION_UNLISTED;
      }
   }
   else
   {
      iY = iX;
   }

   int iLow  = 0;
   int iMid  = (int)dimof (aResMap1) / 2;
   int iHigh = (int)dimof (aResMap1) - 1;
   int iResult;

   while (iLow <= iHigh)
   {
      iResult = iX - aResMap1[iMid].iX;

//DebugOutput::getErrorStream () << "iX = "
//     << iX
//     << ", aResMap1["
//     << iMid
//     << "].iX = "
//     << aResMap1[iMid].iX
//     << ", iResult = "
//     << iResult
//     << endl;

      if (0 == iResult)
      {
         if (iY == aResMap1[iMid].iY)
         {
            return aResMap1[iMid].iOmniId;
         }
         else
         {
            iResult = iY - aResMap1[iMid].iY;

//DebugOutput::getErrorStream () << "iY = "
//     << iY
//     << ", aResMap1["
//     << iMid
//     << "].iY = "
//     << aResMap1[iMid].iY
//     << ", iResult = "
//     << iResult
//     << endl;
         }
      }
      if (0 > iResult)
      {
         // str1 < str2
         iHigh = iMid - 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
      else if (0 < iResult)
      {
         // str1 > str2
         iLow  = iMid + 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
   }

   return DeviceResolution::RESOLUTION_UNLISTED;
}

int
standardNameToID (char *pszId)
{
   int iLow  = 0;
   int iMid  = (int)dimof (aResMap2) / 2;
   int iHigh = (int)dimof (aResMap2) - 1;
   int iResult;

   while (iLow <= iHigh)
   {
      iResult = strcmp (pszId, aResMap2[iMid].pszId);

      if (0 == iResult)
      {
         return aResMap2[iMid].iOmniId;
      }
      else if (0 > iResult)
      {
         // str1 < str2
         iHigh = iMid - 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
      else // (0 < iResult)
      {
         // str1 > str2
         iLow  = iMid + 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
   }

   return DeviceResolution::RESOLUTION_UNLISTED;
}

char * DeviceResolution::
IDToName (int id)
{
   id -= RESOLUTION_UNLISTED;

   if (  0 <= id
      && id < (int)dimof (apszResolutionNames)
      )
   {
#ifdef USE_STANDARD_NAMES
      return apszStandardFormNames[id];
#else
      return apszResolutionNames[id];
#endif
   }
   else
   {
      static char achUnknown[21];

      sprintf (achUnknown, "Unknown (%d)", id + RESOLUTION_UNLISTED);

      return achUnknown;
   }
}

string DeviceResolution::
toString (ostringstream& oss)
{
   oss << "{"
       << STRING_OF (id_d)
       << ", iXRes_d = " << iXRes_d
       << ", iYRes_d = " << iYRes_d
       << hex << ", iCapabilities_d = 0x" << iCapabilities_d
       << dec << ", iDestinationBitsPerPel_d = " << iDestinationBitsPerPel_d
       << ", iScanlineMultiple_d = " << iScanlineMultiple_d
       << "}";

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const DeviceResolution& const_self)
{
   DeviceResolution& self = const_cast<DeviceResolution&>(const_self);
   ostringstream     oss;

   os << self.toString (oss);

   return os;
}
