/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2002
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "Device.hpp"
#include "Omni.hpp"

Device *
loadDevice (char *pszFullDeviceName, char *pszJobProperties)
{
   Device       *pDevice      = 0;
   PFNNEWDEVICE  pfnNewDevice = 0;
   GModule      *hmodDevice   = 0;

   cout << "Trying to load " << pszFullDeviceName << endl;

   hmodDevice = g_module_open (pszFullDeviceName, (GModuleFlags)0);

   if (DebugOutput::shouldOutputDeviceTester ()) DebugOutput::getErrorStream () << "hmodDevice = " << hex << hmodDevice << endl;

   if (!hmodDevice)
   {
      DebugOutput::getErrorStream () << "g_module_error returns " << g_module_error () << endl;
      return 0;
   }

   g_module_symbol (hmodDevice, "newDeviceW_Advanced", (gpointer *)&pfnNewDevice);

   if (DebugOutput::shouldOutputDeviceTester ()) DebugOutput::getErrorStream () << hex << "pfnNewDevice = 0x" << (int)pfnNewDevice << endl;

   if (!pfnNewDevice)
      return 0;

   pDevice = pfnNewDevice (true);

   return pDevice;
}

int
handleDevice (char *pszFullDeviceName, char *pszJobProperties)
{
   Device *pDevice = 0;

   if (strstr (pszFullDeviceName, "/lib"))
   {
      pszFullDeviceName = strstr (pszFullDeviceName, "/lib") + 1;
   }

#define PDC 1
#ifdef PDC
   pDevice = new OmniPDCProxy (0,                 // client exe to spawn
                               pszFullDeviceName, // device name
                               pszJobProperties,  // job properties
                               true);             // is renderer
#else
   pDevice = loadDevice (pszFullDeviceName, pszJobProperties);
#endif

   if (!pDevice)
   {
      DebugOutput::getErrorStream () << "DeviceTester9::" << __FUNCTION__ << ": OmniPDCProxy failed!" << endl;

      return 2;
   }

   if (pDevice->hasError ())
   {
      DebugOutput::getErrorStream () << "DeviceTester9::" << __FUNCTION__ << ": OmniPDCProxy has an error!" << endl;

      return 3;
   }

   Enumeration *pEnumKeys   = pDevice->listJobPropertyKeys ();
   Enumeration *pEnumValues = 0;

   while (pEnumKeys->hasMoreElements ())
   {
      char *pszKey = (char *)pEnumKeys->nextElement ();

      cout << pszKey << " = ";

      string *pValue = pDevice->getJobProperty (pszKey);

      cout << *pValue << endl;

      pEnumValues = pDevice->listKeyValues (pszKey);

      while (pEnumValues->hasMoreElements ())
      {
         char *pszValue = (char *)pEnumValues->nextElement ();

         cout << "\t"
              << pszValue;

         string *pRet = pDevice->translateKeyValue (pszKey, pszValue);

         if (pRet)
         {
            cout << "("
                 << *pRet
                 << ")";
         }

         cout << endl;

         delete pRet;
      }

      delete pValue;
      delete pEnumValues;
   }

   delete pEnumKeys;

   pEnumKeys = pDevice->listDeviceJobPropertyKeys ();

   while (pEnumKeys->hasMoreElements ())
   {
      char *pszKey = (char *)pEnumKeys->nextElement ();

      cout << pszKey << " = ";

      string *pValue = pDevice->getJobProperty (pszKey);

      cout << *pValue << endl;

      pEnumValues = pDevice->listKeyValues (pszKey);

      while (pEnumValues->hasMoreElements ())
      {
         char *pszValue = (char *)pEnumValues->nextElement ();

         cout << "\t"
              << pszValue;

         string *pRet = pDevice->translateKeyValue (pszKey, pszValue);

         if (pRet)
         {
            cout << "("
                 << *pRet
                 << ")";
         }

         cout << endl;

         delete pRet;
      }

      delete pValue;
      delete pEnumValues;
   }

   delete pEnumKeys;

   delete pDevice;

   return 0;
}

int
main (int argc, char *argv[])
{
   char *pszJobProperties = 0;
   int   iFailed = 0;

   if (!g_module_supported ())
   {
      DebugOutput::getErrorStream () << "This program needs glib's module routines!" << endl;
      return 1;
   }

   if (2 > argc)
   {
      cerr << "Usage: " << argv[0] << " printer_name" << endl;
      return 1;
   }

#ifndef RETAIL
   for (int i = 2; i + 1 <= argc; i += 2)
   {
      if (0 == strcmp (argv[i], "debugoutput"))
      {
         DebugOutput::setDebugOutput (argv[i + 1]);
      }
   }
#endif

   if (0 == strcasecmp (argv[1], "all"))
   {
      Enumeration *pEnum = Omni::listDevices ();

      while (pEnum->hasMoreElements ())
      {
         char *pszLibName = (char *)pEnum->nextElement ();

         cout << "Checking: " << pszLibName << endl;

         iFailed |= handleDevice (pszLibName, pszJobProperties);
      }

      delete pEnum;
   }
   else
   {
      char achLibDeviceName[512];  // @TBD

      if (0 != strncasecmp (argv[1], "lib", 3))
      {
         sprintf (achLibDeviceName, "lib%s.so", argv[1]);
      }
      else
      {
         strcpy (achLibDeviceName, argv[1]);
      }

      iFailed |= handleDevice (achLibDeviceName, pszJobProperties);
   }

   return iFailed;
}
