/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _DeviceTray
#define _DeviceTray

#include "Device.hpp"

#include <iostream>
#include <sstream>
#include <string>

class DeviceTray
{
public:
   enum {
      TRAY_UNLISTED         = -1,
      TRAY_NONE,
      TRAY_AUTO,            /* aka default */
      TRAY_AUTO_FEEDER,
      TRAY_AUTOSWITCH,
      TRAY_TRAY,
      TRAY_UPPER_TRAY,
      TRAY_LOWER_TRAY,
      TRAY_MULTI_TRAY,
      TRAY_CASSETTE,
      TRAY_UPPER_CASSETTE,
      TRAY_LOWER_CASSETTE,
      TRAY_MULTI_CASSETTE,
      TRAY_OPTION_CASSETTE,
      TRAY_MANUAL_FEEDER,
      TRAY_MANUAL_ENVELOPE,
      TRAY_PAPER_FEEDER,
      TRAY_ENVELOPE_FEEDER,
      TRAY_CSF,
      TRAY_FRONT_CONTINUOUS,
      TRAY_REAR_CONTINUOUS,
      TRAY_SINGLE_SHEET,
      TRAY_SHEET_FEEDER,
      TRAY_BIN_1,
      TRAY_BIN_2,
      TRAY_PORTABLE_SHEET,
      TRAY_TRAY_1,
      TRAY_TRAY_2,
      TRAY_TRAY_3,
      TRAY_TRAY_4,
      TRAY_TRAY_5,
      TRAY_CASSETTE1,
      TRAY_CASSETTE2,
      TRAY_CASSETTE3,
      TRAY_CASSETTE4,
      TRAY_CASSETTE5,
      TRAY_FRONT_TRAY,
      TRAY_DEPEND_ON_PANEL,
      TRAY_OPTION_MULTI_SF,
      TRAY_MULTI_FEEDER,
      TRAY_PAPER_DECK,
      TRAY_TRACTOR_UNIT,
      TRAY_ROLL_1,
      TRAY_ROLL_2,
      TRAY_ROLL_3,
      TRAY_ROLL_4,
      TRAY_ROLL_5,
      TRAY_ZERO_MARGINS
   };
   enum {
      COUNT = TRAY_ZERO_MARGINS - TRAY_UNLISTED + 1
   };

                       DeviceTray       (Device           *pDevice,
                                         int               id,
                                         int               iType,
                                         BinaryData       *pbdData);
   virtual            ~DeviceTray       ();

   // By bringing named creates into base class, it creates a lot of complexity
   static DeviceTray   *create          (int               id)    { return (DeviceTray *)0; }
   DeviceTray          *create          (char             *pszId);
   static int           nameToID        (char             *pszId);
   static char         *IDToName        (int               id);

   static char         *getName         (Device           *pDevice,
                                         int               id);
   char                *getName         ();

   bool                 isID            (int               id);
   int                  getID           ();
   virtual char        *getDeviceID     ();
   int                  getType         ();
   BinaryData          *getData         ();

   virtual bool         isTraySupported (int               id) = 0;

   virtual Enumeration *getEnumeration  () = 0;

   virtual string       toString        (ostringstream&    oss);
   friend ostream&      operator<<      (ostream&          os,
                                         const DeviceTray& self);

protected:
   virtual DeviceTray  *createV         (int               id) = 0;

   Device     *pDevice_d;
   int         id_d;
   int         iType_d;
   BinaryData *pbdData_d;
};

#endif
