/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "Epson_High_Res_ESCP2_Instance.hpp"
#include "defines.hpp"

#include <iostream>
#include <sstream>

const static bool fTestNoCompression = true;

Epson_High_Res_ESCP2_Instance::
Epson_High_Res_ESCP2_Instance (PrintDevice *pDevice)
   : DeviceInstance (pDevice)
{
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Epson_High_Res_ESCP2_Instance::Epson_High_Res_ESCP2_Instance ()" << endl;

   fHaveSetupPrinter_d = false;
   ptlPrintHead_d.x    = 0;
   ptlPrintHead_d.y    = 0;
   fHaveInitialized_d  = false;
}

Epson_High_Res_ESCP2_Instance::
~Epson_High_Res_ESCP2_Instance ()
{
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Epson_High_Res_ESCP2_Instance::~Epson_High_Res_ESCP2_Instance ()" << endl;

}

void Epson_High_Res_ESCP2_Instance::
initializeInstance ()
{
   if (fHaveInitialized_d)
      return;

   fHaveInitialized_d = true;

}

void Epson_High_Res_ESCP2_Instance::
setupPrinter ()
{
   if (fHaveSetupPrinter_d)
      return;

   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Epson_High_Res_ESCP2_Instance::setupPrinter ()" << endl;

   fHaveSetupPrinter_d = true;


   DeviceResolution *pRes      = getCurrentResolution ();
   DeviceCommand    *pCommands = getCommands ();
  // DeviceResolution *pDR       = getCurrentResolution ();
   BinaryData       *pCmd      = 0;
   DeviceData       *pData     = 0;  //@@PAZ
   pData     = getDeviceData ();  //@@PAZ
   int iXRes;

   if(!pData)
       DebugOutput::getErrorStream () << ">>>> getDeviceData Failed " << endl;
   pCmd = pCommands->getCommandData ("cmdBeginRasterGraphics");
   if (pCmd)
   {
      pDevice_d->sendBinaryDataToDevice (pCmd);
   }
   else
   {
      DebugOutput::getErrorStream () << dec << "Error: There is no cmdBeginRasterGraphics defined for this device!" << endl;
   }

   pCmd = pCommands->getCommandData ("cmdSetResolution");
   if (pCmd)
   {
      pDevice_d->sendPrintfToDevice (pCmd,
                                     1440 / pRes->getYRes (),
                                     1440 / pRes->getYRes (),
                                     1440 / pRes->getXRes (), // Horiz Res (X).
                                     1440);

   }
   else
   {
      DebugOutput::getErrorStream () << dec << "Error: There is no cmdSetResolution defined for this device!" << endl;
   }

   DeviceForm *pForm = getCurrentForm ();

   pCmd = pCommands->getCommandData ("cmdSetFormSize");
   if (pCmd)
   {
      int iNozzles = 48;
      int iPageLength = 0 ;
      int iTemp;
      int iUnits = pForm->getHardCopyCap ()->getYPels ();
      float fHeight = (pForm->getHardCopyCap ()->getCy ()/25400.0)* pRes->getYRes ();

      // @TBD - for some reason the printer thinks that we are beyond the end
      // of the page if we have gone past the end of the page counting the bottom
      // line and the number of nozzles in the head.  To stop this, we are adding
      // the number of nozzles to the end of the page to stop the extra page eject

      if(pData)
         pData->getIntData ("Nozzle_Number", &iNozzles);
      iTemp = (iUnits+iNozzles*2) < (int)fHeight ? (iUnits+iNozzles*2) : (int)fHeight;

      iPageLength |= (iTemp & 0x00FF) << 8;
      iPageLength |= (iTemp & 0xFF00) >> 8;

      pDevice_d->sendPrintfToDevice (pCmd, iPageLength);
   }
   else
   {
      DebugOutput::getErrorStream () << dec << "Error: There is no cmdSetFormSize defined for this device!" << endl;
   }

   pCmd = pCommands->getCommandData ("cmdSetPageSize");
   if (pCmd)
   {
      int iWidth, iHeight;
      iWidth = iHeight = 0;

      float fWidth = (pForm->getHardCopyCap ()->getCx ()/25400.0)* pRes->getXRes ();
      float fHeight = (pForm->getHardCopyCap ()->getCy ()/25400.0)* pRes->getYRes ();

      // flip the integer data around

      iWidth |= ((int)fWidth & 0x00FF) << 8 ;
      iWidth |= ((int)fWidth & 0xFF00) >> 8;
      iHeight |= ((int)fHeight & 0x00FF) << 8 ;
      iHeight |= ((int)fHeight & 0xFF00) >> 8;

//    pDevice_d->sendPrintfToDevice (pCmd, (int)fWidth,
//                                   (int)fHeight);

      pDevice_d->sendPrintfToDevice (pCmd, iWidth,
                                     iHeight );
   }
   else
   {
      DebugOutput::getErrorStream () << dec << "Error: There is no cmdSetPageSize defined for this device!" << endl;
   }

   // @TBD
   fUseMicroweave_d = false;

   if (pRes->isID (DeviceResolution::RESOLUTION_720_X_720))
   {
      if (pRes->getDstBitsPerPel () == 1)
      {

          DebugOutput::getErrorStream () << dec << "DstBitsPerPel = " << pRes->getDstBitsPerPel () << endl;
          fUseMicroweave_d = true;
      }
   }

   pCmd = pCommands->getCommandData ("cmdBidi");
   if (pCmd)
   {
      char chState;

      if (fUseMicroweave_d)
         // Turn off bidirectional for microweaving
         chState = '0';
      else
         chState = '1';
      pDevice_d->sendPrintfToDevice (pCmd, chState);
   }
   else
   {
      DebugOutput::getErrorStream () << dec << "Error: There is no cmdBidi defined for this device!" << endl;
   }

   // @TBD
///pDevice_d->sendBinaryDataToDevice (getCurrentTray ());

   pCmd = pCommands->getCommandData ("cmdSetHardwareMicroweave");
   if (pCmd)
   {
      if (fUseMicroweave_d)
         pDevice_d->sendPrintfToDevice (pCmd, 1);
      else
         pDevice_d->sendPrintfToDevice (pCmd, 0);
   }
   else
   {
      DebugOutput::getErrorStream () << dec << "Error: There is no cmdSetHardwareMicroweave defined for this device!" << endl;
   }

      iXRes = pRes->getXRes();

      if(pData)
      {
          int iDotValue;
          pCmd = pCommands->getCommandData ("cmdSetDotSizeInt");
          if(pCmd)
          {
              switch(iXRes)
              {
              case 1440:
                  pData->getIntData ("DotSize1440", &iDotValue);
                  pDevice_d->sendPrintfToDevice (pCmd, iDotValue);
                  break;

              case 720:
                  pData->getIntData ("DotSize720", &iDotValue);
                  pDevice_d->sendPrintfToDevice (pCmd, iDotValue);
                  break;

              case 360:
                  pData->getIntData ("DotSize360", &iDotValue);
                  pDevice_d->sendPrintfToDevice (pCmd, iDotValue);
              }
          }
          else
          {
              DebugOutput::getErrorStream () << dec << "Error: There is no cmdSetDotSizeInt defined for this device!" << endl;
          }
      }
      else
      {
          pCmd = pCommands->getCommandData ("cmdSetDotSize");
          if(pCmd)
          {
              switch(iXRes)
              {
              case 1440:
                  pDevice_d->sendPrintfToDevice (pCmd, 0x1000);
                  break;

              case 720:
                  pDevice_d->sendPrintfToDevice (pCmd, 0x1100);
                  break;

              case 360:
                  pDevice_d->sendPrintfToDevice (pCmd, 0x1200);
              }
          }
          else
          {
              DebugOutput::getErrorStream () << dec << "Error: There is no cmdSetDotSize defined for this device!" << endl;
          }
      }

   pCmd = pCommands->getCommandData ("cmdSetSpecialResolution");
   if (pCmd)
   {
      if(iXRes < 720)   // low resolution
      {
          pDevice_d->sendPrintfToDevice (pCmd,
                                         14400,
                                         14400 / pRes->getYRes (),
                                         14400 / iXRes );  //sets correct value for X positioning
      }
      else
      {
          int iNozzleValue = 2;
          int iXPos = 0;
          bool bRet = false;

          if(pData)
          {
              bRet =  pData->getIntData ("Nozzle_Spacing", &iNozzleValue);
              bRet =  pData->getIntData ("Positioning_x", &iXPos);
          }

          pDevice_d->sendPrintfToDevice (pCmd,
                                         14400,
                                         (14400 / pRes->getYRes ())* iNozzleValue,
                                         14400 / iXPos );  //sets correct value for X positioning
      }
   }
   else
   {
      DebugOutput::getErrorStream () << dec << "Error: There is no cmdSetDotSize defined for this device!" << endl;
   }
}

bool Epson_High_Res_ESCP2_Instance::
beginJob ()
{
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Epson_High_Res_ESCP2_Instance::beginJob ()" << endl;

   pCmd = pCommands->getCommandData ("cmdUSBInit");

   if (  hasDeviceOption (USB_PORT_SUPPORT)
      && pCmd
      )
   {
      if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "cmdUSBInit = " << *pCmd << endl;

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   pCmd = pCommands->getCommandData ("cmdInit");
   if (pCmd)
   {
      if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "cmdInit = " << *pCmd << endl;

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   pCmd = pCommands->getCommandData ("cmdRemote");
   if (pCmd)
   {
      if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "cmdRemote = " << *pCmd << endl;

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   return true;
}

bool Epson_High_Res_ESCP2_Instance::
beginJob (bool fJobPropertiesChanged)
{
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Epson_High_Res_ESCP2_Instance::beginJob (with props)" << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The job properties are as follows:" << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The orientation is = " << getCurrentOrientation() << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "iDitherID = " << getCurrentDitherID () << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The form is = " << getCurrentForm () << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The tray is = " << getCurrentTray () << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The media is = " << getCurrentMedia () << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The resolution is = " << getCurrentResolution () << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The print mode is = " << getCurrentPrintMode () << endl;

   // @TBD - reinitialize with new job properties

   // Call common code
   return beginJob ();
}

bool Epson_High_Res_ESCP2_Instance::
newFrame ()
{
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Epson_High_Res_ESCP2_Instance::newFrame ()" << endl;

   pCmd = pCommands->getCommandData ("cmdPageEject");
   if (pCmd)
   {
      if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "cmdPageEject = " << *pCmd << endl;

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   ditherNewFrame ();

   ptlPrintHead_d.x = 0;
   ptlPrintHead_d.y = 0;

   return true;
}

bool Epson_High_Res_ESCP2_Instance::
newFrame (bool fJobPropertiesChanged)
{
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Epson_High_Res_ESCP2_Instance::newFrame (with props)" << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The job properties are as follows:" << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The orientation is = " << getCurrentOrientation() << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "iDitherID = " << getCurrentDitherID () << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The form is = " << getCurrentForm () << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The tray is = " << getCurrentTray () << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The media is = " << getCurrentMedia () << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The resolution is = " << getCurrentResolution () << endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The print mode is = " << getCurrentPrintMode () << endl;

   // @TBD - reinitialize with new job properties

   // Call common code
   return newFrame ();
}

bool Epson_High_Res_ESCP2_Instance::
endJob ()
{
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Epson_High_Res_ESCP2_Instance::endJob ()" << endl;

   pCmd = pCommands->getCommandData ("cmdPageEject");
   if (pCmd)
   {
      if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "cmdPageEject = " << *pCmd << endl;

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   pCmd = pCommands->getCommandData ("cmdTerm");
   if (pCmd)
   {
      if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "cmdTerm = " << *pCmd << endl;

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   return true;
}

bool Epson_High_Res_ESCP2_Instance::
abortJob ()
{
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Epson_High_Res_ESCP2_Instance::endJob ()" << endl;

   int cx = getCurrentForm ()->getHardCopyCap ()->getXPels ();

   // The largest size is 24 lines
   int iSize = 24 * (cx + 7) / 8;

   PBYTE abData = new BYTE [iSize];

   memset (abData, 0, sizeof (abData));

   BinaryData data (abData, sizeof (abData));

   pDevice_d->sendBinaryDataToDevice (&data);

   delete []abData;

   return true;
}

string Epson_High_Res_ESCP2_Instance::
toString (ostringstream& oss)
{
   oss << "{ "
       << DeviceInstance::toString (oss) // Add parent's output as well
       << " }";

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const Epson_High_Res_ESCP2_Instance& const_self)
{
   Epson_High_Res_ESCP2_Instance& self = const_cast<Epson_High_Res_ESCP2_Instance&>(const_self);
   ostringstream                  oss;

   os << self.toString (oss);

   return os;
}
