/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2002
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "DriverInfo.hpp"
#include "Driver.hpp"

#include "Device.hpp"
#include "Omni.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <vector>

#include <glib.h>
#include <gmodule.h>

DriverInfo::
DriverInfo ()
{
   pDevice_d    = 0;
#ifndef PDC
   hmodDevice_d = 0;
#endif
}

DriverInfo::
~DriverInfo ()
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputDriverInfo ()) DebugOutput::getErrorStream () << __FUNCTION__ << "::~DriverInfo: enter" << endl;
#endif

   closeDevice ();

#ifndef RETAIL
   if (DebugOutput::shouldOutputDriverInfo ()) DebugOutput::getErrorStream () << __FUNCTION__ << "::~DriverInfo: exit" << endl;
#endif
}

void DriverInfo::
generateDriverInfo ()
{
   vector<string> listExternal (25);
   vector<string> listInternal (25);

   Enumeration *pEnumKeys = pDevice_d->listJobPropertyKeys ();

   while (pEnumKeys->hasMoreElements ())
   {
      char *pszKey = (char *)pEnumKeys->nextElement ();

      listExternal.clear ();
      listInternal.clear ();

      if (listValues (pszKey, listExternal, listInternal))
      {
         string *pXLate   = pDevice_d->translateKeyValue (pszKey,
                                                          (char *)listInternal[0].c_str ());
         char   *pszXLate = pszKey;
         string  temp;

         if (pXLate)
         {
            string::size_type pos = pXLate->find ("=");

            if (string::npos != pos)
            {
               temp = pXLate->substr (0, pos);

               pszXLate = (char *)temp.c_str ();
            }
         }

         dps_d.push_back (DriverProperty (pszKey,
                                          pszXLate,
                                          "ComboBox",
                                          listExternal,
                                          listInternal));

         delete pXLate;
      }
   }

   delete pEnumKeys;

   pEnumKeys = pDevice_d->listDeviceJobPropertyKeys ();

   while (pEnumKeys->hasMoreElements ())
   {
      char *pszKey = (char *)pEnumKeys->nextElement ();

      listExternal.clear ();
      listInternal.clear ();

      if (listValues (pszKey, listExternal, listInternal))
      {
         string *pXLate   = pDevice_d->translateKeyValue (pszKey,
                                                          (char *)listInternal[0].c_str ());
         char   *pszXLate = pszKey;
         string  temp;

         if (pXLate)
         {
            string::size_type pos = pXLate->find ("=");

            if (string::npos != pos)
            {
               temp = pXLate->substr (0, pos);

               pszXLate = (char *)temp.c_str ();
            }
         }

         dps_d.push_back (DriverProperty (pszKey,
                                          pszXLate,
                                          "ComboBox",
                                          listExternal,
                                          listInternal));

         delete pXLate;
      }
   }

   delete pEnumKeys;

   driver_d = Driver ("First Driver", dps_d);
}

vector<string> DriverInfo::
getDeviceList ()
{
   vector<string>  deviceList;
   Enumeration    *pEnum       = Omni::listDevices ();

#ifndef RETAIL
   if (DebugOutput::shouldOutputDriverInfo ()) DebugOutput::getErrorStream () << "\n Enumerating the devices. Please wait ... (It may take few seconds)" << endl;
#endif

   while (pEnum->hasMoreElements ())
   {
      char *pszLibName = (char *)pEnum->nextElement ();

      deviceList.push_back (pszLibName);
   }

   delete pEnum;

#ifndef RETAIL
   if (DebugOutput::shouldOutputDriverInfo ()) DebugOutput::getErrorStream () << endl << "************************************************" << endl;
#endif

   for (int iz = 0; iz < (int)deviceList.size (); iz++)
   {
      int point = deviceList[iz].find_last_of ('/');

      deviceList[iz] = deviceList[iz].substr (point + 4);
      deviceList[iz] = deviceList[iz].erase (deviceList[iz].size() - 3);
   }

   return deviceList;
}

int DriverInfo::
openDevice (char *pszFullDeviceName,
            char *pszJobProperties)
{
#ifdef PDC

   pDevice_d = new OmniPDCProxy (0,
                                 pszFullDeviceName,
                                 pszJobProperties,
                                 true);

#else

   PFNNEWDEVICEWARGS  pfnNewDeviceWArgs = 0;

   Omni::openDeviceLibrary (pszFullDeviceName, &hmodDevice_d);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDriverInfo ()) DebugOutput::getErrorStream () << __FUNCTION__ << "::openDevice: hmodDevice_d = " << hmodDevice_d << endl;
#endif

   if (!hmodDevice_d)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputDriverInfo ()) DebugOutput::getErrorStream () << "g_module_error returns " << g_module_error () << endl;
#endif

      return 2;
   }

   g_module_symbol (hmodDevice_d, "newDeviceW_JopProp_Advanced", (gpointer *)&pfnNewDeviceWArgs);

   if (!pfnNewDeviceWArgs)
      return 3;

   pDevice_d = pfnNewDeviceWArgs (true, pszJobProperties);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDriverInfo ()) DebugOutput::getErrorStream () << __FUNCTION__ << "::openDevice: pDevice_d = " << pDevice_d << endl;
#endif

#endif

   if (  !pDevice_d
      || pDevice_d->hasError ()
      )
   {
      return 0;
   }
   else
   {
      return 1;
   }
}

int DriverInfo::
closeDevice ()
{
   int rc = 0;

#ifndef RETAIL
   if (DebugOutput::shouldOutputDriverInfo ()) DebugOutput::getErrorStream () << __FUNCTION__ << "::~DriverInfo: pDevice_d = " << pDevice_d << endl;
#ifndef PDC
   if (DebugOutput::shouldOutputDriverInfo ()) DebugOutput::getErrorStream () << __FUNCTION__ << "::~DriverInfo: hmodDevice_d = " << hmodDevice_d << endl;
#endif
#endif

   delete pDevice_d;
   pDevice_d = 0;

#ifndef PDC
   if (hmodDevice_d)
   {
      rc = g_module_close (hmodDevice_d);
      hmodDevice_d = 0;
   }
#endif

   return rc;
}

Driver DriverInfo::
getDriver ()
{
   return driver_d;
}

void DriverInfo::
translateAndPush (vector<string>&  list,
                  char            *pszKey,
                  char            *pszValue)
{
   string *pXLate   = pDevice_d->translateKeyValue (pszKey, pszValue);
   bool    fSuccess = false;

   if (pXLate)
   {
      string::size_type pos = pXLate->find ("=");

      if (string::npos != pos)
      {
         list.push_back (pXLate->substr (pos + 1));
         fSuccess = true;
      }
   }

   if (!fSuccess)
   {
      list.push_back (pszValue);
   }

   delete pXLate;
}

bool DriverInfo::
listValues (char            *pszKey,
            vector<string>&  listExternal,
            vector<string>&  listInternal)
{
   Enumeration    *pEnumValues = 0;
   string         *pValue      = pDevice_d->getJobProperty (pszKey);

   if (pValue)
   {
      // Store the default
      translateAndPush (listExternal, pszKey, (char *)pValue->c_str ());

      pEnumValues = pDevice_d->listKeyValues (pszKey);

      while (pEnumValues->hasMoreElements ())
      {
         char *pszValue = (char *)pEnumValues->nextElement ();

         translateAndPush (listExternal, pszKey, pszValue);

         listInternal.push_back (pszValue);
      }

      delete pEnumValues;

      delete pValue;

      return true;
   }

   return false;
}
