/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2002
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <iostream>
#include <vector>
#include <map>

#include "DriverProperty.hpp"

using namespace std;

DriverProperty::
DriverProperty (string         propertyName,
                string         nameForDisplay,
                string         widgetName,
                vector<string> externalValues,
                vector<string> internalValues,
                bool           fEditable = true)
{
   propertyName_d   = propertyName;
   nameForDisplay_d = nameForDisplay;
   widgetName_d     = widgetName;
   fEditable_d      = fEditable;
   externalValues_d = externalValues;
   externalValue_d  = *externalValues_d.begin ();
   internalValues_d = internalValues;

   externalValues_d.erase (externalValues_d.begin ());
}

DriverProperty::
DriverProperty (const DriverProperty& dp)
{
   propertyName_d   = dp.propertyName_d;
   nameForDisplay_d = dp.nameForDisplay_d;
   widgetName_d     = dp.widgetName_d;
   externalValue_d  = dp.externalValue_d;
   fEditable_d      = dp.fEditable_d;
   externalValues_d = dp.externalValues_d;
   internalValues_d = dp.internalValues_d;
}

// @TBD remove
DriverProperty::
DriverProperty ()
{
   propertyName_d    = "";
   nameForDisplay_d  = "";
   widgetName_d      = "";
   externalValue_d   = "";
   fEditable_d       = false;
}

DriverProperty::
~DriverProperty ()
{
   // @TBD
}

DriverProperty DriverProperty::
operator = (DriverProperty dp)
{
   propertyName_d   = dp.propertyName_d;
   nameForDisplay_d = dp.nameForDisplay_d;
   widgetName_d     = dp.widgetName_d;
   externalValue_d  = dp.externalValue_d;
   fEditable_d      = dp.fEditable_d;
   externalValues_d = dp.externalValues_d;
   internalValues_d = dp.internalValues_d;

   return *this;
}

bool DriverProperty::
isEditable ()
{
   return fEditable_d;
}

string DriverProperty::
getName ()
{
   return propertyName_d;
}

string DriverProperty::
getPropertyNameForDisplay ()
{
   return nameForDisplay_d;
}

string DriverProperty::
getWidgetName ()
{
   return widgetName_d;
}

string DriverProperty::
getDefaultExternalValue ()
{
   return externalValue_d;
}

string DriverProperty::
getDefaultInternalValue ()
{
   vector<string>::iterator itrExt = externalValues_d.begin ();
   vector<string>::iterator itrInt = internalValues_d.begin ();

   for ( ; itrExt != externalValues_d.end (); itrExt++, itrInt++)
   {
//////cout << "External = " << (itrExt ? *itrExt : "NULL") << endl;
//////cout << "Internal = " << (itrInt ? *itrInt : "NULL") << endl;
      if (*itrExt == externalValue_d)
      {
/////////cout << "Returning: " << (itrInt ? *itrInt : "NULL") << endl;
         return *itrInt;
      }
   }

   return externalValue_d;
}

vector<string> DriverProperty::
getAvailableValues ()
{
   return externalValues_d;
}

bool DriverProperty::
setDefaultValue (string externalValue)
{
   externalValue_d = externalValue;

   return true;
}

string DriverProperty::
toString (ostringstream& oss)
{
   oss << "Property Name --> " << propertyName_d;
   oss << "      Display Name --> " << nameForDisplay_d << endl;
   oss << "Widget Name --> " << widgetName_d << "       Editable -->" << fEditable_d << endl;
   oss << "Default Value --> " << externalValue_d << endl;

   vector<string>::iterator itr;

   oss << "Available Property Values are: " << endl;
   for (itr = externalValues_d.begin (); itr != externalValues_d.end (); itr++)
      oss << "\t\t " << *itr << endl;

   return oss.str ();
}
