/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "PDCBlitterClient.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <fstream>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <sys/shm.h>
#include <errno.h>

class BCDeviceOrientation : public DeviceOrientation
{
public:
   BCDeviceOrientation (Device *pDevice,
                        int     id)
      : DeviceOrientation (pDevice, id)
   {
   }

   char *
   getDeviceID ()
   {
      return 0;
   }

   bool
   isOrientationSupported (int id)
   {
      return false;
   }

   Enumeration *
   getEnumeration ()
   {
      return 0;
   }

   DeviceOrientation *
   createV (int id)
   {
      return 0;
   }
};

class BCDeviceForm : public DeviceForm
{
public:
   BCDeviceForm (Device      *pDevice,
                 int          id,
                 int          iCapabilities,
                 BinaryData  *data,
                 HardCopyCap *hcInfo)
      : DeviceForm (pDevice, id, iCapabilities, data, hcInfo)
   {
   }

   char *
   getDeviceID ()
   {
      return 0;
   }

   bool
   isFormSupported (int id)
   {
      return false;
   }

   Enumeration *
   getEnumeration ()
   {
      return 0;
   }

   DeviceForm *
   createV (int id)
   {
      return 0;
   }
};

class BCDeviceTray : public DeviceTray
{
public:
   BCDeviceTray (Device     *pDevice,
                 int         id,
                 int         iType,
                 BinaryData *pbdData)
      : DeviceTray (pDevice, id, iType, pbdData)
   {
   }

   char *
   getDeviceID ()
   {
      return 0;
   }

   bool
   isTraySupported (int id)
   {
      return false;
   }

   Enumeration *
   getEnumeration ()
   {
      return 0;
   }

   DeviceTray *
   createV (int id)
   {
      return 0;
   }
};

class BCDeviceMedia : public DeviceMedia
{
public:
   BCDeviceMedia (Device     *pDevice,
                  int         id,
                  BinaryData *pbdData,
                  int         iColorAdjustRequired,
                  int         iAbsorption)
      : DeviceMedia (pDevice, id, pbdData, iColorAdjustRequired, iAbsorption)
   {
   }

   char *
   getDeviceID ()
   {
      return 0;
   }

   bool
   isMediaSupported (int id)
   {
      return false;
   }

   Enumeration *
   getEnumeration ()
   {
      return 0;
   }

   DeviceMedia *
   createV (int id)
   {
      return 0;
   }
};

class BCDeviceResolution : public DeviceResolution
{
public:
   BCDeviceResolution (Device     *pDevice,
                       int         id,
                       int         iXRes,
                       int         iYRes,
                       int         iXInternalRes,
                       int         iYInternalRes,
                       BinaryData *pbdData,
                       int         iCapabilities,
                       int         iDestinationBitsPerPel,
                       int         iScanlineMultiple)
      : DeviceResolution (pDevice,
                          id,
                          iXRes,
                          iYRes,
                          iXInternalRes,
                          iYInternalRes,
                          pbdData,
                          iCapabilities,
                          iDestinationBitsPerPel,
                          iScanlineMultiple)
   {
   }

   char *
   getDeviceID ()
   {
      return 0;
   }

   bool
   isResolutionSupported (int id)
   {
      return false;
   }

   Enumeration *
   getEnumeration ()
   {
      return 0;
   }

   DeviceResolution *
   createV (int id)
   {
      return 0;
   }
};

class BCDevicePrintMode : public DevicePrintMode
{
public:
   BCDevicePrintMode (Device *pDevice,
                      int     id,
                      int     iPhysicalCount,
                      int     iLogicalCount,
                      int     iPlanes)
      : DevicePrintMode (pDevice, id, iPhysicalCount, iLogicalCount, iPlanes)
   {
   }

   char *
   getDeviceID ()
   {
      return 0;
   }

   bool
   isPrintModeSupported (int id)
   {
      return false;
   }

   Enumeration *
   getEnumeration ()
   {
      return 0;
   }

   DevicePrintMode *
   createV (int id)
   {
      return 0;
   }
};

class BCDeviceGamma : public DeviceGamma
{
public:
   BCDeviceGamma (int iCGamma,
                  int iMGamma,
                  int iYGamma,
                  int iKGamma,
                  int iCBias,
                  int iMBias,
                  int iYBias,
                  int iKBias)
   : DeviceGamma (iCGamma,
                  iMGamma,
                  iYGamma,
                  iKGamma,
                  iCBias,
                  iMBias,
                  iYBias,
                  iKBias)
   {
   }
};

DeviceObjectHolder::
DeviceObjectHolder ()
{
   pDO_d  = 0;
   pDF_d  = 0;
   pDT_d  = 0;
   pDM_d  = 0;
   pDR_d  = 0;
   pDPM_d = 0;
   pDG_d  = 0;
}

DeviceObjectHolder::
~DeviceObjectHolder ()
{
   delete pDO_d;
   delete pDF_d;
   delete pDT_d;
   delete pDM_d;
   delete pDR_d;
   delete pDPM_d;
   delete pDG_d;
}

DeviceOrientation * DeviceObjectHolder::
getCurrentOrientation ()
{
   return pDO_d;
}

DeviceForm * DeviceObjectHolder::
getCurrentForm ()
{
   return pDF_d;
}

DeviceTray * DeviceObjectHolder::
getCurrentTray ()
{
   return pDT_d;
}

DeviceMedia * DeviceObjectHolder::
getCurrentMedia ()
{
   return pDM_d;
}

DeviceResolution * DeviceObjectHolder::
getCurrentResolution ()
{
   return pDR_d;
}

DevicePrintMode * DeviceObjectHolder::
getCurrentPrintMode ()
{
   return pDPM_d;
}

DeviceGamma * DeviceObjectHolder::
getCurrentGamma ()
{
   return pDG_d;
}

void DeviceObjectHolder::
setCurrentOrientation (DeviceOrientation *pDO)
{
   delete pDO_d;
   pDO_d = pDO;
}

void DeviceObjectHolder::
setCurrentForm (DeviceForm *pDF)
{
   delete pDF_d;
   pDF_d = pDF;
}

void DeviceObjectHolder::
setCurrentTray (DeviceTray *pDT)
{
   delete pDT_d;
   pDT_d = pDT;
}

void DeviceObjectHolder::
setCurrentMedia (DeviceMedia *pDM)
{
   delete pDM_d;
   pDM_d = pDM;
}

void DeviceObjectHolder::
setCurrentResolution (DeviceResolution *pDR)
{
   delete pDR_d;
   pDR_d = pDR;
}

void DeviceObjectHolder::
setCurrentPrintMode (DevicePrintMode *pDPM)
{
   delete pDPM_d;
   pDPM_d = pDPM;
}

void DeviceObjectHolder::
setCurrentGamma (DeviceGamma *pDG)
{
   delete pDG_d;
   pDG_d = pDG;
}

bool
sendResponse (int fd, PDC_CMD eCommand, char *pszLine)
{
   PDC_PACKET cmdHeader;
   size_t     cbLine    = 0;
   size_t     rc;

   cmdHeader.eCommand          = eCommand;
   cmdHeader.cbLength          = sizeof (cmdHeader);
   cmdHeader.eFormat           = PDCFMT_STRING;
   cmdHeader.achCommandLine[0] = '\0';

   if (  pszLine
      && *pszLine
      )
   {
      cbLine                      = strlen (pszLine);
      cmdHeader.cbLength         += cbLine;
      cmdHeader.achCommandLine[0] = pszLine[0];
   }

   rc = write (fd, &cmdHeader, sizeof (cmdHeader));

   if (rc != sizeof (cmdHeader))
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputPDCBlitterClient ())
         DebugOutput::getErrorStream () << "EpsonBlitter: wrote "
              << rc
              << " bytes, expecting "
              << sizeof (cmdHeader)
              << " @ line "
              << __LINE__
              << endl;
#endif

      return false;
   }

   if (  pszLine
      && *pszLine
      )
   {
      rc = write (fd, pszLine + 1, cbLine);

      if (rc != cbLine)
      {
#ifndef RETAIL
         if (DebugOutput::shouldOutputPDCBlitterClient ())
            DebugOutput::getErrorStream () << "EpsonBlitter: wrote "
                 << rc
                 << " bytes, expecting "
                 << cbLine
                 << " @ line "
                 << __LINE__
                 << endl;
#endif

         return false;
      }
   }

   return true;
}

bool
sendResponse (int fd, PDC_CMD eCommand, string *pLine)
{
   char *pszLine = 0;

   if (pLine)
   {
      pszLine = (char *)pLine->c_str ();
   }

   return sendResponse (fd, eCommand, pszLine);
}

bool
sendResponse (int fd, PDC_CMD eCommand, int iValue)
{
   PDC_PACKET cmdHeader;
   byte       abData[sizeof (int)];
   size_t     rc;

   cmdHeader.eCommand = eCommand;
   cmdHeader.cbLength = sizeof (cmdHeader) - 1 + sizeof (int);
   cmdHeader.eFormat  = PDCFMT_INTEGER;

   for (size_t i = 0; i < sizeof (int); i++)
   {
      abData[i] = iValue & 0xFF;

      iValue >>= 8;
   }

   rc = write (fd, &cmdHeader, sizeof (cmdHeader) - 1);

   if (rc != sizeof (cmdHeader) - 1)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputPDCBlitterClient ())
         DebugOutput::getErrorStream () << "EpsonBlitter: wrote "
              << rc
              << " bytes, expecting "
              << sizeof (cmdHeader) - 1
              << " @ line "
              << __LINE__
              << endl;
#endif

      return false;
   }

   rc = write (fd, abData, sizeof (int));

   if (rc != sizeof (int))
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputPDCBlitterClient ())
         DebugOutput::getErrorStream () << "EpsonBlitter: wrote "
              << rc
              << " bytes, expecting "
              << sizeof (int)
              << " @ line "
              << __LINE__
              << endl;
#endif

      return false;
   }

   return true;
}

bool
sendResponse (int fd, PDC_CMD eCommand, long int lValue)
{
   PDC_PACKET cmdHeader;
   byte       abData[sizeof (long int)];
   size_t     rc;

   cmdHeader.eCommand = eCommand;
   cmdHeader.cbLength = sizeof (cmdHeader) - 1 + sizeof (long int);
   cmdHeader.eFormat  = PDCFMT_LONG;

   for (size_t i = 0; i < sizeof (long int); i++)
   {
      abData[i] = lValue & 0xFF;

      lValue >>= 8;
   }

   rc = write (fd, &cmdHeader, sizeof (cmdHeader) - 1);

   if (rc != sizeof (cmdHeader) - 1)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputPDCBlitterClient ())
         DebugOutput::getErrorStream () << "EpsonBlitter: wrote "
              << rc
              << " bytes, expecting "
              << sizeof (cmdHeader) - 1
              << " @ line "
              << __LINE__
              << endl;
#endif

      return false;
   }

   rc = write (fd, abData, sizeof (long int));

   if (rc != sizeof (long int))
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputPDCBlitterClient ())
         DebugOutput::getErrorStream () << "EpsonBlitter: wrote "
              << rc
              << " bytes, expecting "
              << sizeof (long int)
              << " @ line "
              << __LINE__
              << endl;
#endif

      return false;
   }

   return true;
}

int
main (int argc, char *argv[])
{
   char               *pszDeviceName         = 0;
   char               *pszS2C;
   char               *pszC2S;
   int                 fdS2C                 = -1;            /* server to client */
   int                 fdC2S                 = -1;            /* client to server */
   bool                fRun                  = true;
   PDC_CMD             eCommand;
   PrinterCommand     *pCmd                  = 0;
   char               *pszJobProperties      = 0;
   size_t              cbJobProperties       = 0;
   bool                fNewJobPropertiesSent = false;
   byte               *pbBuffer1             = 0;
   byte               *pbBuffer2             = 0;
   HANDLE              hInstance             = 0;
   GModule            *hmodDevice            = 0;
   int                 iCurrentLanguage      = StringResource::LANGUAGE_DEFAULT;
   StringResource     *pResource             = StringResource::create (iCurrentLanguage, 0);
   PDC_B_HOOKPOINTS    HookPoints;
   DeviceObjectHolder  DOH;
   FILE               *fpError               = 0;

#ifndef RETAIL
   {
      ostringstream oss;

      cout << "Executing ";

      oss << "which ";
      oss << argv[0];

      cout.flush ();

      string str (oss.str ());

      system (str.c_str ());
   }
#endif

   memset (&HookPoints, 0, sizeof (HookPoints));
   initializePDCBlitter (&HookPoints);

   if (getenv ("EPSONSERVERPAUSE"))
   {
      // Handy to attach via gdb debugger.  do:
      // (gdb) file EpsonBlitter
      // (gdb) shell ps -efl | grep EpsonBlitter          to get the <pid>.
      // (gdb) attach <pid>
      // (gdb) set fPause = 0
      // (gdb) c
      bool fPause = true;

      while (fPause)
      {
      }
   }

   /* Get local copy. */
   pszS2C = getenv ("OMNI_BLITTER_S2C");
   pszC2S = getenv ("OMNI_BLITTER_C2S");

   if (  !pszS2C
      || !pszC2S
      )
   {
      DebugOutput::getErrorStream () << argv[0] << " must have OMNI_BLITTER_S2C and OMNI_BLITTER_C2S set in the environment!" << endl;

      return 1;
   }

#ifndef RETAIL
   if (DebugOutput::shouldOutputPDCBlitterClient ())
   {
      DebugOutput::getErrorStream () << "EpsonBlitter: OMNI_BLITTER_S2C=" << pszS2C << endl;
      DebugOutput::getErrorStream () << "EpsonBlitter: OMNI_BLITTER_C2S=" << pszC2S << endl;
      DebugOutput::setOutputPrinterCommand (true);
   }
#endif

   /* Open fifos. */
   if ((fdS2C = open (pszS2C, O_WRONLY)) < 0)
   {
      goto BUGOUT;
   }

   if ((fdC2S = open (pszC2S, O_RDONLY)) < 0)
   {
      goto BUGOUT;
   }

   pCmd = new PrinterCommand ("EpsonBlitter");

#ifndef RETAIL
   if (DebugOutput::shouldOutputPDCBlitterClient ()) DebugOutput::getErrorStream () << "EpsonBlitter: ready." << endl;
#endif

   while (fRun)
   {
      if (!pCmd->readCommand (fdC2S))
         goto BUGOUT;

      eCommand = PDCCMD_NACK;

      switch (pCmd->getCommandType ())
      {
      case PDCCMD_ACK:
      case PDCCMD_NACK:
      case PDCCMD_UNSUPPORTED:
      {
         if (!sendResponse (fdS2C, PDCCMD_NACK, "Unexpected ack/nack/unsupported command"))
            goto BUGOUT;
         break;
      }

      case PDCCMD_INITIALIZE_SESSION:
      {
         // Send the response
         if (!sendResponse (fdS2C, PDCCMD_ACK, PDC_VERSION))
            goto BUGOUT;
         break;
      }

      case PDCCMD_CLOSE_SESSION:
      {
         fRun = false;
         break;
      }

      case PDCMD_SET_TRANSLATABLE_LANGUAGE:
      {
         if (  pCmd->getCommandString ()
            && *pCmd->getCommandString ()
            )
         {
            // @TBD need architecture here
            int iLangIn = StringResource::nameToID (pCmd->getCommandString ());

            if (StringResource::LANGUAGE_UNKNOWN != iLangIn)
            {
               Enumeration *pEnum = StringResource::getLanguages ();

               while (pEnum->hasMoreElements ())
               {
                  int iLangSupported = (int)pEnum->nextElement ();

                  if (iLangIn == iLangSupported)
                  {
                     if (iLangIn != iCurrentLanguage)
                     {
                        delete pResource;

                        pResource        = StringResource::create (iLangIn, 0);
                        iCurrentLanguage = iLangIn;
                     }
                     eCommand = PDCCMD_ACK;
                  }
               }

               delete pEnum;
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_SET_DEVICE_NAME:
      {
         if (  HookPoints.pfnInitializeInstance
            && pCmd->getCommandString ()
            && *pCmd->getCommandString ()
            )
         {
            int cbDeviceName = strlen (pCmd->getCommandString ()) + 1;

            pszDeviceName = (char *)malloc (cbDeviceName);

            if (pszDeviceName)
            {
               strcpy (pszDeviceName, pCmd->getCommandString ());

               hmodDevice = HookPoints.pfnInitializeInstance (pszDeviceName, &hInstance);

               if (  hmodDevice
                  && hInstance
                  )
               {
                  eCommand = PDCCMD_ACK;
               }
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_IS_VALID_DEVICE_NAME:
      {
         if (  HookPoints.pfnInitializeInstance
            && pCmd->getCommandString ()
            && *pCmd->getCommandString ()
            )
         {
            GModule *hmodTemp      = 0;
            HANDLE   hInstanceTemp = 0;

            hmodTemp = HookPoints.pfnInitializeInstance (pCmd->getCommandString (),
                                                         &hInstanceTemp);

            if (  hmodTemp
               && hInstanceTemp
               )
            {
               eCommand = PDCCMD_ACK;
            }

            if (  HookPoints.pfnFreeInstance
               && hmodTemp
               )
            {
               HookPoints.pfnFreeInstance (hmodTemp, hInstanceTemp);
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_SET_JOB_PROPERTIES:
      {
         char *pszCommandString = pCmd->getCommandString ();

         if (  HookPoints.pfnSetJobProperties
            && pszCommandString
            && *pszCommandString
            )
         {
            char *pszDebugLook   = pszCommandString;
            char *pszDebugOutput = 0;

            do
            {
               pszDebugOutput = strstr (pszDebugLook, "debugoutput=");

               if (pszDebugOutput)
               {
                  char *pszSpace = strchr (pszDebugOutput, ' ');
                  char *pszValue = 0;

                  pszValue = pszDebugOutput + 12;

                  if (pszSpace)
                  {
                     pszDebugLook = pszSpace + 1;
                     *pszSpace = '\0';
                  }
                  else
                  {
                     pszDebugLook = 0;
                  }

                  DebugOutput::setDebugOutput (pszValue);

                  if (pszSpace)
                     *pszSpace = ' ';
               }
               else
               {
                  pszDebugLook = 0;
               }

            } while (pszDebugLook);

            if (cbJobProperties < strlen (pszCommandString) + 1)
            {
               cbJobProperties  = strlen (pszCommandString) + 1;
               pszJobProperties = (char *)realloc (pszJobProperties, cbJobProperties);
            }

            if (pszJobProperties)
            {
               strcpy (pszJobProperties, pszCommandString);
               fNewJobPropertiesSent = true;

               if (HookPoints.pfnSetJobProperties (hInstance,
                                                   pszJobProperties,
                                                   &DOH))
               {
                  eCommand = PDCCMD_ACK;
               }
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_LIST_DEVICE_JOB_PROPERTY_KEYS:
      {
         string *pstrDJPKeys = 0;

         if (HookPoints.pfnListDeviceJobPropertyKeys)
         {
            pstrDJPKeys = HookPoints.pfnListDeviceJobPropertyKeys (hInstance);

            if (pstrDJPKeys)
            {
               eCommand = PDCCMD_ACK;
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, pstrDJPKeys))
         {
            delete pstrDJPKeys;
            goto BUGOUT;
         }
         delete pstrDJPKeys;
         break;
      }

      case PDCCMD_LIST_JOB_PROPERTY_KEY_VALUES:
      {
         string *pstrJPKValues    = 0;
         char   *pszCommandString = pCmd->getCommandString ();

         if (  HookPoints.pfnListJobPropertyKeyValues
            && hInstance
            && pszCommandString
            && *pszCommandString
            )
         {
            pstrJPKValues = HookPoints.pfnListJobPropertyKeyValues (hInstance,
                                                                    pszCommandString);

            if (pstrJPKValues)
            {
               eCommand = PDCCMD_ACK;
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, pstrJPKValues))
         {
            delete pstrJPKValues;
            goto BUGOUT;
         }
         delete pstrJPKValues;
         break;
      }

      case PDCCMD_GET_JOB_PROPERTY_TYPE:
      {
         char   *pszCommandString = pCmd->getCommandString ();
         string *pstrJPT          = 0;

         if (  HookPoints.pfnGetJobPropertyType
            && hInstance
            && pszCommandString
            && *pszCommandString
            )
         {
            pstrJPT = HookPoints.pfnGetJobPropertyType (hInstance,
                                                        pszCommandString);

            if (pstrJPT)
            {
               eCommand = PDCCMD_ACK;
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, pstrJPT))
         {
            delete pstrJPT;
            goto BUGOUT;
         }
         delete pstrJPT;
         break;
      }

      case PDCCMD_GET_JOB_PROPERTY:
      {
         char *pszCommandString = pCmd->getCommandString ();
         char *pszRet           = 0;

         if (  HookPoints.pfnGetJobProperty
            && hInstance
            && pszCommandString
            && *pszCommandString
            )
         {
            pszRet = HookPoints.pfnGetJobProperty (hInstance,
                                                   pszCommandString);

            if (pszRet)
            {
               eCommand = PDCCMD_ACK;
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, pszRet))
            goto BUGOUT;
         break;
      }

      case PDCMD_XLATE_JOB_PROPERTY_KEY_VALUE:
      {
         char   *pszCommandString = pCmd->getCommandString ();
         string *pstrRet          = 0;

         if (  HookPoints.pfnTranslateKeyValue
            && hInstance
            && pszCommandString
            && *pszCommandString
            )
         {
            pstrRet = HookPoints.pfnTranslateKeyValue (hInstance,
                                                       pResource,
                                                       pszCommandString);

            if (pstrRet)
            {
               eCommand = PDCCMD_ACK;
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, pstrRet))
         {
            delete pstrRet;
            goto BUGOUT;
         }
         delete pstrRet;
         break;
      }

      case PDCCMD_SET_OUTPUT_STREAM:
      {
         if (  HookPoints.pfnSetOutputStream
            && pCmd->getCommandString ()
            && *pCmd->getCommandString ()
            )
         {
            int fdHandle = STDOUT_FILENO;

            sscanf (pCmd->getCommandString (), "%d", &fdHandle);

            if (HookPoints.pfnSetOutputStream (hInstance,
                                               fdHandle))
            {
               eCommand = PDCCMD_ACK;
            }
         }

         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_SET_ERROR_STREAM:
      {
         if (  pCmd->getCommandString ()
            && *pCmd->getCommandString ()
            )
         {
            int fdHandle = STDERR_FILENO;

            sscanf (pCmd->getCommandString (), "%d", &fdHandle);

            fpError = fdopen (fdHandle, "wb");

            if (fpError)
            {
               DebugOutput::setErrorStream (fpError);
               eCommand = PDCCMD_ACK;
            }
         }

         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_BEGIN_JOB:
      {
         if (  HookPoints.pfnBeginJob
            && hInstance
            )
         {
            if (HookPoints.pfnBeginJob (hInstance))
            {
               eCommand = PDCCMD_ACK;
            }
         }

         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_START_PAGE:
      {
         if (  HookPoints.pfnStartPage
            && hInstance
            )
         {
            if (HookPoints.pfnStartPage (hInstance))
               eCommand = PDCCMD_ACK;
         }

         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_END_PAGE:
      {
         if (  HookPoints.pfnEndPage
            && hInstance
            )
         {
            if (HookPoints.pfnEndPage (hInstance, 0))
               eCommand = PDCCMD_ACK;
         }

         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_END_JOB:
      {
         if (  HookPoints.pfnEndJob
            && hInstance
            )
         {
            if (HookPoints.pfnEndJob (hInstance))
               eCommand = PDCCMD_ACK;
         }

         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_ABORT_JOB:
      {
         if (  HookPoints.pfnAbortJob
            && hInstance
            )
         {
            if (HookPoints.pfnAbortJob (hInstance))
               eCommand = PDCCMD_ACK;
         }

         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_ATTACH_BUFFER1:
      {
         if (pbBuffer1)
         {
            shmdt (pbBuffer1);
         }

         pbBuffer1 = (byte *)shmat (pCmd->getCommandInt (), 0, 0);

         if (0 < pbBuffer1)
         {
            eCommand = PDCCMD_ACK;
         }

         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_ATTACH_BUFFER2:
      {
         if (pbBuffer2)
         {
            shmdt (pbBuffer2);
         }

         pbBuffer2 = (byte *)shmat (pCmd->getCommandInt (), 0, 0);

         if (0 < pbBuffer2)
         {
            eCommand = PDCCMD_ACK;
         }

         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_DETACH_BUFFER1:
      {
         if (pbBuffer1)
         {
            int rc = shmdt (pbBuffer1);

            if (0 == rc)
            {
               eCommand = PDCCMD_ACK;
            }
         }

         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_DETACH_BUFFER2:
      {
         if (pbBuffer2)
         {
            int rc = shmdt (pbBuffer2);

            if (0 == rc)
            {
               eCommand = PDCCMD_ACK;
            }
         }

         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_RASTERIZE:
      {
         if (  HookPoints.pfnRasterize
            && hInstance
            )
         {
            if (  pCmd->getCommandString ()
               && *pCmd->getCommandString ()
               )
            {
               int          iType;
               BITBLT_TYPE  eType;
               RECTL        rectlPageLocation;

               sscanf (pCmd->getCommandString (),
                       "%d %d %d %d %d",
                       &iType,
                       &rectlPageLocation.xLeft,
                       &rectlPageLocation.yBottom,
                       &rectlPageLocation.xRight,
                       &rectlPageLocation.yTop);

               eType = (BITBLT_TYPE)iType;

               if (HookPoints.pfnRasterize (hInstance,
                                            pbBuffer2,
                                            (PBITMAPINFO2)pbBuffer1,
                                            &rectlPageLocation,
                                            eType))
               {
                  eCommand = PDCCMD_ACK;
               }
            }
         }

         if (!sendResponse (fdS2C, eCommand, (char *)0))
            goto BUGOUT;
         break;
      }

      case PDCCMD_PUSH_CURRENT_ORIENTATION:
      {
         char              *pszCommandString = pCmd->getCommandString ();
         DeviceOrientation *pDOTemp          = 0;

         if (  pszCommandString
            && *pszCommandString
            )
         {
            pDOTemp = new BCDeviceOrientation (0,
                                               DeviceOrientation::nameToID (pszCommandString));

            if (pDOTemp)
            {
               DOH.setCurrentOrientation (pDOTemp);

               eCommand = PDCCMD_ACK;
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, (char *)0))
         {
            goto BUGOUT;
         }
         break;
      }

      case PDCCMD_PUSH_CURRENT_DITHER_ID:
      {
         // Send the response
         if (!sendResponse (fdS2C, PDCCMD_ACK, (char *)0))
         {
            goto BUGOUT;
         }
         break;
      }

      case PDCCMD_PUSH_CURRENT_FORM:
      {
         char       *pszCommandString = pCmd->getCommandString ();
         DeviceForm *pDFTemp          = 0;

         if (  pszCommandString
            && *pszCommandString
            )
         {
            char *pszID         = 0;
            int   iCapabilities = 0;
            int   iLeftClip     = 0;
            int   iTopClip      = 0;
            int   iRightClip    = 0;
            int   iBottomClip   = 0;

            pszID = pszCommandString;

            pszCommandString = strchr (pszID, ' ');

            if (pszCommandString)
            {
               *pszCommandString++ = '\0';

               sscanf (pszCommandString,
                       "%d %d %d %d %d",
                       &iCapabilities,
                       &iLeftClip,
                       &iTopClip,
                       &iRightClip,
                       &iBottomClip);

               pDFTemp = new BCDeviceForm (0,
                                           DeviceForm::nameToID (pszID),
                                           iCapabilities,
                                           0,
                                           new HardCopyCap (iLeftClip,
                                                            iTopClip,
                                                            iRightClip,
                                                            iBottomClip));

               if (pDFTemp)
               {
                  if (DOH.getCurrentResolution ())
                     pDFTemp->associateWith (DOH.getCurrentResolution ());

                  DOH.setCurrentForm (pDFTemp);

                  eCommand = PDCCMD_ACK;
               }
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, (char *)0))
         {
            goto BUGOUT;
         }
         break;
      }

      case PDCCMD_PUSH_CURRENT_TRAY:
      {
         char       *pszCommandString = pCmd->getCommandString ();
         DeviceTray *pDTTemp          = 0;

         if (  pszCommandString
            && *pszCommandString
            )
         {
            char *pszID = 0;
            int   iType = 0;

            pszID = pszCommandString;

            pszCommandString = strchr (pszID, ' ');

            if (pszCommandString)
            {
               *pszCommandString++ = '\0';

               sscanf (pszCommandString,
                       "%d",
                       &iType);

               pDTTemp = new BCDeviceTray (0,
                                           DeviceTray::nameToID (pszID),
                                           iType,
                                           0);

               if (pDTTemp)
               {
                  DOH.setCurrentTray (pDTTemp);

                  eCommand = PDCCMD_ACK;
               }
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, (char *)0))
         {
            goto BUGOUT;
         }
         break;
      }

      case PDCCMD_PUSH_CURRENT_MEDIA:
      {
         char        *pszCommandString = pCmd->getCommandString ();
         DeviceMedia *pDMTemp          = 0;

         if (  pszCommandString
            && *pszCommandString
            )
         {
            char *pszID                = 0;
            int   iColorAdjustRequired = 0;
            int   iAbsorption          = 0;

            pszID = pszCommandString;

            pszCommandString = strchr (pszID, ' ');

            if (pszCommandString)
            {
               *pszCommandString++ = '\0';

               sscanf (pszCommandString,
                       "%d %d",
                       &iColorAdjustRequired,
                       &iAbsorption);

               pDMTemp = new BCDeviceMedia (0,
                                            DeviceMedia::nameToID (pszID),
                                            0,
                                            iColorAdjustRequired,
                                            iAbsorption);

               if (pDMTemp)
               {
                  DOH.setCurrentMedia (pDMTemp);

                  eCommand = PDCCMD_ACK;
               }
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, (char *)0))
         {
            goto BUGOUT;
         }
         break;
      }

      case PDCCMD_PUSH_CURRENT_RESOLUTION:
      {
         char             *pszCommandString = pCmd->getCommandString ();
         DeviceResolution *pDRTemp          = 0;

         if (  pszCommandString
            && *pszCommandString
            )
         {
            char *pszID                  = 0;
            int   iXRes                  = 0;
            int   iYRes                  = 0;
            int   iXInternalRes          = 0;
            int   iYInternalRes          = 0;
            int   iCapabilities          = 0;
            int   iDestinationBitsPerPel = 0;
            int   iScanlineMultiple      = 0;

            pszID = pszCommandString;

            pszCommandString = strchr (pszID, ' ');

            if (pszCommandString)
            {
               *pszCommandString++ = '\0';

               sscanf (pszCommandString,
                       "%d %d %d %d %d %d %d",
                       &iXRes,
                       &iYRes,
                       &iXInternalRes,
                       &iYInternalRes,
                       &iCapabilities,
                       &iDestinationBitsPerPel,
                       &iScanlineMultiple);

               pDRTemp = new BCDeviceResolution (0,
                                                 DeviceResolution::nameToID (pszID),
                                                 iXRes,
                                                 iYRes,
                                                 iXInternalRes,
                                                 iYInternalRes,
                                                 0,
                                                 iCapabilities,
                                                 iDestinationBitsPerPel,
                                                 iScanlineMultiple);

               if (pDRTemp)
               {
                  DOH.setCurrentResolution (pDRTemp);

                  eCommand = PDCCMD_ACK;
               }
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, (char *)0))
         {
            goto BUGOUT;
         }
         break;
      }

      case PDCCMD_PUSH_CURRENT_PRINT_MODE:
      {
         char            *pszCommandString = pCmd->getCommandString ();
         DevicePrintMode *pDPMTemp         = 0;

         if (  pszCommandString
            && *pszCommandString
            )
         {
            char *pszID          = 0;
            int   iPhysicalCount = 0;
            int   iLogicalCount  = 0;
            int   iPlanes        = 0;

            pszID = pszCommandString;

            pszCommandString = strchr (pszID, ' ');

            if (pszCommandString)
            {
               *pszCommandString++ = '\0';

               sscanf (pszCommandString,
                       "%d %d %d",
                       &iPhysicalCount,
                       &iLogicalCount,
                       &iPlanes);

               pDPMTemp = new BCDevicePrintMode (0,
                                                 DevicePrintMode::nameToID (pszID),
                                                 iPhysicalCount,
                                                 iLogicalCount,
                                                 iPlanes);

               if (pDPMTemp)
               {
                  DOH.setCurrentPrintMode (pDPMTemp);

                  eCommand = PDCCMD_ACK;
               }
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, (char *)0))
         {
            goto BUGOUT;
         }
         break;
      }

      case PDCCMD_PUSH_CURRENT_GAMMA:
      {
         char        *pszCommandString = pCmd->getCommandString ();
         DeviceGamma *pDGTemp          = 0;

         if (  pszCommandString
            && *pszCommandString
            )
         {
            char *pszID       = 0;
            int   iCGamma     = 0;
            int   iMGamma     = 0;
            int   iYGamma     = 0;
            int   iKGamma     = 0;
            int   iCBias      = 0;
            int   iMBias      = 0;
            int   iYBias      = 0;
            int   iKBias      = 0;

            pszID = pszCommandString;

            pszCommandString = strchr (pszID, ' ');

            if (pszCommandString)
            {
               *pszCommandString++ = '\0';

               sscanf (pszCommandString,
                       "%d %d %d %d %d %d %d %d",
                       &iCGamma,
                       &iMGamma,
                       &iYGamma,
                       &iKGamma,
                       &iCBias,
                       &iMBias,
                       &iYBias,
                       &iKBias);

               pDGTemp = new BCDeviceGamma (iCGamma,
                                            iMGamma,
                                            iYGamma,
                                            iKGamma,
                                            iCBias,
                                            iMBias,
                                            iYBias,
                                            iKBias);

               if (pDGTemp)
               {
                  DOH.setCurrentGamma (pDGTemp);

                  eCommand = PDCCMD_ACK;
               }
            }
         }

         // Send the response
         if (!sendResponse (fdS2C, eCommand, (char *)0))
         {
            goto BUGOUT;
         }
         break;
      }

      default:
      {
         if (!sendResponse (fdS2C, PDCCMD_UNSUPPORTED, "Unknown command"))
            goto BUGOUT;
         break;
      }
      }
   }

BUGOUT:

   // Clean up
   if (-1 != fdS2C)
      close (fdS2C);
   if (-1 != fdC2S)
      close (fdC2S);

   if (pszJobProperties)
   {
      free (pszJobProperties);
   }

   if (pbBuffer1)
   {
      shmdt (pbBuffer1);
   }
   if (pbBuffer2)
   {
      shmdt (pbBuffer2);
   }

   delete pCmd;

   if (pszDeviceName)
   {
      free (pszDeviceName);
   }

   if (  HookPoints.pfnFreeInstance
      && hmodDevice
      )
   {
      HookPoints.pfnFreeInstance (hmodDevice, hInstance);
   }

   if (fpError)
   {
      fclose (fpError);
   }

   return 0;
}
