/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _PDCBlitterClient
#define _PDCBlitterClient

#include <unistd.h>

#include <string>

#include <glib.h>
#include <gmodule.h>

#include "Device.hpp"

class DeviceObjectHolder
{
public:
                      DeviceObjectHolder    ();
                     ~DeviceObjectHolder    ();

   DeviceOrientation *getCurrentOrientation ();
   DeviceForm        *getCurrentForm        ();
   DeviceTray        *getCurrentTray        ();
   DeviceMedia       *getCurrentMedia       ();
   DeviceResolution  *getCurrentResolution  ();
   DevicePrintMode   *getCurrentPrintMode   ();
   DeviceGamma       *getCurrentGamma       ();

   void               setCurrentOrientation (DeviceOrientation *pDO);
   void               setCurrentForm        (DeviceForm        *pDF);
   void               setCurrentTray        (DeviceTray        *pDT);
   void               setCurrentMedia       (DeviceMedia       *pDM);
   void               setCurrentResolution  (DeviceResolution  *pDR);
   void               setCurrentPrintMode   (DevicePrintMode   *pDPM);
   void               setCurrentGamma       (DeviceGamma       *pDG);

private:
   DeviceOrientation *pDO_d;
   DeviceForm        *pDF_d;
   DeviceTray        *pDT_d;
   DeviceMedia       *pDM_d;
   DeviceResolution  *pDR_d;
   DevicePrintMode   *pDPM_d;
   DeviceGamma       *pDG_d;
};

typedef void *HANDLE, **PHANDLE;

typedef GModule *    (*PFNINITIALIZEINSTANCE)            (char               *pszDeviceName,
                                                          PHANDLE             phInstance);
typedef bool         (*PFNSETJOBPROPERTIES)              (HANDLE              hInstance,
                                                          char               *pszJobProperties,
                                                          DeviceObjectHolder *pDOH);
typedef string *     (*PFNLISTDEVICEJOBPROPERTYKEYS)     (HANDLE              hInstance);
typedef string *     (*PFNLISTJOBPROPERTYKEYVALUES)      (HANDLE              hInstance,
                                                          char               *pszKey);
typedef string *     (*PFNGETJOBPROPERTYTYPE)            (HANDLE              hInstance,
                                                          char               *pszKey);
typedef char *       (*PFNGETJOBPROPERTY)                (HANDLE              hInstance,
                                                         char                *pszKey);
typedef string *     (*PFNTRANSLATEKEYVALUE)             (HANDLE              hInstance,
                                                          StringResource     *pResource,
                                                          char               *pszKeyValue);
typedef bool         (*PFNSTARTPAGE)                     (HANDLE              hInstance);
typedef bool         (*PFNENDPAGE)                       (HANDLE              hInstance,
                                                          int                 iCanceled);
typedef bool         (*PFNBEGINJOB)                      (HANDLE              hInstance);
typedef bool         (*PFNENDJOB)                        (HANDLE              hInstance);
typedef bool         (*PFNABORTJOB)                      (HANDLE              hInstance);
typedef int          (*PFNRASTERIZE)                     (HANDLE              hInstance,
                                                          PBYTE               pbBits,
                                                          PBITMAPINFO2        pbmi2,
                                                          PRECTL              prectlPageLocation,
                                                          BITBLT_TYPE         eType);
typedef bool         (*PFNFREEINSTANCE)                  (GModule            *hmodDevice,
                                                          HANDLE              hInstance);
typedef bool         (*PFNSETOUTPUTSTREAM)               (HANDLE              hInstance,
                                                          int                 fdHandle);

typedef struct _PDCBlitterHookPoints {
   PFNINITIALIZEINSTANCE        pfnInitializeInstance;
   PFNSETJOBPROPERTIES          pfnSetJobProperties;
   PFNLISTDEVICEJOBPROPERTYKEYS pfnListDeviceJobPropertyKeys;
   PFNLISTJOBPROPERTYKEYVALUES  pfnListJobPropertyKeyValues;
   PFNGETJOBPROPERTYTYPE        pfnGetJobPropertyType;
   PFNGETJOBPROPERTY            pfnGetJobProperty;
   PFNTRANSLATEKEYVALUE         pfnTranslateKeyValue;
   PFNSTARTPAGE                 pfnStartPage;
   PFNENDPAGE                   pfnEndPage;
   PFNBEGINJOB                  pfnBeginJob;
   PFNENDJOB                    pfnEndJob;
   PFNABORTJOB                  pfnAbortJob;
   PFNRASTERIZE                 pfnRasterize;
   PFNFREEINSTANCE              pfnFreeInstance;
   PFNSETOUTPUTSTREAM           pfnSetOutputStream;
} PDC_B_HOOKPOINTS, *PPDC_B_HOOKPOINTS;

void          initializePDCBlitter          (PPDC_B_HOOKPOINTS pHookPoints);

#endif
