/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _UPDF_Device
#define _UPDF_Device

#include <Device.hpp>
#include <OmniProxy.hpp>

#include <libxml/parser.h>
#include <libxml/tree.h>

struct UPDF_Instance;

class UPDF_Device : public PrintDevice
{
public:
   enum {
      TOP_TO_BOTTOM = 1
   };

                              UPDF_Device              (xmlDocPtr    docDC,
                                                        xmlDocPtr    docUDR,
                                                        char        *pszDriverName,
                                                        char        *pszDeviceName,
                                                        char        *pszShortName);
                              UPDF_Device              (xmlDocPtr    docDC,
                                                        xmlDocPtr    docUDR,
                                                        char        *pszDriverName,
                                                        char        *pszDeviceName,
                                                        char        *pszShortName,
                                                        char        *pszJobProperties);
                             ~UPDF_Device              ();

   virtual DeviceGamma       *getCurrentGamma          ();

   virtual DeviceOrientation *getDefaultOrientation    ();
   virtual char              *getDefaultDitherID       ();
   virtual DeviceForm        *getDefaultForm           ();
   virtual DeviceTray        *getDefaultTray           ();
   virtual DeviceMedia       *getDefaultMedia          ();
   virtual DeviceResolution  *getDefaultResolution     ();
   virtual DeviceCommand     *getDefaultCommands       ();
   virtual DeviceData        *getDefaultData           ();
   virtual DeviceString      *getDefaultString         ();
   virtual DevicePrintMode   *getDefaultPrintMode      ();

   static UPDF_Device        *isAUPDF_Device           (Device      *pDevice);

   xmlNodePtr                 findDCEntry              (xmlNodePtr   root,
                                                        char        *pszName,
                                                        bool         fDebugOutput);
   xmlNodePtr                 findUDREntry             (xmlNodePtr   root,
                                                        char        *pszName,
                                                        bool         fDebugOutput);
   xmlNodePtr                 findLocaleEntry          (xmlNodePtr   root,
                                                        char        *pszName,
                                                        bool         fDebugOutput);
   xmlNodePtr                 findCSEntry              (xmlNodePtr   root,
                                                        char        *pszName,
                                                        bool         fDebugOutput);

   xmlNodePtr                 findDCEntryKeyValue      (char        *pszKey,
                                                        char        *pszValue,
                                                        bool         fDebugOutput);
   xmlNodePtr                 findUDREntryKeyValue     (char        *pszKey,
                                                        char        *pszValue,
                                                        bool         fDebugOutput);
   xmlNodePtr                 findLocaleEntryKeyValue  (char        *pszKey,
                                                        char        *pszValue,
                                                        bool         fDebugOutput);
   xmlNodePtr                 findCSEntryKeyValue      (char        *pszKey,
                                                        char        *pszValue,
                                                        bool         fDebugOutput);

   static xmlNodePtr          findEntry                (xmlNodePtr   root,
                                                        char        *pszName,
                                                        bool         fDebugOutput);

   xmlNodePtr                 findEntryKeyValue        (xmlNodePtr   root,
                                                        char        *pszKey,
                                                        char        *pszValue,
                                                        bool         fDebugOutput);

   int                        getVirtualUnits          ();

   xmlNodePtr                 getRootDC                ();
   xmlNodePtr                 getRootUDR               ();
   xmlNodePtr                 getRootLocale            ();
   xmlNodePtr                 getRootCS                ();

private:
   void                       commonInit               (xmlDocPtr    docDC,
                                                        xmlDocPtr    docUDR);

   char                      *getDefaultFeature        (char        *pszKey);

   int                        setVirtualUnits          ();

   UPDF_Instance      *pInstance_d;

   xmlDocPtr           docDC_d;          // Device Configuration
   xmlNodePtr          nodeRootDC_d;

   xmlDocPtr           docUDR_d;         // Unit Description Reference
   xmlNodePtr          nodeRootUDR_d;

   xmlDocPtr           docLocale_d;      // Locale
   xmlNodePtr          nodeRootLocale_d;

   xmlDocPtr           docCS_d;          // Command Sequences
   xmlNodePtr          nodeRootCS_d;

   int                 iVirtualUnits_d;

   char               *pszDriverName_d;
   char               *pszDeviceName_d;
   char               *pszShortName_d;
};

static inline xmlNodePtr
firstNode (xmlNodePtr elm)
{
   while (  0 != elm
         && (  XML_COMMENT_NODE == elm->type
            || XML_TEXT_NODE    == elm->type
            )
         )
   {
      // Move to the next node
      elm = elm->next;
   }

   return elm;
}

static inline xmlNodePtr
nextNode (xmlNodePtr elm)
{
   do
   {
      // Move to the next node
      elm = elm->next;

   } while (  0 != elm
           && (  XML_COMMENT_NODE == elm->type
              || XML_TEXT_NODE    == elm->type
              )
           );

   return elm;
}

static inline void
printXMLNode (xmlNodePtr elm,
              bool       fDebugOutput)
{
   if (fDebugOutput)
   {
      xmlElemDump (stderr, elm->doc, elm);
   }
}

#define FINDDCENTRY(pUPDFDevice, \
                    elm,         \
                    pszName,     \
                    pszFunction) \
   pUPDFDevice->findDCEntry (elm, pszName, DebugOutput::shouldOutput##pszFunction ())
#define FINDUDRENTRY(pUPDFDevice, \
                     elm,         \
                     pszName,     \
                     pszFunction) \
   pUPDFDevice->findUDREntry (elm, pszName, DebugOutput::shouldOutput##pszFunction ())
#define FINDLOCALEENTRY(pUPDFDevice, \
                        elm,         \
                        pszName,     \
                        pszFunction) \
   pUPDFDevice->findLocaleEntry (elm, pszName, DebugOutput::shouldOutput##pszFunction ())
#define FINDCSENTRY(pUPDFDevice, \
                    elm,         \
                    pszName,     \
                    pszFunction) \
   pUPDFDevice->findCSEntry (elm, pszName, DebugOutput::shouldOutput##pszFunction ())

#define FINDDCENTRYKEYVALUE(pUPDFDevice, \
                            pszKey,      \
                            pszValue,    \
                            pszFunction) \
   pUPDFDevice->findDCEntryKeyValue (pszKey, pszValue, DebugOutput::shouldOutput##pszFunction ())
#define FINDUDRENTRYKEYVALUE(pUPDFDevice, \
                             pszKey,      \
                             pszValue,    \
                             pszFunction) \
   pUPDFDevice->findUDREntryKeyValue (pszKey, pszValue, DebugOutput::shouldOutput##pszFunction ())
#define FINDLOCALEENTRYKEYVALUE(pUPDFDevice, \
                                pszKey,      \
                                pszValue,    \
                                pszFunction) \
   pUPDFDevice->findLocaleEntryKeyValue (pszKey, pszValue, DebugOutput::shouldOutput##pszFunction ())
#define FINDCSENTRYKEYVALUE(pUPDFDevice, \
                            pszKey,      \
                            pszValue,    \
                            pszFunction) \
   pUPDFDevice->findCSEntryKeyValue (pszKey, pszValue, DebugOutput::shouldOutput##pszFunction ())

extern "C" {
   Device *newDeviceW_Advanced         (bool    fAdvanced);
   Device *newDeviceW_JopProp_Advanced (char   *pszJobProperties, bool fAdvanced);
   void    deleteDevice                (Device *pDevice);
};

#endif
