/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <UPDF_DeviceTray.hpp>

/*
*/
static int     getOmniTrayID             (xmlNodePtr      node);

UPDF_DeviceTray::
UPDF_DeviceTray (Device      *pDevice,
                 int          id,
                 int          iType,
                 BinaryData  *data,
                 xmlNodePtr   node)
   : DeviceTray (pDevice,
                 id,
                 iType,
                 data)
{
   node_d = node;
}

UPDF_DeviceTray::
~UPDF_DeviceTray ()
{
}

DeviceTray * UPDF_DeviceTray::
createV (int id)
{
   return UPDF_DeviceTray::create (pDevice_d, id);
}

DeviceTray * UPDF_DeviceTray::
create (Device *pDevice, char *pszId)
{
   int id = DeviceTray::nameToID (pszId);

   if (DeviceTray::TRAY_UNLISTED != id)
   {
      return create (pDevice, id);
   }
   else
   {
      UPDF_Device *pUPDFDevice = UPDF_Device::isAUPDF_Device (pDevice);
      xmlNodePtr   nodeTrays   = 0;

      nodeTrays = findTrays (pUPDFDevice);

      if (nodeTrays)
      {
         xmlNodePtr nodeItem  = firstNode (nodeTrays->xmlChildrenNode);
         xmlNodePtr nodeFound = 0;

         while (nodeItem != 0)
         {
            char *pszNodeId = (char *)xmlGetProp (nodeItem, (xmlChar *)"ID");

            if (pszNodeId)
            {
               if (0 == strcmp (pszNodeId, pszId))
               {
                  nodeFound = nodeItem;
               }

               free (pszNodeId);
            }

            if (nodeFound)
            {
               id = getOmniTrayID (nodeFound);

               if (DeviceTray::TRAY_UNLISTED != id)
               {
                  return create (pDevice, id);
               }

               nodeFound = 0;
            }

            nodeItem = nextNode (nodeItem);
         }
      }

      return 0;
   }
}

static int
getOmniTrayID (xmlNodePtr node)
{
   int id = DeviceTray::TRAY_UNLISTED;

   if (!node)
   {
      return id;
   }

   char *pszID = (char *)xmlGetProp (node, (xmlChar *)"ID");

   if (pszID)
   {
      if (0 == strcasecmp (pszID, "Source_Auto"))
         id = DeviceTray::TRAY_AUTO;
      else if (0 == strcasecmp (pszID, "Source_Tray2"))
         id = DeviceTray::TRAY_TRAY_2;
      else if (0 == strcasecmp (pszID, "Source_Tray3"))
         id = DeviceTray::TRAY_TRAY_3;

      free (pszID);
   }

   return id;
}

static xmlNodePtr
skipInvalidTrays (xmlNodePtr node)
{
   if (!node)
   {
      return 0;
   }

   int id = getOmniTrayID (node);

   while (  node
           && (  id == DeviceTray::TRAY_UNLISTED
              || id == DeviceTray::TRAY_NONE
              )
           )
   {
      node = nextNode (node);
      id   = getOmniTrayID (node);
   }

   return node;
}

static DeviceTray *
createFromXMLNode (Device      *pDevice,
                   xmlNodePtr   node,
                   int          id)
{
   UPDF_Device *pUPDFDevice = UPDF_Device::isAUPDF_Device (pDevice);

   if (!pUPDFDevice)
   {
      return 0;
   }

   int          iType = 0;
   BinaryData  *data  = 0;

   return new UPDF_DeviceTray (pUPDFDevice,
                               id,
                               iType,
                               data,
                               node);
}

DeviceTray * UPDF_DeviceTray::
create (Device *pDevice, int id)
{
   UPDF_Device *pUPDFDevice = UPDF_Device::isAUPDF_Device (pDevice);

#ifndef RETAIL
   if (DebugOutput::shouldOutputUPDFDeviceTray ()) DebugOutput::getErrorStream () << "UPDF_DeviceTray::" << __FUNCTION__ << ": pDevice = " << pDevice << ", id = " << id << endl;
#endif

   if (!pUPDFDevice)
   {
      return 0;
   }

   xmlNodePtr nodeTrays = 0;

   nodeTrays = findTrays (pUPDFDevice);

   if (nodeTrays)
   {
      xmlNodePtr nodeItem = firstNode (nodeTrays->xmlChildrenNode);

      while (nodeItem != 0)
      {
         int iTrayID = getOmniTrayID (nodeItem);

         if (  id == iTrayID
            && !(  id == DeviceTray::TRAY_UNLISTED
                || id == DeviceTray::TRAY_NONE
                )
            )
         {
            return createFromXMLNode (pDevice, nodeItem, id);
         }

         nodeItem = nextNode (nodeItem);
      }

      return 0;
   }
   else
   {
      return 0;
   }
}

char * UPDF_DeviceTray::
getDeviceID ()
{
   return 0;
}

bool UPDF_DeviceTray::
isTraySupported (int id)
{
   UPDF_Device *pUPDFDevice = UPDF_Device::isAUPDF_Device (pDevice_d);

   if (!pUPDFDevice)
   {
      return 0;
   }

   xmlNodePtr nodeTrays = 0;

   nodeTrays = findTrays (pUPDFDevice);

   if (nodeTrays)
   {
      xmlNodePtr nodeItem = firstNode (nodeTrays->xmlChildrenNode);

      while (nodeItem != 0)
      {
         int iTrayID = getOmniTrayID (nodeItem);

         if (  id == iTrayID
            && !(  id == DeviceTray::TRAY_UNLISTED
                || id == DeviceTray::TRAY_NONE
                )
            )
         {
            return true;
         }

         nodeItem = nextNode (nodeItem);
      }
   }

   return false;
}

Enumeration * UPDF_DeviceTray::
getEnumeration ()
{
   class UPDF_Tray_Enumerator : public Enumeration
   {
   public:
      UPDF_Tray_Enumerator (Device *pDevice, xmlNodePtr nodeItem)
      {
         pDevice_d  = pDevice;
         nodeItem_d = skipInvalidTrays (nodeItem);
      }

      virtual bool hasMoreElements ()
      {
         return nodeItem_d ? true : false;
      }

      virtual void *nextElement ()
      {
         void *pvRet = 0;

         if (nodeItem_d)
         {
            int id = getOmniTrayID (nodeItem_d);

            pvRet = createFromXMLNode (pDevice_d,
                                       nodeItem_d,
                                       id);

            nodeItem_d = nextNode (nodeItem_d);
            nodeItem_d = skipInvalidTrays (nodeItem_d);
         }

         return pvRet;
      }

   private:
      Device     *pDevice_d;
      xmlNodePtr  nodeItem_d;
   };

   UPDF_Device *pUPDFDevice = UPDF_Device::isAUPDF_Device (pDevice_d);

   if (!pUPDFDevice)
   {
      return new UPDF_Tray_Enumerator (pDevice_d, 0);
   }

   xmlNodePtr nodeTrays = 0;

   nodeTrays = findTrays (pUPDFDevice);

   if (nodeTrays)
   {
      xmlNodePtr nodeItem = firstNode (nodeTrays->xmlChildrenNode);

      return new UPDF_Tray_Enumerator (pDevice_d, nodeItem);
   }
   else
   {
      return new UPDF_Tray_Enumerator (pDevice_d, 0);
   }
}

xmlNodePtr UPDF_DeviceTray::
findTrays (UPDF_Device *pUPDFDevice)
{
   xmlNodePtr nodeTrays = 0;

   if (!pUPDFDevice)
   {
      return nodeTrays;
   }

   if (  ((nodeTrays = FINDUDRENTRY (pUPDFDevice, nodeTrays, "Device_Features",    UPDFDeviceTray)) != 0)
      && ((nodeTrays = FINDUDRENTRY (pUPDFDevice, nodeTrays, "PrintCapabilities",  UPDFDeviceTray)) != 0)
      && ((nodeTrays = FINDUDRENTRY (pUPDFDevice, nodeTrays, "Features",           UPDFDeviceTray)) != 0)
      && ((nodeTrays = FINDUDRENTRY (pUPDFDevice, nodeTrays, "PrintMediaHandling", UPDFDeviceTray)) != 0)
      && ((nodeTrays = FINDUDRENTRY (pUPDFDevice, nodeTrays, "MediaSourceList",    UPDFDeviceTray)) != 0)
      )
      return nodeTrays;

   return nodeTrays;
}

string UPDF_DeviceTray::
toString (ostringstream& oss)
{
   ostringstream oss2;

   oss << "{"
       << DeviceTray::toString (oss2)
       << "}";

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const UPDF_DeviceTray& const_self)
{
   UPDF_DeviceTray& self = const_cast<UPDF_DeviceTray&>(const_self);
   ostringstream    oss;

   os << self.toString (oss);

   return os;
}
