/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _UPDF_Instance
#define _UPDF_Instance

#include <UPDF_Device.hpp>
#include "ParameterConverter.hpp"

//#define HAVE_SYMBOL_TABLE 1

typedef map <string, xmlNodePtr> XMLObjectMap;

class UPDF_Instance : public DeviceInstance
{
public:
                        UPDF_Instance             (PrintDevice         *pDevice);
   virtual             ~UPDF_Instance             ();

   void                 initializeInstance        ();

   virtual bool         handleDeviceJobProperty   (char                *pszKey,
                                                   char                *pszValue);
// virtual string      *queryDeviceJobProperties  ();
   virtual Enumeration *listDeviceJobPropertyKeys ();
   virtual Enumeration *listKeyValues             (char                *pszKey);
   virtual string      *getJobPropertyType        (char                *pszKey);
   virtual string      *getJobProperty            (char                *pszKey);
   virtual string      *translateKeyValue         (char                *pszKey,
                                                   char                *pszValue);

   virtual bool         beginJob                  ();
   virtual bool         beginJob                  (bool                 fJobPropertiesChanged);
   virtual bool         newFrame                  ();
   virtual bool         newFrame                  (bool                 fJobPropertiesChanged);
   virtual bool         endJob                    ();
   virtual bool         abortJob                  ();

   virtual string       toString                  (ostringstream&       oss);
   friend ostream&      operator<<                (ostream&             os,
                                                   const UPDF_Instance& device);

   void                 setupPrinter              ();

   char                *getXMLObjectValue         (char                *pszObjectName,
                                                   char                *pszVariableName);

   bool                 executeEvent              (char                *pszPrefix,
                                                   bool                 fPreEvent);

   POINTL               ptlPrintHead_d;

private:
   void                 loadNonDominantDefaults   (UPDF_Device         *pUPDFDevice,
                                                   xmlNodePtr           node);
   void                 loadLocaleDefaults        ();

   bool                 fHaveInitialized_d;
   bool                 fHaveSetupPrinter_d;
#ifdef HAVE_SYMBOL_TABLE
   SYMBOLTABLE          symbolTable_d;
#endif

   XMLObjectMap         xmlObjectMap_d;
};

#endif
