/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <Device.hpp>

Device *
createDevice (char     *pszDriverLibrary,
              char     *pszJobProperties,
              bool      fAdvanced,
              GModule **phmodDevice)
{
#define LIBDEVICENAME3(s) STRINGIZE(s)
#define LIBDEVICENAME2(x) LIBDEVICENAME3(lib##x##.so)
#define LIBDEVICENAME1(x) LIBDEVICENAME2(x)
#define LIBDEVICENAME     LIBDEVICENAME1(DEVICENAME)

   char               achLibDeviceName[512]; // @TBD - need max path define
   Device            *pDevice           = 0;
   PFNNEWDEVICE       pfnNewDevice      = 0;
   PFNNEWDEVICEWARGS  pfnNewDeviceWArgs = 0;

   if (  0 == pszDriverLibrary
      || !*pszDriverLibrary
      )
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": pszDriverLibrary is not filled in!" << endl;
      return 0;
   }

   if (0 != strncasecmp (pszDriverLibrary, "lib", 3))
   {
      sprintf (achLibDeviceName, "lib%s.so", pszDriverLibrary);
   }
   else
   {
      strcpy (achLibDeviceName, pszDriverLibrary);
   }

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester ())
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": Trying to load " << pszDriverLibrary << endl;
   }
#endif

   *phmodDevice = g_module_open (pszDriverLibrary, (GModuleFlags)0);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester ())
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": *phmodDevice = " << hex << *phmodDevice << dec << endl;
   }
#endif

   if (!*phmodDevice)
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": g_module_error returns " << g_module_error () << endl;
      return 0;
   }

   // nm libEpson_Stylus_Color_860.so | grep newDevice

   // returns 00011e20 T newDeviceW_Advanced for
   // extern Device *newDevice (bool fAdvanced);
   g_module_symbol (*phmodDevice, "newDeviceW_Advanced", (gpointer *)&pfnNewDevice);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester ())
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": pfnNewDevice = 0x" << hex << (int)pfnNewDevice << dec << endl;
   }
#endif

   if (!pfnNewDevice)
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": g_module_error returns " << g_module_error () << endl;

      return 0;
   }

   // returns 00011d30 T newDeviceW_JopProp_Advanced for
   // extern Device *newDevice (char *pszJobProperties, bool fAdvanced);
   g_module_symbol (*phmodDevice, "newDeviceW_JopProp_Advanced", (gpointer *)&pfnNewDeviceWArgs);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester ())
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": pfnNewDeviceWArgs = 0x" << hex << (int)pfnNewDeviceWArgs << dec << endl;
   }
#endif

   if (!pfnNewDeviceWArgs)
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": g_module_error returns " << g_module_error () << endl;

      return 0;
   }

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester ())
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": ********** Creating device with passed in properties **********" << endl;
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": The Properties are: " << pszJobProperties << endl;
   }
#endif

   pDevice = pfnNewDeviceWArgs (pszJobProperties, fAdvanced);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester ())
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": pfnNewDeviceWArgs (...) returns " << *pDevice << endl;
   }
#endif

   return pDevice;
}

void
printUsage (char *pszExeName)
{
   DebugOutput::getErrorStream () << "Usage: " << pszExeName << " required_parms optional_parms" << endl
      << "   where required_parms are:" << endl
      << "      UPDFDeviceName device_name" << endl
      << "   where optional_parms are:" << endl
      << "      orientation orientation_id" << endl
      << "      form form_id" << endl
      << "      tray tray_id" << endl
      << "      media media_id" << endl
      << "      resolution resolution_id" << endl
      << "      dithering dithering_id" << endl
      << "      printmode printmode_id" << endl
      << "      OtherJobPropertyKey value" << endl;
}

int
main (int argc, char *argv[])
{
#define JP_BLOCK_SIZE  256
   char              *pszDriverLibrary  = "libUPDFDevice.so";
   string             stringJobProp     = "";
   GModule           *hmodDevice        = 0;
   Enumeration       *pEnum             = 0;
   Device            *pDevice           = 0;
   DeviceOrientation *pDO               = 0;
   DeviceForm        *pDF               = 0;
   HardCopyCap       *pHCC              = 0;
   DeviceTray        *pDT               = 0;
   DeviceMedia       *pDM               = 0;
   DeviceResolution  *pDR               = 0;
   DevicePrintMode   *pDPM              = 0;
   int                rc                = 0;

   DebugOutput::setDebugOutput ("UPDF");

   for (int i = 1; i < argc; i++)
   {
      if (0 == strcmp ("UPDFDeviceName", argv[i]))
      {
         i++;

         if (3 > argc - i)
         {
            cerr << "Error: not enough parameters." << endl;
            cerr << "Error: Must be in form of: UPDFDeviceName begin-quote ... end-quote." << endl;

            rc = 1;
            goto BUGOUT;
         }
         if (0 != strcmp ("begin-quote", argv[i]))
         {
            cerr << "Error: Missing begin-quote keyword." << endl;
            cerr << "Error: Must be in form of: UPDFDeviceName begin-quote ... end-quote." << endl;

            rc = 2;
            goto BUGOUT;
         }

         i++;

         if (stringJobProp.c_str ()[0])
            stringJobProp += " ";
         stringJobProp += "UPDFDeviceName=\"";

         bool fFirst = true;

         while (  1 <= argc - i
               && 0 != strcmp ("end-quote", argv[i])
               )
         {
            if (!fFirst)
            {
               stringJobProp += " ";
            }
            else
            {
               fFirst = false;
            }

            stringJobProp += argv[i];
            i++;
         }

         if (  1 > argc - i
            || 0 != strcmp ("end-quote", argv[i])
            )
         {
            cerr << "Error: Missing end-quote." << endl;
            cerr << "Error: Must be in form of: UPDFDeviceName begin-quote ... end-quote." << endl;

            rc = 3;
            goto BUGOUT;
         }

         stringJobProp += "\"";
      }
      else
      {
         // Assume that it is a Job Property (where there needs to be at least
         // 2 parameters)
         if (2 <= argc - i)
         {
            if (stringJobProp.c_str ()[0])
               stringJobProp += " ";
            stringJobProp += argv[i];
            stringJobProp += "=";
            stringJobProp += argv[i + 1];

            i++;
         }
         else
         {
            printUsage (argv[0]);

            rc = 4;
            goto BUGOUT;
         }
      }
   }

   cerr << "The job properties are \"" << stringJobProp << "\"" << endl;

   pDevice = createDevice (pszDriverLibrary,
                           (char *)stringJobProp.c_str (),
                           true,
                           &hmodDevice);

   pDO = pDevice->getCurrentOrientation ();
   pEnum = pDO->getEnumeration ();

   cout << "Current orientation is:" << endl;
   cout << DeviceOrientation::IDToName (pDO->getID ()) << endl;

   cout << "Portrait is";
   if (!pDO->isOrientationSupported (DeviceOrientation::ORIENTATION_PORTRAIT))
      cout << " not";
   cout << " supported." << endl;
   cout << "Landscape is";
   if (!pDO->isOrientationSupported (DeviceOrientation::ORIENTATION_LANDSCAPE))
      cout << " not";
   cout << " supported." << endl;
   cout << "Reverse-Portrait is";
   if (!pDO->isOrientationSupported (DeviceOrientation::ORIENTATION_REVERSE_PORTRAIT))
      cout << " not";
   cout << " supported." << endl;
   cout << "Reverse-Landscape is";
   if (!pDO->isOrientationSupported (DeviceOrientation::ORIENTATION_REVERSE_LANDSCAPE))
      cout << " not";
   cout << " supported." << endl;

   cout << "Enumerating orientations:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDO = (DeviceOrientation *)pEnum->nextElement ();

      cout << DeviceOrientation::IDToName (pDO->getID ()) << endl;

      delete pDO;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pDF = pDevice->getCurrentForm ();
   pEnum = pDF->getEnumeration ();

   cout << "Current form is:" << endl;
   cout << DeviceForm::IDToName (pDF->getID ()) << endl;
   pHCC = pDF->getHardCopyCap ();
   cout << "Form dimensions are: ("
        << pHCC->getCx ()
        << ","
        << pHCC->getCy ()
        << ") (L:"
        << pHCC->getLeftClip ()
        << ",T:"
        << pHCC->getTopClip ()
        << ",R:"
        << pHCC->getRightClip ()
        << ",B:"
        << pHCC->getBottomClip ()
        << ")"
        << endl;

   cout << "Enumerating forms:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDF = (DeviceForm *)pEnum->nextElement ();

      cout << DeviceForm::IDToName (pDF->getID ()) << endl;

      delete pDF;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pDT = pDevice->getCurrentTray ();
   pEnum = pDT->getEnumeration ();

   cout << "Current tray is:" << endl;
   cout << DeviceTray::IDToName (pDT->getID ()) << endl;

   cout << "Enumerating trays:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDT = (DeviceTray *)pEnum->nextElement ();

      cout << DeviceTray::IDToName (pDT->getID ()) << endl;

      delete pDT;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pDM = pDevice->getCurrentMedia ();
   pEnum = pDM->getEnumeration ();

   cout << "Current media is:" << endl;
   cout << DeviceMedia::IDToName (pDM->getID ()) << endl;

   cout << "Enumerating medias:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDM = (DeviceMedia *)pEnum->nextElement ();

      cout << DeviceMedia::IDToName (pDM->getID ()) << endl;

      delete pDM;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pDR = pDevice->getCurrentResolution ();
   pEnum = pDR->getEnumeration ();

   cout << "Current resolution is:" << endl;
   cout << DeviceResolution::IDToName (pDR->getID ()) << endl;

   cout << "Resolution dimensions are: ("
        << pDR->getXRes ()
        << ","
        << pDR->getYRes ()
        << ")"
        << endl;

   cout << "Enumerating resolutions:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDR = (DeviceResolution *)pEnum->nextElement ();

      cout << DeviceResolution::IDToName (pDR->getID ()) << endl;

      delete pDR;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pDPM = pDevice->getCurrentPrintMode ();
   pEnum = pDPM->getEnumeration ();

   cout << "Current print mode is:" << endl;
   cout << DevicePrintMode::IDToName (pDPM->getID ()) << endl;

   cout << "Print Mode dimensions are: (CT:"
        << pDPM->getColorTech ()
        << ",P:"
        << pDPM->getPhysicalCount ()
        << ",L:"
        << pDPM->getLogicalCount ()
        << ",#planes:"
        << pDPM->getNumPlanes ()
        << ")"
        << endl;

   cout << "Enumerating print modes:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDPM = (DevicePrintMode *)pEnum->nextElement ();

      cout << DevicePrintMode::IDToName (pDPM->getID ()) << endl;

      delete pDPM;
   }
   cout << "Done!" << endl;

   delete pEnum;

   rc = 0;

BUGOUT:
   // Clean up
   delete pDevice;

   if (hmodDevice)
   {
#ifndef RETAIL
      int rc2 =
#endif
           g_module_close (hmodDevice);

#ifndef RETAIL
      if (DebugOutput::shouldOutputDeviceTester ())
      {
         DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": g_module_close returns " << rc2 << endl;
      }
#endif
   }

   return rc;
}
