/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "Device.hpp"
#include "Omni.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <dlfcn.h>

int
handleDevice (char *pszFullDeviceName)
{
   Device       *pDevice      = 0;
   PFNNEWDEVICE  pfnNewDevice = 0;
   void         *vhDevice     = 0;

   cout << "Trying to load " << pszFullDeviceName << endl;

   vhDevice = dlopen (pszFullDeviceName, RTLD_NOW | RTLD_GLOBAL);

   if (DebugOutput::shouldOutputDeviceTester ()) cerr << "vhDevice = " << hex << vhDevice << endl;

   if (!vhDevice)
   {
      cerr << "dlerror returns " << dlerror () << endl;
      return 2;
   }

   pfnNewDevice = (PFNNEWDEVICE)dlsym (vhDevice, "newDeviceW_Advanced");

   if (DebugOutput::shouldOutputDeviceTester ()) cerr << hex << "pfnNewDevice = 0x" << (int)pfnNewDevice << endl;

   if (!pfnNewDevice)
      return 3;

   pDevice = pfnNewDevice (true);

   if (DebugOutput::shouldOutputDeviceTester ()) cerr << "pDevice = " << *pDevice << endl;

   DeviceForm  *pDF = pDevice->getCurrentForm ();
   Enumeration *pEnum = pDF->getEnumeration ();

   cout << "Enumerating forms:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDF = (DeviceForm *)pEnum->nextElement ();

      HardCopyCap *pHCC = pDF->getHardCopyCap ();

      cout << DeviceForm::IDToName (pDF->getID ())
           << " \""
           << pDF->getName ()
           << "\" ("
           << pHCC->getCx ()
           << ", "
           << pHCC->getCy ()
           << ", "
           << pHCC->getLeftClip ()
           << ", "
           << pHCC->getTopClip ()
           << ", "
           << pHCC->getRightClip ()
           << ", "
           << pHCC->getBottomClip ()
           << ")"
           << endl;

      delete pDF;
   }
   cout << "Done!" << endl;

   delete pEnum;

   delete pDevice;

   int rc = 0;

   rc = dlclose (vhDevice);

   if (DebugOutput::shouldOutputDeviceTester ()) cerr << dec << "dlclose returns " << rc << endl;

   return 0;
}

int
getFormCx (int iFormID)
{
   switch (iFormID)
   {
   case DeviceForm::FORM_TEST:                  return 130050;
   case DeviceForm::FORM_TEST2:                 return 0;        // @TBD
   case DeviceForm::FORM_LEDGER:                return 279400;
   case DeviceForm::FORM_LEGAL:                 return 215900;
   case DeviceForm::FORM_LEGAL_EXTRA:           return 241300;
   case DeviceForm::FORM_LETTER:                return 215900;
   case DeviceForm::FORM_LETTER_PLUS:           return 215900;
   case DeviceForm::FORM_LETTER_EXTRA:          return 241300;
   case DeviceForm::FORM_HALF_LETTER:           return 139690;
   case DeviceForm::FORM_EXECUTIVE:             return 184150;
   case DeviceForm::FORM_STATEMENT:             return 139700;
   case DeviceForm::FORM_TABLOID:               return 279400;
   case DeviceForm::FORM_UNIVERSAL:             return 297040;
   case DeviceForm::FORM_FOOLSCAP:              return 203200;
   case DeviceForm::FORM_QUARTO:                return 215900;
   case DeviceForm::FORM_FOLIO:                 return 215900;
   case DeviceForm::FORM_FOLIO_SP:              return 215000;
   case DeviceForm::FORM_ROC_8K:                return 273050;
   case DeviceForm::FORM_ROC_16K:               return 196850;
   case DeviceForm::FORM_GOVERNMENT_LETTER:     return 203200;
   case DeviceForm::FORM_GOVERNMENT_LEGAL:      return 203200;
   case DeviceForm::FORM_EDP:                   return 279400;
   case DeviceForm::FORM_EUROPEAN_EDP:          return 294000;
   case DeviceForm::FORM_PRC_16K:               return 146000;
   case DeviceForm::FORM_PRC_32K:               return 97000;
   case DeviceForm::FORM_DAI_PA_KAI:            return 275000;
   case DeviceForm::FORM_JUURO_KU_KAI:          return 198000;
   case DeviceForm::FORM_PA_KAI:                return 267000;
   case DeviceForm::FORM_WIDE:                  return 345000;
   case DeviceForm::FORM_LETTER_WIDE:           return 228400;
   case DeviceForm::FORM_FOOLSCAP_WIDE:         return 215900;
   case DeviceForm::FORM_A3_WIDE:               return 330200;
   case DeviceForm::FORM_A4_WIDE:               return 223500;
   case DeviceForm::FORM_WIDE_FORMAT:           return 762000;
   case DeviceForm::FORM_10_X_11:               return 254000;
   case DeviceForm::FORM_11_X_12:               return 279400;
   case DeviceForm::FORM_11_X_15:               return 279400;
   case DeviceForm::FORM_12_X_19:               return 304800;
   case DeviceForm::FORM_15_X_11:               return 381000;
   case DeviceForm::FORM_5_X_7:                 return 127000;
   case DeviceForm::FORM_3_X_5_CARD:            return 76200;
   case DeviceForm::FORM_4_X_6_CARD:            return 101600;
   case DeviceForm::FORM_5_X_8_CARD:            return 127000;
   case DeviceForm::FORM_8_X_10_CARD:           return 203200;
   case DeviceForm::FORM_A6_CARD:               return 104980;
   case DeviceForm::FORM_CARD_148:              return 148000;
   case DeviceForm::FORM_HAGAKI_CARD:           return 100000;
   case DeviceForm::FORM_OUFUKU_CARD:           return 148000;
   case DeviceForm::FORM_POSTCARD:              return 96970;
   case DeviceForm::FORM_INDEX_4_X_6_EXT:       return 152400;
   case DeviceForm::FORM_A2_ENVELOPE:           return 111125;
   case DeviceForm::FORM_C5_ENVELOPE:           return 162000;
   case DeviceForm::FORM_C6_ENVELOPE:           return 114000;
   case DeviceForm::FORM_C7_ENVELOPE:           return 98400;
   case DeviceForm::FORM_C9_ENVELOPE:           return 98400;
   case DeviceForm::FORM_C10_ENVELOPE:          return 104700;
   case DeviceForm::FORM_D5_ENVELOPE:           return 176000;
   case DeviceForm::FORM_DL_ENVELOPE:           return 110000;
   case DeviceForm::FORM_ENVELOPE_132_X_220:    return 132000;
   case DeviceForm::FORM_ENVELOPE_6_X_9:        return 152400;
   case DeviceForm::FORM_ENVELOPE_7_X_9:        return 177800;
   case DeviceForm::FORM_ENVELOPE_6_1_2:        return 165100;
   case DeviceForm::FORM_ENVELOPE_9_X_11:       return 228600;
   case DeviceForm::FORM_ENVELOPE_10_X_13:      return 254000;
   case DeviceForm::FORM_ENVELOPE_10_X_14:      return 254000;
   case DeviceForm::FORM_ENVELOPE_10_X_15:      return 254000;
   case DeviceForm::FORM_MONARCH_ENVELOPE:      return 98425;
   case DeviceForm::FORM_PERSONAL_ENVELOPE:     return 92075;
   case DeviceForm::FORM_NUMBER_9_ENVELOPE:     return 98425;
   case DeviceForm::FORM_NUMBER_10_ENVELOPE:    return 104775;
   case DeviceForm::FORM_NUMBER_11_ENVELOPE:    return 114300;
   case DeviceForm::FORM_NUMBER_12_ENVELOPE:    return 120650;
   case DeviceForm::FORM_NUMBER_14_ENVELOPE:    return 127000;
   case DeviceForm::FORM_ITALIAN_ENVELOPE:      return 100000;
   case DeviceForm::FORM_POSTFIX_ENVELOPE:      return 114000;
   case DeviceForm::FORM_INVITE_ENVELOPE:       return 220000;
   case DeviceForm::FORM_CHOU2_ENVELOPE:        return 111100;
   case DeviceForm::FORM_CHOU3_ENVELOPE:        return 120000;
   case DeviceForm::FORM_CHOU4_ENVELOPE:        return 90000;
   case DeviceForm::FORM_KAHU_ENVELOPE:         return 240000;
   case DeviceForm::FORM_KAKU2_ENVELOPE:        return 240000;
   case DeviceForm::FORM_YOU4_ENVELOPE:         return 105000;
   case DeviceForm::FORM_PRC1_ENVELOPE:         return 102000;
   case DeviceForm::FORM_PRC2_ENVELOPE:         return 102000;
   case DeviceForm::FORM_PRC3_ENVELOPE:         return 125000;
   case DeviceForm::FORM_PRC4_ENVELOPE:         return 110000;
   case DeviceForm::FORM_PRC5_ENVELOPE:         return 110000;
   case DeviceForm::FORM_PRC6_ENVELOPE:         return 120000;
   case DeviceForm::FORM_PRC7_ENVELOPE:         return 160000;
   case DeviceForm::FORM_PRC8_ENVELOPE:         return 120000;
   case DeviceForm::FORM_PRC9_ENVELOPE:         return 229000;
   case DeviceForm::FORM_PRC10_ENVELOPE:        return 324000;
   case DeviceForm::FORM_DISK_LABELS:           return 54000;
   case DeviceForm::FORM_EURO_LABELS:           return 36000;
   case DeviceForm::FORM_SHIPPING_LABELS:       return 54000;
   case DeviceForm::FORM_STANDARD_LABELS_CLEAR: return 28000;
   case DeviceForm::FORM_STANDARD_LABELS_WHITE: return 28000;
   case DeviceForm::FORM_2A0:                   return 1189000;
   case DeviceForm::FORM_4A0:                   return 1682000;
   case DeviceForm::FORM_A0:                    return 841000;
   case DeviceForm::FORM_A1:                    return 594000;
   case DeviceForm::FORM_A2:                    return 420000;
   case DeviceForm::FORM_A3:                    return 297000;
   case DeviceForm::FORM_A3_EXTRA:              return 322000;
   case DeviceForm::FORM_A4:                    return 210000;
   case DeviceForm::FORM_A4_EXTRA:              return 235500;
   case DeviceForm::FORM_A4_TAB:                return 225000;
   case DeviceForm::FORM_A5:                    return 148000;
   case DeviceForm::FORM_A5_EXTRA:              return 174000;
   case DeviceForm::FORM_A6:                    return 105000;
   case DeviceForm::FORM_A7:                    return 74000;
   case DeviceForm::FORM_A8:                    return 52000;
   case DeviceForm::FORM_A9:                    return 37000;
   case DeviceForm::FORM_A10:                   return 26000;
   case DeviceForm::FORM_B_PLUS:                return 304800;
   case DeviceForm::FORM_B0:                    return 1000000;
   case DeviceForm::FORM_B1:                    return 707000;
   case DeviceForm::FORM_B2:                    return 500000;
   case DeviceForm::FORM_B3:                    return 353000;
   case DeviceForm::FORM_B4:                    return 250000;
   case DeviceForm::FORM_B5:                    return 176000;
   case DeviceForm::FORM_B5_EXTRA:              return 201000;
   case DeviceForm::FORM_B6:                    return 125000;
   case DeviceForm::FORM_B6_C4:                 return 125000;
   case DeviceForm::FORM_B7:                    return 88000;
   case DeviceForm::FORM_B8:                    return 62000;
   case DeviceForm::FORM_B9:                    return 44000;
   case DeviceForm::FORM_B10:                   return 31000;
   case DeviceForm::FORM_JIS_B0:                return 1030000;
   case DeviceForm::FORM_JIS_B1:                return 728000;
   case DeviceForm::FORM_JIS_B2:                return 515000;
   case DeviceForm::FORM_JIS_B3:                return 364000;
   case DeviceForm::FORM_JIS_B4:                return 257000;
   case DeviceForm::FORM_JIS_B5:                return 182000;
   case DeviceForm::FORM_JIS_B6:                return 128000;
   case DeviceForm::FORM_JIS_B7:                return 91000;
   case DeviceForm::FORM_JIS_B8:                return 64000;
   case DeviceForm::FORM_JIS_B9:                return 45000;
   case DeviceForm::FORM_JIS_B10:               return 32000;
   case DeviceForm::FORM_JIS_EXEC:              return 216000;
   case DeviceForm::FORM_C0:                    return 917000;
   case DeviceForm::FORM_C1:                    return 648000;
   case DeviceForm::FORM_C2:                    return 458000;
   case DeviceForm::FORM_C3:                    return 324000;
   case DeviceForm::FORM_C4:                    return 229000;
   case DeviceForm::FORM_C5:                    return 162000;
   case DeviceForm::FORM_C6:                    return 114000;
   case DeviceForm::FORM_C6_C5:                 return 114000;
   case DeviceForm::FORM_C7:                    return 81000;
   case DeviceForm::FORM_C7_C6:                 return 81000;
   case DeviceForm::FORM_C8:                    return 57000;
   case DeviceForm::FORM_C9:                    return 40000;
   case DeviceForm::FORM_C10:                   return 28000;
   case DeviceForm::FORM_E1:                    return 711200;
   case DeviceForm::FORM_RA0:                   return 860000;
   case DeviceForm::FORM_RA1:                   return 610000;
   case DeviceForm::FORM_RA2:                   return 430000;
   case DeviceForm::FORM_SRA0:                  return 900000;
   case DeviceForm::FORM_SRA1:                  return 640000;
   case DeviceForm::FORM_SRA2:                  return 450000;
   case DeviceForm::FORM_US_FANFOLD:            return 377825;
   case DeviceForm::FORM_FANFOLD_1:             return 368300;
   case DeviceForm::FORM_FANFOLD_2:             return 304800;
   case DeviceForm::FORM_FANFOLD_3:             return 241300;
   case DeviceForm::FORM_FANFOLD_4:             return 215900;
   case DeviceForm::FORM_FANFOLD_5:             return 215900;
   case DeviceForm::FORM_GERMAN_LEGAL_FANFOLD:  return 215900;
   case DeviceForm::FORM_EUROPEAN_FANFOLD:      return 215900;
   case DeviceForm::FORM_PANORAMIC:             return 210000;
   case DeviceForm::FORM_PHOTO_4_6:             return 101600;
   case DeviceForm::FORM_PHOTO_100_150:         return 100000;
   case DeviceForm::FORM_PHOTO_200_300:         return 200000;
   case DeviceForm::FORM_PLOTTER_SIZE_A:        return 215900;
   case DeviceForm::FORM_PLOTTER_SIZE_B:        return 279400;
   case DeviceForm::FORM_PLOTTER_SIZE_C:        return 431800;
   case DeviceForm::FORM_PLOTTER_SIZE_D:        return 558800;
   case DeviceForm::FORM_PLOTTER_SIZE_E:        return 863690;
   case DeviceForm::FORM_PLOTTER_SIZE_F:        return 1117600;
   case DeviceForm::FORM_US_A_ARCHITECTURAL:    return 228600;
   case DeviceForm::FORM_US_B_ARCHITECTURAL:    return 304800;
   case DeviceForm::FORM_US_C_ARCHITECTURAL:    return 457200;
   case DeviceForm::FORM_US_D_ARCHITECTURAL:    return 609600;
   case DeviceForm::FORM_US_E_ARCHITECTURAL:    return 914400;
   case DeviceForm::FORM_SUPER_A3_B:            return 328920;
   case DeviceForm::FORM_SUPER_A:               return 227076;
   case DeviceForm::FORM_SUPER_B:               return 330200;
   }

   return 0; // @TBD
}

int
getFormCy (int iFormID)
{
   switch (iFormID)
   {
   case DeviceForm::FORM_TEST:                  return 86700;
   case DeviceForm::FORM_TEST2:                 return 0;        // @TBD
   case DeviceForm::FORM_LEDGER:                return 431800;
   case DeviceForm::FORM_LEGAL:                 return 355600;
   case DeviceForm::FORM_LEGAL_EXTRA:           return 381000;
   case DeviceForm::FORM_LETTER:                return 279400;
   case DeviceForm::FORM_LETTER_PLUS:           return 322326;
   case DeviceForm::FORM_LETTER_EXTRA:          return 304800;
   case DeviceForm::FORM_HALF_LETTER:           return 215970;
   case DeviceForm::FORM_EXECUTIVE:             return 266700;
   case DeviceForm::FORM_STATEMENT:             return 215900;
   case DeviceForm::FORM_TABLOID:               return 431800;
   case DeviceForm::FORM_UNIVERSAL:             return 431800;
   case DeviceForm::FORM_FOOLSCAP:              return 330200;
   case DeviceForm::FORM_QUARTO:                return 275082;
   case DeviceForm::FORM_FOLIO:                 return 330000;
   case DeviceForm::FORM_FOLIO_SP:              return 315000;
   case DeviceForm::FORM_ROC_8K:                return 393700;
   case DeviceForm::FORM_ROC_16K:               return 273050;
   case DeviceForm::FORM_GOVERNMENT_LETTER:     return 254000;
   case DeviceForm::FORM_GOVERNMENT_LEGAL:      return 330200;
   case DeviceForm::FORM_EDP:                   return 355600;
   case DeviceForm::FORM_EUROPEAN_EDP:          return 355600;
   case DeviceForm::FORM_PRC_16K:               return 215000;
   case DeviceForm::FORM_PRC_32K:               return 151000;
   case DeviceForm::FORM_DAI_PA_KAI:            return 395000;
   case DeviceForm::FORM_JUURO_KU_KAI:          return 275000;
   case DeviceForm::FORM_PA_KAI:                return 389000;
   case DeviceForm::FORM_WIDE:                  return 279400;
   case DeviceForm::FORM_LETTER_WIDE:           return 337800;
   case DeviceForm::FORM_FOOLSCAP_WIDE:         return 330200;
   case DeviceForm::FORM_A3_WIDE:               return 482600;
   case DeviceForm::FORM_A4_WIDE:               return 355600;
   case DeviceForm::FORM_WIDE_FORMAT:           return 1066800;
   case DeviceForm::FORM_10_X_11:               return 279400;
   case DeviceForm::FORM_11_X_12:               return 304800;
   case DeviceForm::FORM_11_X_15:               return 381000;
   case DeviceForm::FORM_12_X_19:               return 482600;
   case DeviceForm::FORM_15_X_11:               return 279400;
   case DeviceForm::FORM_5_X_7:                 return 177800;
   case DeviceForm::FORM_3_X_5_CARD:            return 127000;
   case DeviceForm::FORM_4_X_6_CARD:            return 152400;
   case DeviceForm::FORM_5_X_8_CARD:            return 203200;
   case DeviceForm::FORM_8_X_10_CARD:           return 254000;
   case DeviceForm::FORM_A6_CARD:               return 148090;
   case DeviceForm::FORM_CARD_148:              return 105000;
   case DeviceForm::FORM_HAGAKI_CARD:           return 148000;
   case DeviceForm::FORM_OUFUKU_CARD:           return 200000;
   case DeviceForm::FORM_POSTCARD:              return 147030;
   case DeviceForm::FORM_INDEX_4_X_6_EXT:       return 203200;
   case DeviceForm::FORM_A2_ENVELOPE:           return 146050;
   case DeviceForm::FORM_C5_ENVELOPE:           return 229000;
   case DeviceForm::FORM_C6_ENVELOPE:           return 162000;
   case DeviceForm::FORM_C7_ENVELOPE:           return 190500;
   case DeviceForm::FORM_C9_ENVELOPE:           return 225400;
   case DeviceForm::FORM_C10_ENVELOPE:          return 241300;
   case DeviceForm::FORM_D5_ENVELOPE:           return 250000;
   case DeviceForm::FORM_DL_ENVELOPE:           return 220000;
   case DeviceForm::FORM_ENVELOPE_132_X_220:    return 220000;
   case DeviceForm::FORM_ENVELOPE_6_X_9:        return 228600;
   case DeviceForm::FORM_ENVELOPE_7_X_9:        return 228600;
   case DeviceForm::FORM_ENVELOPE_6_1_2:        return 92070;
   case DeviceForm::FORM_ENVELOPE_9_X_11:       return 279400;
   case DeviceForm::FORM_ENVELOPE_10_X_13:      return 330200;
   case DeviceForm::FORM_ENVELOPE_10_X_14:      return 355600;
   case DeviceForm::FORM_ENVELOPE_10_X_15:      return 381000;
   case DeviceForm::FORM_MONARCH_ENVELOPE:      return 190500;
   case DeviceForm::FORM_PERSONAL_ENVELOPE:     return 165100;
   case DeviceForm::FORM_NUMBER_9_ENVELOPE:     return 225425;
   case DeviceForm::FORM_NUMBER_10_ENVELOPE:    return 241300;
   case DeviceForm::FORM_NUMBER_11_ENVELOPE:    return 263525;
   case DeviceForm::FORM_NUMBER_12_ENVELOPE:    return 279400;
   case DeviceForm::FORM_NUMBER_14_ENVELOPE:    return 292100;
   case DeviceForm::FORM_ITALIAN_ENVELOPE:      return 230000;
   case DeviceForm::FORM_POSTFIX_ENVELOPE:      return 229000;
   case DeviceForm::FORM_INVITE_ENVELOPE:       return 220000;
   case DeviceForm::FORM_CHOU2_ENVELOPE:        return 146000;
   case DeviceForm::FORM_CHOU3_ENVELOPE:        return 235000;
   case DeviceForm::FORM_CHOU4_ENVELOPE:        return 205000;
   case DeviceForm::FORM_KAHU_ENVELOPE:         return 322100;
   case DeviceForm::FORM_KAKU2_ENVELOPE:        return 332000;
   case DeviceForm::FORM_YOU4_ENVELOPE:         return 235000;
   case DeviceForm::FORM_PRC1_ENVELOPE:         return 165000;
   case DeviceForm::FORM_PRC2_ENVELOPE:         return 176000;
   case DeviceForm::FORM_PRC3_ENVELOPE:         return 176000;
   case DeviceForm::FORM_PRC4_ENVELOPE:         return 208000;
   case DeviceForm::FORM_PRC5_ENVELOPE:         return 220000;
   case DeviceForm::FORM_PRC6_ENVELOPE:         return 320000;
   case DeviceForm::FORM_PRC7_ENVELOPE:         return 230000;
   case DeviceForm::FORM_PRC8_ENVELOPE:         return 309000;
   case DeviceForm::FORM_PRC9_ENVELOPE:         return 324000;
   case DeviceForm::FORM_PRC10_ENVELOPE:        return 458000;
   case DeviceForm::FORM_DISK_LABELS:           return 70000;
   case DeviceForm::FORM_EURO_LABELS:           return 89000;
   case DeviceForm::FORM_SHIPPING_LABELS:       return 101000;
   case DeviceForm::FORM_STANDARD_LABELS_CLEAR: return 89000;
   case DeviceForm::FORM_STANDARD_LABELS_WHITE: return 89000;
   case DeviceForm::FORM_2A0:                   return 1682000;
   case DeviceForm::FORM_4A0:                   return 2378000;
   case DeviceForm::FORM_A0:                    return 1189000;
   case DeviceForm::FORM_A1:                    return 841000;
   case DeviceForm::FORM_A2:                    return 594000;
   case DeviceForm::FORM_A3:                    return 420000;
   case DeviceForm::FORM_A3_EXTRA:              return 445000;
   case DeviceForm::FORM_A4:                    return 297000;
   case DeviceForm::FORM_A4_EXTRA:              return 322300;
   case DeviceForm::FORM_A4_TAB:                return 297000;
   case DeviceForm::FORM_A5:                    return 210000;
   case DeviceForm::FORM_A5_EXTRA:              return 235000;
   case DeviceForm::FORM_A6:                    return 148000;
   case DeviceForm::FORM_A7:                    return 105000;
   case DeviceForm::FORM_A8:                    return 74000;
   case DeviceForm::FORM_A9:                    return 52000;
   case DeviceForm::FORM_A10:                   return 37000;
   case DeviceForm::FORM_B_PLUS:                return 486918;
   case DeviceForm::FORM_B0:                    return 1414000;
   case DeviceForm::FORM_B1:                    return 1000000;
   case DeviceForm::FORM_B2:                    return 707000;
   case DeviceForm::FORM_B3:                    return 500000;
   case DeviceForm::FORM_B4:                    return 353000;
   case DeviceForm::FORM_B5:                    return 250000;
   case DeviceForm::FORM_B5_EXTRA:              return 276000;
   case DeviceForm::FORM_B6:                    return 176000;
   case DeviceForm::FORM_B6_C4:                 return 324000;
   case DeviceForm::FORM_B7:                    return 125000;
   case DeviceForm::FORM_B8:                    return 88000;
   case DeviceForm::FORM_B9:                    return 62000;
   case DeviceForm::FORM_B10:                   return 44000;
   case DeviceForm::FORM_JIS_B0:                return 1456000;
   case DeviceForm::FORM_JIS_B1:                return 1030000;
   case DeviceForm::FORM_JIS_B2:                return 728000;
   case DeviceForm::FORM_JIS_B3:                return 515000;
   case DeviceForm::FORM_JIS_B4:                return 364000;
   case DeviceForm::FORM_JIS_B5:                return 257000;
   case DeviceForm::FORM_JIS_B6:                return 182000;
   case DeviceForm::FORM_JIS_B7:                return 128000;
   case DeviceForm::FORM_JIS_B8:                return 91000;
   case DeviceForm::FORM_JIS_B9:                return 64000;
   case DeviceForm::FORM_JIS_B10:               return 45000;
   case DeviceForm::FORM_JIS_EXEC:              return 330000;
   case DeviceForm::FORM_C0:                    return 1297000;
   case DeviceForm::FORM_C1:                    return 917000;
   case DeviceForm::FORM_C2:                    return 648000;
   case DeviceForm::FORM_C3:                    return 458000;
   case DeviceForm::FORM_C4:                    return 324000;
   case DeviceForm::FORM_C5:                    return 229000;
   case DeviceForm::FORM_C6:                    return 162000;
   case DeviceForm::FORM_C6_C5:                 return 229000;
   case DeviceForm::FORM_C7:                    return 114000;
   case DeviceForm::FORM_C7_C6:                 return 162000;
   case DeviceForm::FORM_C8:                    return 81000;
   case DeviceForm::FORM_C9:                    return 57000;
   case DeviceForm::FORM_C10:                   return 40000;
   case DeviceForm::FORM_E1:                    return 1016000;
   case DeviceForm::FORM_RA0:                   return 1220000;
   case DeviceForm::FORM_RA1:                   return 860000;
   case DeviceForm::FORM_RA2:                   return 610000;
   case DeviceForm::FORM_SRA0:                  return 1280000;
   case DeviceForm::FORM_SRA1:                  return 900000;
   case DeviceForm::FORM_SRA2:                  return 640000;
   case DeviceForm::FORM_US_FANFOLD:            return 279400;
   case DeviceForm::FORM_FANFOLD_1:             return 279400;
   case DeviceForm::FORM_FANFOLD_2:             return 215900;
   case DeviceForm::FORM_FANFOLD_3:             return 279400;
   case DeviceForm::FORM_FANFOLD_4:             return 304800;
   case DeviceForm::FORM_FANFOLD_5:             return 279400;
   case DeviceForm::FORM_GERMAN_LEGAL_FANFOLD:  return 330200;
   case DeviceForm::FORM_EUROPEAN_FANFOLD:      return 304800;
   case DeviceForm::FORM_PANORAMIC:             return 594000;
   case DeviceForm::FORM_PHOTO_100_150:         return 150000;
   case DeviceForm::FORM_PHOTO_200_300:         return 300000;
   case DeviceForm::FORM_PHOTO_4_6:             return 152400;
   case DeviceForm::FORM_PLOTTER_SIZE_A:        return 279400;
   case DeviceForm::FORM_PLOTTER_SIZE_B:        return 431800;
   case DeviceForm::FORM_PLOTTER_SIZE_C:        return 558800;
   case DeviceForm::FORM_PLOTTER_SIZE_D:        return 863600;
   case DeviceForm::FORM_PLOTTER_SIZE_E:        return 1117600;
   case DeviceForm::FORM_PLOTTER_SIZE_F:        return 1727200;
   case DeviceForm::FORM_US_A_ARCHITECTURAL:    return 304800;
   case DeviceForm::FORM_US_B_ARCHITECTURAL:    return 457200;
   case DeviceForm::FORM_US_C_ARCHITECTURAL:    return 609600;
   case DeviceForm::FORM_US_D_ARCHITECTURAL:    return 914400;
   case DeviceForm::FORM_US_E_ARCHITECTURAL:    return 1219200;
   case DeviceForm::FORM_SUPER_A3_B:            return 483020;
   case DeviceForm::FORM_SUPER_A:               return 355600;
   case DeviceForm::FORM_SUPER_B:               return 482600;
   }

   return 0;
}

int
main (int argc, char *argv[])
{
   int iFailed = 0;

   if (2 != argc)
   {
      cerr << "Usage: " << argv[0] << " [ printer_library_name | all ]" << endl;
      return 1;
   }

   if (0 == strcasecmp (argv[1], "all"))
   {
      Enumeration *pEnum     = Omni::listDevices ();

      while (pEnum->hasMoreElements ())
      {
         char *pszLibName = (char *)pEnum->nextElement ();

         cout << "Checking: " << pszLibName << endl;

         iFailed |= handleDevice (pszLibName);
      }

      delete pEnum;
   }
   else
   {
      char achLibDeviceName[512];  // @TBD

      if (0 != strncasecmp (argv[1], "lib", 3))
      {
         sprintf (achLibDeviceName, "lib%s.so", argv[1]);
      }
      else
      {
         strcpy (achLibDeviceName, argv[1]);
      }

      iFailed |= handleDevice (achLibDeviceName);
   }

   return iFailed;
}
